#include "Control.h"
#include <qstrlist.h>
#include <time.h>
#include <qregexp.h>
#include <qfiledialog.h>
#include "OneLiner.h"
#include <qprogressdialog.h>
#include <qvalidator.h>

#define Inherited ControlData

// used to label x-axis in plot
char *labfn(float val,int isy) {
static char r[30];

sprintf(r,"%1.2f",1/val);
return r;
}

/*class DirVal: public QValidator {
public:
DirVal(QWidget *p=NULL, const char *n=NULL):QValidator(p,n) { }
virtual QValidator::State validate ( QString &s, int &i ) const {
QDir d(s);
if (d.exists()) return Valid;
return Acceptable;
}
};*/

Control::Control(QWidget* parent,const char* name):Inherited( parent, name ) {
	setCaption( "EMAN" );

	QPopupMenu *m1=new QPopupMenu(NULL,"File");
	menu->insertItem("File",m1,100);
//	m1->insertItem("Add Directory",this,SLOT(addDir()),0);
//	m1->insertItem("Reload dir list",this,SLOT(procDList()),0);
//	m1->insertSeparator();
	m1->insertItem("Exit",this,SLOT(quit()),0);

	m1=new QPopupMenu(NULL,"Analysis");
	menu->insertItem("Analysis",m1,100);
	m1->insertItem("Convergence",this,SLOT(doConverge()),0);
	
	inspector->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
	inspector->setLineWidth( 2 );
	inspector->addWidget(&ihist,0);
	inspector->addWidget(&idir,1);
	inspector->addWidget(&ifile,2);
	inspector->addWidget(&icls,3);
	inspector->addWidget(&itri,4);
	inspector->raiseWidget(1);
//inspector->setBackgroundMode(NoBackground);

	files->addColumn("");
	files->addColumn("File");
	files->setSorting(1,TRUE);
	files->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
	files->setLineWidth( 2 );
	files->setRootIsDecorated(FALSE);
	files->setAllColumnsShowFocus(TRUE);
	 
	history->addColumn("Program");
	history->addColumn("Time");
	history->addColumn("Run Time");
	history->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
	history->setLineWidth( 2 );
	history->setRootIsDecorated(TRUE);
	history->setAllColumnsShowFocus(TRUE);
	history->setSorting(1);

	histlist.setAutoDelete( TRUE );

//	dirsel->setValidator(new DirVal(dirsel));

	connect(files,SIGNAL(selectionChanged(QListViewItem *)),
		this,SLOT(fileSel(QListViewItem *)));

	connect(history,SIGNAL(selectionChanged(QListViewItem *)),
		this,SLOT(histSel(QListViewItem *)));

	connect(dirsel,SIGNAL(activated(const QString&)),this,SLOT(dispDir(const QString&)));
	connect(dirbrow,SIGNAL(clicked()),this,SLOT(dirBrow()));

	connect(&ifile,SIGNAL(showcls(QString)),this,SLOT(dispCls(QString)));
	connect(&ifile,SIGNAL(showtri(QString)),this,SLOT(dispTri(QString)));

	connect(inspsel,SIGNAL(selected(int)),SLOT(showInsp(int)));
//setBackgroundMode(NoBackground);

	convaxes=NULL;
	convdata.setAutoDelete(TRUE);
	convplot.setAutoDelete(TRUE);

	dispDir(".");
	
	dtime.setTime_t(0);
	tmrct=-1;
	startTimer(100);
}

Control::~Control() {
}

void Control::showInsp(int i) {
//printf("Showinsp %d\n",i);

curinsp=i;
switch (curinsp) {
case 0:
	break;
case 1:
	tmrct=-1;
//	dirUpdate();
	break;
case 2:
	ifile.setFile(curfile);
	break;
case 3:
	icls.setFile(curfile);
	break;
case 4:
	itri.setFile(curfile);
	break;
}
inspector->raiseWidget(i);
}

void Control::dirBrow() {
QString d=QFileDialog::getExistingDirectory(getenv("PWD"));

if (d.isNull() || d.isEmpty()) return;
dirsel->insertItem(d,0);
dirsel->setCurrentItem(0);
dispDir(d);
}

void Control::dispDir(const QString &u) {
//printf("dispDir(%s)\n",(const char *)u);
QString v(u);
QString filt("*");
if (v.left(2)==". " || v==".") v=QDir::currentDirPath();
if (v.left(2)=="~/") v=QDir::homeDirPath()+u.mid(1);
QFileInfo fi(v);
if (!fi.isDir()) {
	int p=v.findRev('/');
	filt=v.mid(p+1);
	v=v.left(p);
}
QDir dir(v,filt);
if (!dir.exists()) { v=curdir; dirsel->clearEdit(); }
else dirsel->insertItem(u,0);

curdir=v;
curfilt=filt;
if (curdir.left(2)==". ") curdir=".";
dirfi.setFile(curdir);

inspsel->setTabEnabled(1,TRUE);

inspsel->setTabEnabled(2,FALSE);
inspsel->setTabEnabled(3,FALSE);
inspsel->setTabEnabled(4,FALSE);
showTab(1);
tmrct=-1;
}

void Control::showTab(int i) {
//printf("Showtab %d\n",i);
if (inspsel->currentTab()!=i) inspsel->setCurrentTab(i);
//else showInsp(i);
showInsp(i);

}

void Control::dirUpdate() {
//printf("dirUpdate '%s'\n",(const char *)curdir);
int i;
QDir dir(curdir,curfilt);

files->clear();
dir.setFilter(QDir::Dirs|QDir::Readable);
QStringList dlist=dir.entryList();
for (i=0; i<dlist.count(); i++) {
	new QListViewItem(files,"/ ",dlist[i]);
}

dir.setFilter(QDir::Files|QDir::Readable);
QStringList list=dir.entryList();
for (i=0; i<list.count(); i++) {
	if ((*list.at(i)).right(4)==".hed") continue;
	new QListViewItem(files,"  ",list[i]);
}

curfile=(const char *)0;
dispHist();

idir.setDir(curdir);
inspsel->setTabEnabled(0,FALSE);
inspsel->setTabEnabled(2,FALSE);
inspsel->setTabEnabled(3,FALSE);
inspsel->setTabEnabled(4,FALSE);
}

void Control::histSel(QListViewItem *qlv) {
if (!qlv) return;

int i;

i=qlvlist.find(qlv);
if (i<0) return;
ihist.setHist(histlist.at(i));

inspsel->setTabEnabled(0,TRUE);
showTab(0);
}

void Control::fileSel(QListViewItem *qlv) {
if (!qlv) return;

EMData tmp;

//printf("fileSel(%s)\n",(const char *)qlv->text(0));
//printf("fileSel(%s)\n",(const char *)qlv->text(1));

curfile=curdir+"/"+qlv->text(1);

QFileInfo fi(curfile);
if (fi.isDir()) {
	if (curfile.right(2)=="/.") curfile=curdir;
	if (curfile.right(3)=="/..") curfile=curdir.left(curdir.findRev('/'));
	dispDir(curfile);
	return;
}

int r=tmp.readImage(curfile,-1,1);	// read header

printf("%d %d %d %d\n",tmp.xSize(),tmp.ySize(),tmp.zSize(),r);

inspsel->setTabEnabled(2,TRUE);
if (tmp.xSize()>1 && tmp.ySize()>1 && tmp.zSize()==1 && !r) {
	inspsel->setTabEnabled(3,TRUE);
	inspsel->setTabEnabled(4,TRUE);
	if (inspsel->currentTab()<2) showTab(2);
	else showTab(inspsel->currentTab());
}
else {
	inspsel->setTabEnabled(3,FALSE);
	inspsel->setTabEnabled(4,FALSE);
	showTab(2);
}
dispHist();

}

void Control::dispFile(QString file) {
//printf("dispFile(%s)\n",file);
curfile=file;

showTab(2);
}

void Control::dispCls(QString file) {
curfile=file;

showTab(3);
}

void Control::dispTri(QString file) {
curfile=file;

showTab(4);
}

void Control::dispHist() {
QString file,file2;

//printf("dispHist '%s','%s'\n",(const char *)curdir,(const char *)curfile);
if (curfile.isNull()||curfile.findRev('/')==-1) file=curfile;
else file=curfile.mid(curfile.findRev('/')+1);
file2=file;
file2.replace(QRegExp(".img"), ".hed");

QFile in(curdir+"/.emanlog");

char t[1024];
EMHist *hi,*hic=NULL,*hii;
QListViewItem *qlvi,*pi;

history->clear();
histlist.clear();
qlvlist.clear();

inspsel->setTabEnabled(0,FALSE);

if (in.open(IO_ReadOnly)) {
while (in.readLine(t,1023)>0) {
	if (!t[0] || !isdigit(t[0])) continue;		// deal with blank lines
	hi=new EMHist(t);
	hi->setDir(curdir);

	if (hi->type==LOG_END) {
		for (hii=histlist.last(); hii!=NULL; hii=histlist.prev()) {
			if (hii->id==hi->id) break;
		}
		if (hii) {
			hii->setFinTime(hi->etime); 
			qlvlist.at(histlist.find(hii))->setText(2,hii->rtstr);
		}
	}
	if (!file.isNull()) {
		if (hi->type==LOG_BEGIN) {
			if (hic) delete hic;
			hic=hi;
		} else if ((file==hi->file || file2==hi->file) && hic){
			qlvi=new QListViewItem(history,hic->command,hic->tstr,hic->rtstr);
			qlvlist.append(qlvi);
			histlist.append(hic);
			hic=NULL;
			delete hi;
		}
		else delete hi;
	}
	else {
		if (hi->type==LOG_BEGIN) {
			for (hii=histlist.first(); hii!=NULL; hii=histlist.next()) {
				if (hii->id==hi->ppid) break;
			}
			if (hii==NULL) qlvi=new QListViewItem(history,hi->command,hi->tstr,hi->rtstr);
			else qlvi=new QListViewItem(qlvlist.at(histlist.find(hii)),hi->command,hi->tstr,hi->rtstr);

			qlvlist.append(qlvi);
			histlist.append(hi);
		}
		else delete hi;
	}

}
}
if (hic) delete hic;

}

void Control::quit() {
exit(0);
}

void Control::doConverge() {
int i,j,n,m=0,eo=0;
QString path(dirsel->currentText()),file1,file2,command;

if (path.at(1)==' ') path=".";

for (i=0; i<100; i++) {
	file1.sprintf("%s/threed.%0da.mrc",(const char *)path,i);
	if (!QFile::exists(file1)) break;
}
m=i-1;
if (m<=0) { printf("No data to analyze '%s'\n",(const char *)file1); return; }

QProgressDialog progress( "Calculating", "Abort", m+1, NULL, 0 ,TRUE );
progress.setProgress( 0 );

for (i=0; i<m; i++) {
	file1.sprintf("threed.%0da.mrc",i);
	file2.sprintf("threed.%0da.mrc",i+1);

	command.sprintf("cd %s; proc3d %s %s fsc=p%02d%02d",(const char *)path,(const char *)file1,(const char *)file2,i,i+1);
	system(command);
//	puts(command.data());

	progress.setProgress( i+1 );
	if ( progress.wasCancelled() ) break;
}

file1.sprintf("%s/threed.te.mrc",(const char *)path);
file2.sprintf("%s/threed.to.mrc",(const char *)path);
if (QFile::exists(file1) && QFile::exists(file2)) {
	command.sprintf("cd %s; proc3d threed.te.mrc threed.to.mrc fsc=peo",(const char *)path);
	system(command);
	eo=1;
}
progress.setProgress(m+1);

QString sapix;
sapix=OneLiner::getOneLine("A/pixel for raw data","");
float apix=sapix.toFloat();
if (apix<=0) apix=1.0;

Axes2d *axes=NULL;
DataSet *d;
LSPlot *p;
float *f;

//if (axes) { delete axes; axes=convaxes=NULL; }

if (!axes) {
	axes=new Axes2d;
	axes->resize(500,400);
	axes->setBackgroundMode( QWidget::NoBackground );
	axes->setAction(LeftButton,D2_ACT_XVALUE|D2_ACT_YVALUE|D2_ACT_MPOS);
//	axes->setAction(MidButton,D2_ACT_YLOG);
	axes->setAction(LeftButton|ControlButton,D2_ACT_LEGEND);
	axes->setAction(RightButton,D2_ACT_ZOOM);
	axes->setOptions(D2_OPT_AUTOSCALE|D2_OPT_FRAME|D2_OPT_SHOWY0,D2_OPT_SCALE|D2_OPT_TIC|D2_OPT_CSCA|D2_OPT_AXIS,
		D2_OPT_SCALE|D2_OPT_TIC);
	axes->setlabfn(labfn);
	axes->setXLabel("s (1/A)");
	axes->setYLabel("Fourier Shell Correlation");
	convaxes=axes;
}
else {
	convdata.clear();
	convplot.clear();
}
axes->show();

int lineparm[24][4] = { 0,0,0,0, 0,0,0,1, 0,0,0,3, 0,0,0,2,
						0,0,240,0, 0,0,240,1, 0,0,240,3, 0,0,240,2,
						240,0,0,0, 240,0,0,1, 240,0,0,3, 240,0,0,2,
						200,0,200,0, 200,0,200,1, 200,0,200,3, 200,0,200,2,
						0,240,0,0, 0,240,0,1, 0,240,0,3, 0,240,0,2,
						0,200,200,0, 0,200,200,1, 0,200,200,3, 0,200,200,2 };

for (i=0; i<m; i++) {
	d=new DataSet;
	p=new LSPlot;

	file1.sprintf("%s/p%02d%02d",(const char *)path,i,i+1);
	d->readFile(file1);
	f=d->getColumn(0)->getData();
	n=d->getColumn(0)->getSize();
	for (j=0; j<n; j++) f[j]*=1.0/(apix*2*n);
	d->getColumn(0)->setData();

	p->setData(d);
	p->setCols(0,1);
	file2.sprintf("%d<->%d",i,i+1);
	p->setLabel(file2);
	p->setColor(lineparm[i%24][0],lineparm[i%24][1],lineparm[i%24][2]);
	p->setLine(lineparm[i%24][3],1);
	p->setSym(SYM_PLUS,4);

	convdata.append(d);
	convplot.append(p);

	axes->addPlot(p);
}

if (eo) {
	d=new DataSet;
	p=new LSPlot;

	file1.sprintf("%s/peo",(const char *)path);
	d->readFile(file1);
	f=d->getColumn(0)->getData();
	n=d->getColumn(0)->getSize();
	for (j=0; j<n; j++) f[j]*=1.0/(apix*2*n);
	d->getColumn(0)->setData();

	p->setData(d);
	p->setCols(0,1);
	p->setLabel("EO-test");
	p->setColor(0,0,0);
	p->setLine(0,3);
	p->setSym(SYM_PLUS,5);

	convdata.append(d);
	convplot.append(p);

	axes->addPlot(p);

}

}

void Control::timerEvent(QTimerEvent *e) {
if (tmrct<0) {
	tmrct=0;
	dirUpdate();
	dirfi.refresh();
	dtime=dirfi.lastModified();
}

tmrct++;
if (tmrct%40!=0 || curinsp!=1) return;

dirfi.refresh();
if (dirfi.lastModified()>dtime) {
	dtime=dirfi.lastModified();
	dirUpdate();
}


}
