#include <GL/gl.h>
#include "MatrixView.h"
#include <stdio.h>
#include "util.h"
#include "EMData.h"

MatrixView::MatrixView( const QGLFormat & format, QWidget * parent, const char * name, const QGLWidget * shareWidget ,WFlags f ) :
	QGLWidget(format,parent,name,shareWidget,f) {
points = NULL;
matrix = NULL;
spherequad = NULL;
sel0= -1;
sel1= -1;
selalt=-1;
pscale=1.0;
dscale=1.0;
timer=new QTimer(this);
connect(timer,SIGNAL(timeout()),this,SLOT(timeout()));
timer->start(50);
modes=0;
imagefile=NULL;
}

MatrixView::~MatrixView() {
if (spherequad) gluDeleteQuadric(spherequad);
if (points) free(points);
if (matrix) free(matrix);
}

void MatrixView::initializeGL() {
// cube definition
glNewList(MV_CUBE,GL_COMPILE);
glDisable(GL_LIGHTING);
glBegin(GL_LINE_STRIP);
glVertex3i(-1,-1,-1);
glVertex3i( 1,-1,-1);
glVertex3i( 1, 1,-1);
glVertex3i(-1, 1,-1);
glVertex3i(-1,-1,-1);
glVertex3i(-1,-1, 1);
glVertex3i( 1,-1, 1);
glVertex3i( 1, 1, 1);
glVertex3i(-1, 1, 1);
glVertex3i(-1,-1, 1);
glEnd();
glBegin(GL_LINES);
glVertex3i(-1, 1,-1);
glVertex3i(-1, 1, 1);
glVertex3i( 1,-1,-1);
glVertex3i( 1,-1, 1);
glVertex3i( 1, 1,-1);
glVertex3i( 1, 1, 1);
glEnd();
glEnable(GL_LIGHTING);
glEndList();

// Small sphere for 'points' in the matrix
spherequad=gluNewQuadric();
gluQuadricDrawStyle(spherequad,GLU_FILL);
setPointSize(.01);

float amb[] = { .3, .3, .3, 1.0 };
float diff[] = { .9, .9, .9, 1.0 };
float pos[] = { -2., 4., 4., 1.0 };

//glLightfv(GL_LIGHT0,GL_AMBIENT,amb);
//glLightfv(GL_LIGHT0,GL_DIFFUSE,diff);
glEnable(GL_COLOR_MATERIAL);
glLightModelfv(GL_LIGHT_MODEL_AMBIENT,amb);
glLightfv(GL_LIGHT0,GL_POSITION,pos);
glEnable(GL_LIGHT0);
glEnable(GL_LIGHTING);
glEnable(GL_CULL_FACE);
glEnable(GL_DEPTH_TEST);
}

void MatrixView::resizeGL( int width, int height ) {
GLfloat w = (float) width / (float) height;
GLfloat h = 1.0;
 
    glViewport( 0, 0, width, height );
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glFrustum( -w, w, -h, h, 5.0, 40.0 );
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glTranslatef( 0.0, 0.0, -7.0 );

}

void MatrixView::setPointSize(float s) {
makeCurrent();
glNewList(MV_POINT,GL_COMPILE);
gluSphere(spherequad,s,6,6);
glEndList();
}

void MatrixView::setGeom() {
glRotatef(angle.phi()*180./PI,0,1.0,0);
glRotatef(angle.alt()*180./PI,1.0,0,0);
glRotatef(angle.az()*180./PI,0,1.0,0);
glScalef(dscale,dscale,dscale);
glTranslatef(-curpos.X(),-curpos.Y(),-curpos.Z());
}

void MatrixView::paintGL() {
//	printf("Paint\n");
int i;
glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

glPushMatrix();

setGeom();

// The bounding cube
glColor3f(1.0,1.0,1.0);
glCallList(MV_CUBE);


// The 'tagged' points
glColor3f(.3,.3,.9);

if (sel0>=0) {
	// hilighted point
	glPushMatrix();
	glTranslatef(points[sel0*DIM],points[sel0*DIM+1],points[sel0*DIM+2]);
	glCallList(MV_POINT);
	glPopMatrix();

}

if (sel1>=0) {
	// hilighted point
	glPushMatrix();
	glTranslatef(points[sel1*DIM],points[sel1*DIM+1],points[sel1*DIM+2]);
	glCallList(MV_POINT);
	glPopMatrix();

}

// The points
glCallList(MV_POINTS);

// lines between points
if (sel0>=0 && matrix) {
	glDisable(GL_LIGHTING);
	glBegin(GL_LINES);
	for (i=0; i<npoints; i++) {
		if (i==sel0) continue;
		if (sel1>=0 && sel1!=i) continue;
		float d1=sqrt(hypot3(points[i*DIM]-points[sel0*DIM],points[i*DIM+1]-points[sel0*DIM+1],points[i*DIM+2]-points[sel0*DIM+2]))/pscale;
		float d2=(matrix[i*npoints+sel0]+matrix[sel0*npoints+i])/2.0;
		d1=(d1-d2)*5.0;	// should be reasonable scaling for most situations
		if (d1>d2) glColor3f(1.0-d1,1.0,1.0-d1);		// green
		else glColor3f(1.0,1.0+d1,1.0+d1);
		glVertex3f(points[i*DIM],points[i*DIM+1],points[i*DIM+2]);
		// scale line length to 'true' length
		if (sel1>=0) {
			float v[3],s;
			v[0]=points[sel0*DIM]-points[i*DIM];
			v[1]=points[sel0*DIM+1]-points[i*DIM+1];
			v[2]=points[sel0*DIM+2]-points[i*DIM+2];
			s=sqrt(hypot3(v[0],v[1],v[2]));
			s=matrix[sel0*npoints+i]*pscale/s;
			glVertex3f(v[0]*s+points[i*DIM],v[1]*s+points[i*DIM+1],v[2]*s+points[i*DIM+2]);
		}
		else glVertex3f(points[sel0*DIM],points[sel0*DIM+1],points[sel0*DIM+2]);
		// this version just displays the line all the way
//		glVertex3f(points[sel0*DIM],points[sel0*DIM+1],points[sel0*DIM+2]);
	}
	glEnd();
	glEnable(GL_LIGHTING);
}

// isoaltitude lines/planes
if (modes&(MV_ALTLINE|MV_ALTPLANE)) {
	int flip=1;
	int pln=0;
	int bg=0;
	glDisable(GL_CULL_FACE);
	glDisable(GL_LIGHTING);
	glColor3f(.9,.2,.9);
	glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
	if (modes&MV_ALTLINE) {  glBegin(GL_LINE_STRIP); }	// note: GL_LINE_LOOP buggy, use strip with a hack
	else glBegin(GL_TRIANGLE_FAN);
	for (i=0; i<npoints; i++) {
		if (i&&points[i*DIM+4]!=points[(i-1)*DIM+4]) { 
			pln++;
			if (selalt==-1 || selalt==pln) {
				glVertex3f(points[bg*DIM],points[bg*DIM+1],points[bg*DIM+2]);
				glEnd(); 
				if (modes&MV_ALTLINE) glBegin(GL_LINE_STRIP); 
				else glBegin(GL_TRIANGLE_FAN);
				flip^=1;
				if (flip) glColor3f(.9,.2,.9);
				else glColor3f(.9,.9,.2);
				bg=i;
			}
		}
		if (selalt==-1 || selalt==pln) {
			glVertex3f(points[i*DIM],points[i*DIM+1],points[i*DIM+2]);
		}
	}
	glVertex3f(points[bg*DIM],points[bg*DIM+1],points[bg*DIM+2]);
	glEnd();
	
	glColor3f(.2,.2,.9);
	glBegin(GL_LINE_STRIP);
	for (i=0; i<npoints; i++) {
		if (i&&points[i*DIM+4]!=points[(i-1)*DIM+4]) { 
			glVertex3f(points[i*DIM],points[i*DIM+1],points[i*DIM+2]);
		}
	}
	glEnd();
	glEnable(GL_CULL_FACE);
	glEnable(GL_LIGHTING);
}

glPopMatrix();

// Asym triangle point viewer 
glPushMatrix();
if (modes&MV_TRI) {
	glTranslatef(-.7,.42,0);
	glScalef(.75,.75,.75);
	glBegin(GL_LINE_LOOP);
	glColor3f(1.,1.,1.);
	glVertex3f(-.5,0,1.9);
	glVertex3f(.5,0,1.9);
	glVertex3f(0,.866,1.9);
	glEnd();
	glBegin(GL_TRIANGLES);
	glColor3f(.05,.05,.05);
	glVertex3f(-.5,0,1.9);
	glVertex3f(.5,0,1.9);
	glVertex3f(0,.866,1.9);
	glEnd();
	
	glColor3f(.4,.4,.4);
	for (i=0; i<npoints; i++) {
		float y=1.0-(points[i*DIM+4]-alt0)/(alt1-alt0);
		float x=((points[i*DIM+5]-az0)/(az1-az0)-.5)*(1.0-y);
		y*=.866;
		glPushMatrix();
		glTranslatef(x,y,1.9);
		if (i==sel0||i==sel1) {
			glColor3f(.3,.9,.3);
			glCallList(MV_POINT);		
			glColor3f(.4,.4,.4);
		}
		else glCallList(MV_POINT);		
		glPopMatrix();
	}
}
glPopMatrix();

glPushMatrix();
if (sel0>=0) {
	// image
	EMData d1;
	if (imagefile && (modes&MV_IMAGE)) {
		d1.readImage(imagefile,sel0);
		d1.normalize();
		float *f = d1.getData();
		for (i=0; i<d1.xSize()*d1.ySize(); i++) f[i]=f[i]*.1+.2;
		glRasterPos3f(-.4,-1.,1.9);
		glDrawPixels(d1.xSize(),d1.ySize(),GL_LUMINANCE,GL_FLOAT,f);
		d1.doneData();
	}

	if (sel1>=0) {
		// image
		if (imagefile && (modes&MV_IMAGE)) {
			EMData d,*d2;
			d.readImage(imagefile,sel1);
			d2=d.RTFAlign(&d1);
			d2->normalize();
			float *f = d2->getData();
			for (i=0; i<d2->xSize()*d2->ySize(); i++) f[i]=f[i]*.1+.2;
			glRasterPos3f(.2,-1.,1.9);
			glDrawPixels(d2->xSize(),d2->ySize(),GL_LUMINANCE,GL_FLOAT,f);
			d2->doneData();
			delete d2;
		}
	}
}

glPopMatrix();

}

void MatrixView::setPoints(int n,float *Points) {
int i;
points=Points;
npoints=n;

// Find the extent of each axis
float x0=10.0,x1=-10.0,y0=10.0,y1=-10.0,z0=10.0,z1=-10.0,ma;
for (i=0; i<n; i++) {
	if (points[i*DIM]<x0)   x0=points[i*DIM];
	if (points[i*DIM]>x1)   x1=points[i*DIM];
	if (points[i*DIM+1]<y0) y0=points[i*DIM+1];
	if (points[i*DIM+1]>y1) y1=points[i*DIM+1];
	if (points[i*DIM+2]<z0) z0=points[i*DIM+2];
	if (points[i*DIM+2]>z1) z1=points[i*DIM+2];
}
ma=x1-x0;
if (y1-y0>ma) ma=y1-y0;
if (z1-z0>ma) ma=z1-z0;

printf("(%f : %f) (%f : %f) (%f : %f)  %f\n",x0,x1,y0,y1,z0,z1,ma);

// Scale so the data fills the box as much as possible
// without distortion
ma/=2.0;
for (i=0; i<n; i++) {
	points[i*DIM]=(points[i*DIM]-(x1+x0)/2.0)/ma;
	points[i*DIM+1]=(points[i*DIM+1]-(y1+y0)/2.0)/ma;
	points[i*DIM+2]=(points[i*DIM+2]-(z1+z0)/2.0)/ma;
}
pscale=1.0/ma;

// Limits of errors
float max=0;
for (i=0; i<n; i++) if (points[i*DIM+3]>max) max=points[i*DIM+3];

// Display list for points
glNewList(MV_POINTS,GL_COMPILE);
for (i=0; i<n; i++) {
	glPushMatrix();
	glTranslatef(points[i*DIM],points[i*DIM+1],points[i*DIM+2]);
	float t=points[i*DIM+3];
	if (t>max/2) glColor3f(1.0,1.0-2*points[i*DIM+3]/max,1.0-2*points[i*DIM+3]/max);
	else glColor3f(2.0*points[i*DIM+3]/max,1.0,2.0*points[i*DIM+3]/max);	
	glCallList(MV_POINT);
	glPopMatrix();
}
glEndList();

// lines connecting the points
glNewList(MV_CONNECT,GL_COMPILE);
glDisable(GL_LIGHTING);
glBegin(GL_LINE_STRIP);
for (i=0; i<n; i++) glVertex3f(points[i*DIM],points[i*DIM+1],points[i*DIM+2]);
glEnd();
glEnable(GL_LIGHTING);
glEndList();

// find the extent of the Euler angles
alt0=points[4];
alt1=points[4];
az0=points[5];
az1=points[5];
for (i=1; i<npoints; i++) {
	if (alt0>points[i*DIM+4]) alt0=points[i*DIM+4];
	if (alt1<points[i*DIM+4]) alt1=points[i*DIM+4];
	if (az0>points[i*DIM+5]) az0=points[i*DIM+5];
	if (az1<points[i*DIM+5]) az1=points[i*DIM+5];
}


}

#define ANIMSTEP	.05
void MatrixView::timeout() {
if (modes&MV_CYCLESEL) {
	if (sel0<0||sel0>=npoints) { sel0=0; sel1=1; }
	else { sel0++; sel1++; }
	update();
	return;
}

if (destpos==curpos) return;

if ((destpos-curpos).length()<ANIMSTEP) { curpos=destpos; update(); return; }
Vect step = destpos - lastpos;
step.normalize();
curpos=curpos+step*ANIMSTEP;
update();
return;
}

void MatrixView::mousePressEvent ( QMouseEvent *e ) {
mdx=e->x();
mdy=e->y();


// Select a point with the cursor
if (e->button()&LeftButton) {
	GLdouble model[16];
	GLdouble proj[16];
	GLint view[4];
	GLdouble x,y,z;
	int i,bestp;
	float bestd=999.0;

	modes&= ~MV_CYCLESEL;		// stop spinning if we have a manual point selection

	makeCurrent();
	glPushMatrix();
	setGeom();
	
	glGetDoublev(GL_MODELVIEW_MATRIX,model);
	glGetDoublev(GL_PROJECTION_MATRIX,proj);
	glGetIntegerv(GL_VIEWPORT,view);

	for (i=0; i<npoints; i++) {
		gluProject(points[i*DIM],points[i*DIM+1],points[i*DIM+2],model,proj,view,&x,&y,&z);
		y=height()-y;
		if (hypot(x-mdx,y-mdy)<bestd) { bestd=hypot(x-mdx,y-mdy); bestp=i; }
	}
	
	if (bestd>10.0) {
		if (sel0==-1) destpos.setXYZ(0,0,0);
		sel1=sel0=-1;
	}
	else {
		if (sel0>=0) sel1=bestp;
		else sel0=bestp;
	}
//	printf("Select %d, %f\n",bestp,bestd);
	glPopMatrix();

	lastpos=curpos;
	if (sel0>=0) destpos.setXYZ(points[sel0*DIM],points[sel0*DIM+1],points[sel0*DIM+2]);

	update();
}
if (sel0==sel1) sel1= -1;

if (matrix && sel0>=0 && sel1>=0) {
	float d1=sqrt(hypot3(points[sel1*DIM]-points[sel0*DIM],points[sel1*DIM+1]-points[sel0*DIM+1],points[sel1*DIM+2]-points[sel0*DIM+2]))/pscale;
	float d2=(matrix[sel1*npoints+sel0]+matrix[sel0*npoints+sel1])/2.0;
	printf("%d - %d.\t%f mx - %f 3d\t%f\n",sel0,sel1,d2,d1,fabs(d1-d2));
}

}

void MatrixView::mouseReleaseEvent ( QMouseEvent *e ) {

}

void MatrixView::mouseMoveEvent ( QMouseEvent *e ) {
int x,y;

x=e->x();
y=e->y();

if (e->state()&RightButton) {
	angle.setAngle(angle.alt()+(y-mdy)/80.0,angle.az()+(x-mdx)/80.0,angle.phi());
	update();
	mdy=y;
	mdx=x;
	return;
}
if (e->state()&MidButton) {
	dscale-=(y-mdy)/50.0;
	if (dscale<.8) dscale=.8;
//	if (dscale>2.0) setPointSize(.005);
//	else setPointSize(.01);
	update();
	mdy=y;
	mdx=x;
	return;
}

}

void MatrixView::keyPressEvent ( QKeyEvent *e ) {
switch(e->ascii()) {
case 'l': modes^=MV_ALTLINE; selalt= -1; break;
case 't': modes^=MV_TRI; break;
case 'i': modes^=MV_IMAGE; break;
case 'c': modes^=MV_CYCLESEL; break;
case 'p': modes^=MV_ALTPLANE; selalt= -1; break;
case '+':
case '=': selalt++; break;
case '-': selalt--; break;
}
update();
}
