// EMDataC.C - Steve Ludtke  11/27/2000
// More stuff for the bloated EMData object
// These routines are mainly for treating EMData objects as
// vectors and matrices. Inefficient, but functional.
//
// $Id: EMDataC.C,v 1.37 2007/01/22 23:31:18 gtang Exp $
//
#include <stdio.h>
#include <gsl/gsl_multimin.h>

#ifndef _WIN32
 #include <unistd.h>
#else
 #include "UnixToWin.h"
#endif

#ifndef twoPI
#define twoPI 6.2831853072F
#endif

#ifndef halfPI
#define halfPI 1.5707963268F
#endif


#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
//#include <list.h>
//#include "List.h"
#include "mrc.h"
#include "imagic.h"
#include "emim.h"
#include "gatan.h"
#include "util.h"
#include "EMData.h"
#include "Vect.h"
#include "crossCommonLine.h"

void EMData::cutSlice(EMData *map,float DZ,Euler *ort,int interp,float DX,float DY) {
float *sdata = map->getDataRO();
float *ddata = getData();
Euler Ort(0,0,0);
if (!ort) ort=&Ort;
float *mx = ort->mat();

int Mx=map->xSize();
int My=map->ySize();
int Mz=map->zSize();

int x,y,x2,y2;
float xx,yy,zz;
for (y=0; y<ySize(); y++) {
	y2=y-ySize()/2-DY;
	for (x=0; x<xSize(); x++) {
		x2=x-xSize()/2-DX;
				
		xx=x2*mx[0]+y2*mx[1]+DZ*mx[2]+Mx/2;
		yy=x2*mx[3]+y2*mx[4]+DZ*mx[5]+My/2;
		zz=x2*mx[6]+y2*mx[7]+DZ*mx[8]+Mz/2;

		if (xx<0||yy<0||zz<0 || xx>Mx-2||yy>My-2||zz>Mz-2) {
			ddata[x+y*xSize()]=0;
			continue;
		}

		float t=xx-floor(xx);
		float u=yy-floor(yy);
		float v=zz-floor(zz);
		
		if (interp) {
			int k=(int)(floor(xx)+floor(yy)*Mx+floor(zz)*Mx*My);

			ddata[x+y*xSize()]=trilin(sdata[k],sdata[k+1],sdata[k+Mx],
				sdata[k+Mx+1],sdata[k+Mx*My],sdata[k+1+Mx*My],sdata[k+Mx+Mx*My],
				sdata[k+1+Mx+Mx*My],t,u,v);
		}
		else {
			int k=(int)(round(xx)+round(yy)*Mx+round(zz)*Mx*My);
			ddata[x+y*xSize()]=sdata[k];
		}
	}
}

doneData();
map->doneData();

}

void EMData::uncutSlice(EMData *map,float DZ,Euler *ort,float DX,float DY) {
float *ddata = map->getDataRO();
float *sdata = getData();
Euler Ort(0,0,0);
if (!ort) ort=&Ort;
float *mx = ort->mat();

int Mx=map->xSize();
int My=map->ySize();
int Mz=map->zSize();

int x,y,x2,y2;
float xx,yy,zz;
for (y=0; y<ySize(); y++) {
	y2=y-ySize()/2-DY;
	for (x=0; x<xSize(); x++) {
		x2=x-xSize()/2-DX;
				
		xx=x2*mx[0]+y2*mx[1]+DZ*mx[2]+Mx/2;
		yy=x2*mx[3]+y2*mx[4]+DZ*mx[5]+My/2;
		zz=x2*mx[6]+y2*mx[7]+DZ*mx[8]+Mz/2;
		if (xx<0||yy<0||zz<0 || xx>Mx-2||yy>My-2||zz>Mz-2) continue;
		
		int k=round(xx)+round(yy)*Mx+round(zz)*Mx*My;
		ddata[k]=sdata[x+y*xSize()];
		
	}
}

doneData();
map->doneData();
}


// returns an EMData object containing all of the data at y=n
EMData *EMData::row(int n) {
EMData *ret = new EMData;
ret->setSize(xSize(),1,1);
memcpy(ret->getData(),getDataRO()+xSize()*n,xSize()*sizeof(float));
doneData();
ret->doneData();
return ret;
}

// returns an EMData object containing all of the data at x=n
EMData *EMData::col(int n) {
EMData *ret = new EMData;
ret->setSize(ySize(),1,1);
float *dst=ret->getData(),*src=getDataRO();
for (int i=0; i<ySize(); i++) dst[i]=src[i*xSize()+n];
ret->doneData();
doneData();
return ret;
}

void EMData::setRow(EMData *d,int n) {
float *dst=getData(),*src=d->getDataRO();

memcpy(dst+xSize()*n,src,xSize()*sizeof(float));
doneData();
d->doneData();
}

void EMData::setCol(EMData *d,int n) {
float *dst=getData(),*src=d->getDataRO();
for (int i=0; i<ySize(); i++) dst[i*xSize()+n]=src[i];
d->doneData();
doneData();
}


// UNIMPLEMENTED
// 2d matrix multiply  this * second
// Note both row and column vectors (if vector matrix mult) stored
// as nx>1 ny=1
EMData *EMData::mxMult(EMData *second) {
return NULL;
}

// Distance between 2 vectors, n specifies the row in 'second'
float EMData::dist(EMData *second,int n) {
if (second->xSize()!=xSize() || ySize()!=1) return -1.0;
float ret=0;
if (n>second->ySize()||n<0) return -1.0;
float *d1 = getDataRO(), *d2 = second->getDataRO() + second->xSize()*n;
for (int i=0; i<xSize(); i++) ret+=SQR(d1[i]-d2[i]);
return sqrt(ret);
}


/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////   Cross  Common  Line  Search         /////////////////////////////////////////////////
////////////////////////////////// ------   Xiangan Liu   (2004)       //////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// this cross common line search algorithm is a global optimal search algorithm. it is somekind of like global SIMPLEX
void EMData::crossCommonLineSearchPrepare(ccmlParm* parm) {
	int searchMode = parm->searchMode;
	if ((searchMode < 0)||(searchMode > 1)) {
		fprintf(stderr,"ERROR: searchMode %d does NOT exist in function EMData::crossCommonLineSearch()\n", searchMode);
		exit(0);
	}

	//define a random generator and initialize random seed
	//printf("run place 1-1%s  %s\n",parm->rawImageFN, parm->refImagesFN);
	float ran();
	srandom((unsigned)time(NULL)%100000);


	//get raw image header information
	char *rawImageFN = parm->rawImageFN;
	char *refImagesFN = parm->refImagesFN;
	readImage(rawImageFN,0,1); //read header only of the first image
	int rawImageSizeX = xSize();
	int rawImageSizeY = ySize();

	XYData *sf = parm->scateringFactor;
	float deltaR = parm->deltaR;
	float RMin = parm->RMin;
	float RMax = parm->RMax;
	int FFTOverSampleScale = parm->FFTOverSampleScale;  //this one should be defined in input arguments, due to technique problem, just set it here temporarily
	int numOfRefImages = parm->numOfRefImages;

	float PFTSTEP = parm->pftStepSize;
	int   PFTTOTALSTEPS = 360.0/PFTSTEP+5;

	int numSym = euler.getMaxSymEl();
	int numPoints = (RMax - RMin + 1.0)/deltaR;
	int totalNumPoints = numSym*numPoints;

	int   startOfRadDist = RMin*FFTOverSampleScale;
	float stepOfRadDist  = FFTOverSampleScale*deltaR; //dx = FFTOverSampleScale*deltaR
	float stepOfCTFtoFFT = 1.0 / (float) CTFOS;  //dx = 1/CTFOS

	float *weightAmpRefs  = new float[numOfRefImages * numPoints];
	float *weightSNR  = new float[numPoints];
	float *weightSNRSqrt  = new float[numPoints];

	//Save all the parameters to structure "parm"
	parm->numSym = numSym;
	parm->numPoints = numPoints;
	parm->totalNumPoints = totalNumPoints;

	parm->weightSNR = weightSNR;
	parm->weightSNRSqrt = weightSNRSqrt;
	parm->weightAmpRefs = weightAmpRefs;
	parm->weightAmpRef = weightAmpRefs; //this one initially points to weightAmpRefs[0], it will be changed later


	//Allocate spaces for internal usage and save all the parameters to structure "parm"
	//phaseChangeByCenterShift(dx, dy, theta, RMin, deltaR, numPoints, phase1, phaseShift);
	//phaseResidue = phaseResidual(mode, amplitude1, amplitude2, phase1Shift, phase2, numPoints);
	parm->ignoreThisSym = new int[numSym];
	parm->theta         = new float[numSym];
	parm->amplitude1    = new float[totalNumPoints];
	parm->amplitude2    = new float[totalNumPoints];
	parm->phase1        = new float[totalNumPoints];
	parm->phase1Shift   = new float[totalNumPoints];
	parm->phase2        = new float[totalNumPoints];


	int totalPFTPoints = numPoints*PFTTOTALSTEPS; //range from 0--360, conjugate is ignored for fast speed, step is given by PFTSTEP
	float *refsPFT = new float[totalPFTPoints*2*numOfRefImages];
	parm->totalPFTPoints = totalPFTPoints; //save PFT, range from 0--180, other is the congugate, step is given by PFTSTEP
	parm->refsPFT = refsPFT;


	//find a weight from SNR
	float *thisSNR;
	int NP=ceil(CTFOS*sqrt(2.0)*ny/2);
	if (! hasCTF() || parm->weightMode == 0) {
		thisSNR = new float[NP]; //if no ctf parameters, set it to 1. this pointer need to be deleted
		for(int i=0; i<NP; i++) thisSNR[i] = 1;
		for(int i=0; i<numPoints; i++) {
			int j = (RMin + i*deltaR)*CTFOS + 0.5; // 0.5 is for rounding to the nearest point
			weightSNR[i] = thisSNR[j];
			weightSNRSqrt[i] = sqrt(weightSNR[i]);
		}
	}
	else {
		if (parm->weightMode == 1) { // Xiangan's original ad hoc weighting scheme
			thisSNR = ctfCurve(11, sf); //get pure ctf curve only output in range [0, 1] for mode 11, mode 10 --> [-1, 1], this pointer need to be deleted
			
			int firstpeak = 0;
			for(int i=0; i<NP; i++) { //find the first peak of the ctf
				firstpeak = i;
				if (thisSNR[i] >0.98) break;
			}
			
			for(int i=0; i<numPoints; i++) {
				int j = (RMin + i*deltaR)*CTFOS + 0.5; // 0.5 is for rounding to the nearest point
				weightSNR[i] = thisSNR[j];
				if (weightSNR[i]>1.0) weightSNR[i] = 1.0;
				if (j < firstpeak) weightSNR[i]= pow(weightSNR[i], 0.2f); //before first peak, the weight should be large
				else weightSNR[i] = 1.0-exp(-pow((weightSNR[i]/0.3), 4.0)); //make it like a step function at threshold = 0.22
				//if (j < firstpeak) weightSNR[i] = cos(cos(sin(sin(weightSNR[i]*1.57)*1.57)*1.57)*1.57); //make it like a step function at threshold = 0.22
				//else weightSNR[i] = cos(cos(sin(weightSNR[i]*1.57)*1.57)*1.57);
		
				if (weightSNR[i]<0)    weightSNR[i]= 0;
				//printf("%d  %f %f\n", i, weightSNR[i],thisSNR[j]);
				weightSNRSqrt[i] = sqrt(weightSNR[i]);
			}
		}
		else if (parm->weightMode == 2) {	// absolute SNR weighting
			thisSNR = ctfCurve(9, sf); //get SNR of the raw image, this pointer need to be deleted
			for(int i=0; i<numPoints; i++) {
				int j = (RMin + i*deltaR)*CTFOS + 0.5; // 0.5 is for rounding to the nearest point
				weightSNR[i] = thisSNR[j];
				weightSNRSqrt[i] = sqrt(weightSNR[i]);
			}
		}
		else if (parm->weightMode == 3) {	// wiener filter
			thisSNR = ctfCurve(12, sf); // winer filter
			for(int i=0; i<numPoints; i++) {
				int j = (RMin + i*deltaR)*CTFOS + 0.5; // 0.5 is for rounding to the nearest point
				weightSNR[i] = thisSNR[j];
				weightSNRSqrt[i] = sqrt(weightSNR[i]);
			}
		}
		else {
			fprintf(stderr,"ERROR: crossCommonLine search weight mode (%d) is not supported\n", parm->weightMode);
			exit(1);
		}
	}

//exit(0);
	//to get amplitude weight to make it flat, also the CTF of a raw image is applied to the reference/projected image
	float *thisCTF;
	if (hasCTF()) {
		int ctfmode = 0;	// ctf and B factor with sign
		if (parm->rawImagePhaseCorrected) ctfmode = 2;	// ctf and B factor without sign
		thisCTF = ctfCurve(ctfmode, sf); //get CTF curve of the raw image (B factor is applied), this pointer need to be deleted
	} else {
		thisCTF = new float[ny*CTFOS]; //if no ctf parameters, set it to 1. this pointer need to be deleted
		for(int i=0; i<ny*CTFOS; i++) thisCTF[i] = 1;
		//for(int i= 0; i<ny*CTFOS; i++ ) printf("%f  ",thisCTF[i]);
		//printf("===%f  %f\n", sf->x(0), sf->y(0));exit(0);
	}



	Euler *refsEuler = new Euler[numOfRefImages]; //save the Euler angles of reference images
	parm->refsEuler = refsEuler; //save the pointer
	
	//read ref imags and creates its PFTs and make ref amplitude weights
	for (int numRef=0; numRef<numOfRefImages; numRef++) {
		EMData oneOfRefs;
		oneOfRefs.readImage(refImagesFN,numRef);
		if ((rawImageSizeX != oneOfRefs.xSize())||(rawImageSizeY != oneOfRefs.ySize())) {
			printf("Image %s size (%dx%d) is different to reference image %s size (%dx%d),\nQUIT!!!",
				rawImageFN,rawImageSizeX,rawImageSizeY,refImagesFN,oneOfRefs.xSize(),oneOfRefs.ySize());
			exit(0);
		}

		EMData *oneOfRefsFFT = oneOfRefs.doOverSampleFFT(FFTOverSampleScale);

		oneOfRefsFFT -> applyRadFn(ny*CTFOS, 0, stepOfCTFtoFFT, thisCTF, 1);  //apply the CTF of raw image to refs
		oneOfRefsFFT -> calcRadDist(numPoints, startOfRadDist, stepOfRadDist, &weightAmpRefs[numRef * numPoints]); //calculate radian distribution of FFT intensity
		
		//printf("Euler 1 : %f  %f   %f\n",oneOfRefsFFT -> getEuler()->alt(),oneOfRefsFFT ->getEuler()->az(),oneOfRefsFFT -> getEuler()->phi());
		refsEuler[numRef]= *(oneOfRefsFFT -> getEuler()); //copy the euler to refsEuler
		//printf("Euler 2 : %f  %f   %f\n",refsEuler[numRef].alt(),refsEuler[numRef].az(),refsEuler[numRef].phi());
		
		for(int i=0; i<numPoints; i++) weightAmpRefs[i + numRef*numPoints] = sqrt(weightAmpRefs[i + numRef*numPoints]);

		for(int i=0; i<PFTTOTALSTEPS; i++) {
			float thetaCommonLine = i*PFTSTEP/180.0*PI; //convert to Rad, because PFTSTEP is in degree
			int ampOffsetAddress = (2*i+0)*numPoints + totalPFTPoints*2*numRef;
			int phsOffsetAddress = (2*i+1)*numPoints + totalPFTPoints*2*numRef;
			oneOfRefsFFT -> GetCommonLineValueNoScale(parm, thetaCommonLine, &refsPFT[ampOffsetAddress], &refsPFT[phsOffsetAddress]);
		}

		delete oneOfRefsFFT; //delete the FFT
	}

	if(parm->verbose > 3) printf("PFTs of refence images has been created \n");

	delete [] thisSNR;
	delete [] thisCTF;


}  //end function crossCommonLineSearchPrepare


void EMData::crossCommonLineSearchReadRawParticle(ccmlParm* parm) {

	//read raw image and do FFT of raw image and get radius distribution as amplitude weight
	EMData newRawImage;
	newRawImage.readImage(parm->rawImageFN, parm->thisRawImageSN); //read raw image
	
	float dx=0, dy=0, dz=0;
	if(parm->iniParmsGiven) {
		dx = parm->iniCx - newRawImage.xSize()/2.0; 
		dy = parm->iniCy - newRawImage.ySize()/2.0; 
	}
	newRawImage.applyMask(parm->rMask, 6, dx, dy, dz); //apply mask type 6 [edge mean value] to raw image, center will be image center

	float *ctfParm1 = getCTF();
	float *ctfParm2 = newRawImage.getCTF();
	//if (ctfParm1[0] != ctfParm2[0]) { //the defocus has been changed, need to recall crossCommonLineSearchPrepare
	if ( CTFCmp(&newRawImage) ) { //the defocus has been changed, need to recall crossCommonLineSearchPrepare
		crossCommonLineSearchFinalize(parm); //delete all allocated space
		crossCommonLineSearchPrepare(parm);  //redo ref PFTs, allocate interal spaces and Weightings
	}

	if(parm->verbose > 0) printf("\n--------------------------------------------------------------------\n");
	if(parm->verbose > 0) printf("Run image [\"%s\"] :: %d\n",parm->rawImageFN, parm->thisRawImageSN);

	float PFTSTEP = parm->pftStepSize;
	int   PFTTOTALSTEPS = 360.0/PFTSTEP+5;

	int FFTOverSampleScale = parm->FFTOverSampleScale;  //this one should be defined in input arguments, due to technique problem, just set it here temporarily

	float deltaR = parm->deltaR;
	float RMin = parm->RMin;
	float RMax = parm->RMax;

	int numSym = parm->numSym;
	int numPoints = parm->numPoints;
	int totalNumPoints = parm->totalNumPoints;
	int totalPFTPoints = parm->totalPFTPoints;
	int numOfRefImages = parm->numOfRefImages;
	if(parm->verbose > 3) printf("numSym = %d, numPoints = %d, totalNumPoints = %d, refsSize= %d\n", numSym, numPoints, totalNumPoints, numOfRefImages);


	int   startOfRadDist = RMin*FFTOverSampleScale;
	float stepOfRadDist  = FFTOverSampleScale*deltaR; //dx = FFTOverSampleScale*deltaR
	float stepOfCTFtoFFT = 1.0 / (float) CTFOS;  //dx = 1/CTFOS

	float *weightAmpRaw  = new float[numPoints];
	parm->weightAmpRaw = weightAmpRaw;
	EMData *thisFFT = newRawImage.doOverSampleFFT(FFTOverSampleScale); //if it is real image, do over sample FFT. This over sample FFT includes toCorner() function
	thisFFT ->calcRadDist(numPoints, startOfRadDist, stepOfRadDist, weightAmpRaw);  //calculate radian distribution of FFT intensity, used for no scaling problem
	for(int i=0; i<numPoints; i++) weightAmpRaw[i] = sqrt(weightAmpRaw[i]);
	//for(int i=0; i<numPoints; i++) weightAmpRaw[i] = FFTOverSampleScale*FFTOverSampleScale;

	//create pft of raw image to matrix
	float *thisPFT = new float[totalPFTPoints*2];
	parm->thisPFT = thisPFT;
	for(int i=0; i<PFTTOTALSTEPS; i++) {
		float thetaCommonLine = i*PFTSTEP/180.0*PI; //convert to Rad, because PFTSTEP is in degree
		int ampOffsetAddress = (2*i+0)*numPoints;
		int phsOffsetAddress = (2*i+1)*numPoints;
		thisFFT -> GetCommonLineValueNoScale(parm, thetaCommonLine, &thisPFT[ampOffsetAddress], &thisPFT[phsOffsetAddress]);
	}
	if(parm->verbose > 3) printf("PFT of raw image has been created \n");

}  //end function crossCommonLineSearchPrepare


void EMData::crossCommonLineSearchReleaseParticle(ccmlParm* parm) {
	delete [] parm->weightAmpRaw;
	delete [] parm->thisPFT;
}

void EMData::crossCommonLineSearchFinalize(ccmlParm* parm) {

	//free the allocated spaces
	delete [] parm->theta;
	delete [] parm->amplitude1;
	delete [] parm->amplitude2;
	delete [] parm->phase1;
	delete [] parm->phase1Shift;
	delete [] parm->phase2;


	delete [] parm->weightSNR;
	delete [] parm->weightAmpRefs;
	delete [] parm->refsEuler;


	//large memory occupy, has to be deleted
	delete [] parm->refsPFT;

}  //end function crossCommonLineSearch




ccmlOutputParm EMData::crossCommonLineSearch(ccmlParm* parm) {

	float ran();
	srandom((unsigned)time(NULL)%10000+((unsigned)time(NULL)%100)*10000);
			
	
	int searchMode = parm->searchMode;
	//define a function pointer which belongs to type "crossCommonLineSearchMethod", here the type name and variable name are the same
	crossCommonLineSearchMethod crossCommonLineSearchMethod;
	switch(searchMode) { //select different method to minimize residual due to center shift, or ignore center shift (case 2)
		case 0: crossCommonLineSearchMethod = &EMData::crossCommonLineCoarseSearch; break;
		case 1: crossCommonLineSearchMethod = &EMData::crossCommonLineRefinement; break;
		default:{
			crossCommonLineSearchMethod = NULL;
			fprintf(stderr,"ERROR: searchMode %d does NOT exist in function EMData::crossCommonLineSearch()\n", searchMode);
			exit(0);
		}
	}//end switch(searchMode)
	

	///////////////////////////////////////////////////////////////////////////////////
	////////////////////////// start to search   //////////////////////////////////////
	ccmlOutputParm finalParms = (*this.*crossCommonLineSearchMethod)(parm);
	///////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////
      
	finalParms.verbose = parm->verbose;
	finalParms.rawImageSN = parm->thisRawImageSN;
	finalParms.copyIniParm(parm);
	finalParms.copyRefParm(parm);
	finalParms.convertCalculatedOrientationToAsymmetricUnit();


	if(parm->verbose > 1) printf("\nResidual = %7.5f\n", finalParms.residual);


	float tmpCx, tmpCy, tmpAlt, tmpAz, tmpPhi;
	if(parm->iniParmsGiven) { //if the initial parameters are specified, print out the given ones
		tmpCx  = parm->iniCx;  tmpCy = parm->iniCy;
		tmpAlt = parm->iniAlt*180./PI; tmpAz = parm->iniAz*180./PI; tmpPhi = parm->iniPhi*180./PI;
	} else {
		tmpCx  = parm->refCx;  tmpCy = parm->refCy;
		tmpAlt = parm->refAlt*180./PI; tmpAz = parm->refAz*180./PI; tmpPhi = parm->refPhi*180./PI;
	}
	if(parm->verbose > 1) printf("Ini/Ref    Center and Euler angles :: %7.2f  %7.2f    %7.2f  %7.2f  %7.2f\n",tmpCx, tmpCy, tmpAlt, tmpAz, tmpPhi);


	tmpCx  = finalParms.cx;  tmpCy = finalParms.cy;
	tmpAlt = finalParms.alt*180./PI; tmpAz = finalParms.az*180./PI; tmpPhi = finalParms.phi*180./PI;
	if(parm->verbose > 1) printf("Calculated Center and Euler angles :: %7.2f  %7.2f    %7.2f  %7.2f  %7.2f\n",tmpCx, tmpCy, tmpAlt, tmpAz, tmpPhi);

	return finalParms;
}  //end function crossCommonLineSearch



// random number from 0---1, befor use it, srandom(time(0)) has to be done
float ran() {return (float)random()/(1.0+RAND_MAX);}



//================= new pso function ====================//

/*
ccmlOutputParm EMData::crossCommonLineRefinement(ccmlParm *parm) {
	int    totalIgnoredCommonLines = 0;
	float  RATIO;
	double zScoreCriterion = parm->zScoreCriterion;
	double residualCriterion = parm->residualCriterion;
	double solutionCenterDiffCriterion = parm->solutionCenterDiffCriterion;
	double solutionOrientationDiffCriterion = parm->solutionOrientationDiffCriterion;
	int totalNumOfConfigurations = parm-> numOfStartConfigurations; //in general it should be total # of variables + 1
	int numOfConfToBeUpdated = parm-> numOfStartConfigurations;

	//preset orientation and center parameters, construct a parameter object and paramenter object list
	Quaternion quatIni(parm->iniQ);
	Quaternion quatOld(parm->iniQ);
	//printf("Output Ini Q :: %8.4f %8.4f %8.4f %8.4f\n", quatIni.w,quatIni.x,quatIni.y,quatIni.z);
	float cxIni = parm->iniCx;
	float cyIni = parm->iniCy;
	float residualIni = parm->residual+1;
	float cxOld = parm->iniCx;
	float cyOld = parm->iniCy;
	float residualOld = residualIni;
	list< configParm > configurationParmsList;
	configParm confOld(quatOld.w, quatOld.x, quatOld.y, quatOld.z, cxOld, cyOld, residualOld+1.0, &configurationParmsList); //initial parameters
	configurationParmsList.push_back(confOld); //put the best parms to the list.

printf("old: vx = %f, vy = %f\n",confOld.vx,confOld.vy);
printf("List: vx = %f, vy = %f\n",configurationParmsList.end()->vx,configurationParmsList.end()->vy);

	//flip the orientation
	Quaternion quatFlip(quatOld.getEuler().alt(), quatOld.getEuler().az(), quatOld.getEuler().phi()+PI);
	configParm confFlip(quatFlip.w, quatFlip.x, quatFlip.y, quatFlip.z, cxOld, cyOld, residualOld+1.0, &configurationParmsList); //initial parameters
	configurationParmsList.push_back(confFlip); //put the best parms to the list.


	//handle the extral parms
	float  *extrCx = parm->extrCx;
	float  *extrCy = parm->extrCy;
	float  *extrAlt= parm->extrAlt;
	float  *extrAz = parm->extrAz;
	float  *extrPhi= parm->extrPhi;
	double *extrQw = parm->extrQw;
	double *extrQx = parm->extrQx;
	double *extrQy = parm->extrQy;
	double *extrQz = parm->extrQz;
	Quaternion *extrQ = parm->extrQ;
	float  *extrResidual = parm->extrResidual;
	int totalExtrNum = parm->totalExtrNum;
	for (int i=0; i<totalExtrNum; i++) {
		if(configurationParmsList.size() == totalNumOfConfigurations) break;
		configParm confExtr(extrQw[i], extrQx[i], extrQy[i], extrQz[i], extrCx[i], extrCy[i], extrResidual[i]+1.0, &configurationParmsList); //extral initial parameters
		configurationParmsList.push_back(confExtr); //put the extral parms to the list.

		//flip the orientation
		Quaternion quatExtrFlip(extrAlt[i], extrAz[i], extrPhi[i]+PI);
		configParm confExtrFlip(quatExtrFlip.w, quatExtrFlip.x, quatExtrFlip.y, quatExtrFlip.z, extrCx[i], extrCy[i], extrResidual[i]+1.0, &configurationParmsList); //initial parameters
		configurationParmsList.push_back(confExtrFlip); //put the best parms to the list.

	}

	//If the list is not long enough (<totalNumOfConfigurations), just copy the existed elements
	int lengthOfConfList = configurationParmsList.size();
	list<configParm>::iterator iterCopyFrom=configurationParmsList.begin(); //find the starting pointer
	for (int i=lengthOfConfList; i< totalNumOfConfigurations; i++) { //make copy until the list full
		//configParm confCopy = *iterCopyFrom;
		configurationParmsList.push_back(*iterCopyFrom); //copy to the end of the list.
		iterCopyFrom++;
	}

	list<configParm>::iterator iterIniSpeed=configurationParmsList.begin(); //initialize the speed/steps
	for (int i=lengthOfConfList; i< totalNumOfConfigurations; i++) { 
		float dev = 10.0;
		iterIniSpeed->vx = dev*(ran()-0.5); iterIniSpeed->vy = dev*(ran()-0.5);
		float nx=ran(),ny=ran(),nz=ran(),theta=dev/180.0*3.14*(ran()-0.5);
		float N = sqrt(nx*nx + ny*ny + nz*nz);
		nx /= N; ny /= N; nz /= N;
		iterIniSpeed->vq.setQuaternion(cos(theta/2), sin(theta/2)*nx, sin(theta/2)*ny, sin(theta/2)*nz); //rotate along Z 10 degree
		iterIniSpeed++;
	}
	
	//save the best one
	float bestResidual= residualOld;
	float bestCx = cxOld, bestCy = cyOld, deltaResidual;
	Quaternion bestQuat(quatOld);

	{
		float dev = 5.0;
		float nx=ran(),ny=ran(),nz=ran(),theta=dev/180.0*3.14*(ran()-0.5);
		float N = sqrt(nx*nx + ny*ny + nz*nz);
		nx /= N; ny /= N; nz /= N;
		Quaternion randQ(cos(theta/2), sin(theta/2)*nx, sin(theta/2)*ny, sin(theta/2)*nz); 
		bestQuat = randQ*bestQuat;
		bestCx += dev*(ran()-0.5), bestCy += dev*(ran()-0.5);
	}
	

	//current using parameters
	configParm confNew = confOld, confTmp;
	list<configParm>::iterator itr;
	float cx, cy, residual;
	Quaternion quat(parm->iniQ);

	int acceptThisNewConf = 1;
	float ranStep;
	double s, theta1, theta2;

	int   iterationOfRandomJump = parm->numOfRandomJump;  //50  is default
	int   iterationOfFastShrink = parm->numOfFastShrink;  //150 is default

	int maxIteration = parm-> maxNumOfIteration;

	//preset controll parameters
	float maxXShift = parm->centerSearchRange;
	float maxYShift = parm->centerSearchRange;
	double maxRange = parm->orientationSearchRange; //max range in Rad
	double cosMaxRange = cos(maxRange/2.0);

	float stepX = maxXShift;  //step sizes on x, y directions
	float stepY = maxYShift;
	double range = maxRange/2.0;
	
	float vx, vy, vxOld, vyOld;
	Quaternion vq, vqOld;

	int totalNumIteration = 0, totalNumIterationBest = 0;
	double totalResidual = 0, totalResidual2 = 0;
	float MINSTEP = 0.35, MINRANGE = 0.25;

	for(int iter=0; iter<maxIteration; iter++) {

		int countRedoSameVariable = 0;
		for(int variables=0; variables<5; variables++) {
			list<configParm>::iterator iterConf;
			if(configurationParmsList.size() == totalNumOfConfigurations) {
				configurationParmsList.sort();  //work on the bad one, so we need to sort
				int snList;
				if(iter < (maxIteration-iterationOfFastShrink)) snList = ran()*numOfConfToBeUpdated + 1; //randomly pick one conf from the last "numOfConfToBeUpdated" confs.
				else        snList = sqrt(ran())*numOfConfToBeUpdated + 1; //randomly pick one conf from the last "numOfConfToBeUpdated" confs.
				iterConf=configurationParmsList.end(); //find the ending pointer
				for(int dum=0; dum<snList; dum++) iterConf--; //find the pointer we picked
				confOld = *iterConf; //point to a random element of the list
				//if(iter < (maxIteration-iterationOfFastShrink)) {
				if(iter < maxIteration) {
					configurationParmsList.erase(iterConf); //delete the picked one, since it will be move to the last one
					configurationParmsList.push_back(confOld);//put the selected one to back for replacing if new conf is adopted
					iterConf=configurationParmsList.end(); //find the ending pointer
				}
			}

			quatIni.setQuaternion(confOld.wAnc, confOld.xAnc, confOld.yAnc, confOld.zAnc);  //restore the parameters of the ancestor
			cxIni = confOld.cxAnc; cyIni = confOld.cyAnc;

			quatOld.setQuaternion(confOld.w, confOld.x, confOld.y, confOld.z);  //restore the parameters
			cxOld = confOld.cx; cyOld = confOld.cy;
			residualOld = confOld.residual;
			
			vxOld = confOld.vx; vyOld = confOld.vy; vqOld.setQuaternion(confOld.vq); 


			float learningFactor = 2.0;
			cx = cxOld; cy = cyOld; quat.setQuaternion(quatOld); //copy the old parms, then update one of them
			vx = vxOld; vy = vyOld; vq.setQuaternion(vqOld);
			
			int maxRedoNum = 500; 
			int countRedo = 0;
			if(variables < 3) {
					float psoweight = 0.6*(1-iter/(maxIteration+0.1)) + 0.4;
					quat = quatOld.pso(quatIni, bestQuat, vqOld, &vq, ran(), ran(), learningFactor,psoweight);
					
			} else if (variables < 4) {
				redoX:
					if(countRedo++ > maxRedoNum) continue; //to break dead lock
					vx = (0.6*(1-iter/(maxIteration+0.1)) + 0.4)*vxOld + learningFactor*ran()*(cxIni - cxOld) + learningFactor*ran()*(bestCx - cxOld);
					cx = cxOld + vx;
					if ((cx>500)||(cx<300)) goto redoX;
			} else if (variables < 5) {
				redoY:
					if(countRedo++ > maxRedoNum) continue; //to break dead lock
					vy = (0.6*(1-iter/(maxIteration+0.1)) + 0.4)*vyOld + learningFactor*ran()*(cyIni - cyOld) + learningFactor*ran()*(bestCy - cyOld);
					cy = cyOld + vy;
					if ((cy>500)||(cy<300)) goto redoY;
			}
if((iter<5)&&(variables > 2)) printf("vx, vy old = %f, %f : new = %f, %f \n",vxOld,vyOld, vx,vy);

			//printf("dev, dx, dy  %f  %f %f : %f  %f %f\n", cosMaxRange, cos(maxRange/2.0), quat.dotQuaternion(quatIni), acos(quat.dotQuaternion(quatIni))*2/3.14*180,fabs(cx-cxIni), fabs(cy-cyIni));

			Euler thisEuler = quat.getEuler(); //convert to Euler angles
			parm->alt = thisEuler.alt();
			parm->az  = thisEuler.az();
			parm->phi = thisEuler.phi();
			parm->cx  = cx;
			parm->cy  = cy;

			residual = phaseResidualMultiProjectionWithCenterShift(parm);

			//if(iter < (maxIteration-iterationOfFastShrink)) {
			if(iter < (iterationOfRandomJump/2)) {
				totalNumIteration++;
				totalResidual += residual;
				totalResidual2 += (residual*residual);
			}
			
			//printf("anc, res : %7.5f  %7.5f\n",confOld.residualAnc, residual);

			if ((confOld.residualAnc > residual)||(iter<0)) { //save the this(local) best one
						printf("@@@%1d@@@",iter/250); 
						printf(" %7.2f  %7.2f  %7.2f  %7.2f  %7.2f  %7.5f  %7.5f\n", cx, cy, parm->alt*180/3.14, parm->az*180/3.14, parm->phi*180/3.14, residual, confOld.residualAnc);
						fflush(stdout);
				confOld.setAncestorConfigParm(quat.w, quat.x, quat.y, quat.z, cx, cy, residual); //copy current paramters as best parms of this configuration, then it will refine around these parms next time
				iterConf->setAncestorConfigParm(quat.w, quat.x, quat.y, quat.z, cx, cy, residual); //set current paramters to the corresponding configuration of the list, then it will refine around these parms next time
			}
			
			confNew.setConfigParm(quat.w, quat.x, quat.y, quat.z, cx, cy, residual, &configurationParmsList); //create a temple object for all parameters of the current configuration
			confNew.setAncestorConfigParm(confOld); //copy the parameters of its ancestor
			confNew.setSpeed(vx,vy,vq); //save the speed

			if(configurationParmsList.size() == totalNumOfConfigurations) {
				configurationParmsList.pop_back();    //delete the last element of the list
				if(parm->verbose > 2) {if(iter%15==0) {printf("*%1d",iter/250); fflush(stdout);}}
			}

			configurationParmsList.push_back(confNew);


			//save the best one
			if((bestResidual > residual)||(iter<0)) { //save the best one
					//if(parm->verbose > 12) {
						printf("####%1d####",iter/250); 
						printf(" %7.2f  %7.2f  %7.2f  %7.2f  %7.2f  %7.5f\n", cx, cy, parm->alt*180/3.14, parm->az*180/3.14, parm->phi*180/3.14, residual);
						fflush(stdout);
					//}
				bestQuat.setQuaternion(quat); bestCx = cx; bestCy = cy; bestResidual = residual;
				totalNumIterationBest = totalNumIteration;
				totalIgnoredCommonLines = parm->totalIgnoredCommonLines;
			}
		}//end variables
	}//end iteration

	totalResidual /= totalNumIteration;
	totalResidual2 /= totalNumIteration;
	float sigmaOfThisRun = sqrt(totalResidual2 -totalResidual*totalResidual);
	RATIO = (totalResidual-bestResidual)/sigmaOfThisRun;
	printf("sigma = %7.5f\n",sigmaOfThisRun);

	//////////////////////////////////////////////////////////////////////////
	///////////////// check if it pass all criteria  /////////////////////////
	//////////////////////////////////////////////////////////////////////////
	float modifiedZScoreCriterion = 6.5;
	//if (sigmaOfThisRun < 0.03) modifiedZScoreCriterion = zScoreCriterion-6.5+(4100*sigmaOfThisRun*sigmaOfThisRun-340*sigmaOfThisRun+10);
	//else modifiedZScoreCriterion = zScoreCriterion-6.5+4.0*exp(-4.0*sigmaOfThisRun);
	modifiedZScoreCriterion = zScoreCriterion;


	if(parm->verbose > 3) printf("\nAverage residual = %8.5f, ratio = %8.5f, diff = %8.5f, best = %8.5f\n", totalResidual, (totalResidual-bestResidual)/sigmaOfThisRun, totalResidual - bestResidual,bestResidual);
	if(parm->verbose > 3) printf("Total ignored common Lines :: %d\n", totalIgnoredCommonLines);

	//just output the calculated results, the common parameter will be copied in upper level function
	ccmlOutputParm finalResult;
	finalResult.setCalculatedCenterAndOrientation(bestCx, bestCy, bestQuat);
	finalResult.totalIgnoredCommonLines = totalIgnoredCommonLines;
	finalResult.zScore = RATIO;
	finalResult.zScoreCriterion = modifiedZScoreCriterion;
	finalResult.sigma = sigmaOfThisRun;
	finalResult.residual = bestResidual;
	finalResult.sym=parm->sym;
	return finalResult;

} //end crossCommonLinerefinement

//================= new pso function ====================//


*/

ccmlOutputParm EMData::crossCommonLineRefinement(ccmlParm *parm) {
	int    totalIgnoredCommonLines = 0;
	float  RATIO;
	double zScoreCriterion = parm->zScoreCriterion;
	double residualCriterion = parm->residualCriterion;
	double solutionCenterDiffCriterion = parm->solutionCenterDiffCriterion;
	double solutionOrientationDiffCriterion = parm->solutionOrientationDiffCriterion;
	int totalNumOfConfigurations = parm-> numOfStartConfigurations; //in general it should be total # of variables + 1
	int numOfConfToBeUpdated = parm-> numOfStartConfigurations;

	//preset orientation and center parameters, construct a parameter object and paramenter object list
	Quaternion quatIni(parm->iniQ);
	Quaternion quatOld(parm->iniQ);
	//printf("Output Ini Q :: %8.4f %8.4f %8.4f %8.4f\n", quatIni.w,quatIni.x,quatIni.y,quatIni.z);
	float cxIni = parm->iniCx;
	float cyIni = parm->iniCy;
	float residualIni = parm->residual+1;
	float cxOld = parm->iniCx;
	float cyOld = parm->iniCy;
	float residualOld = residualIni;
	list< configParm > configurationParmsList;
	configParm confOld(quatOld.w, quatOld.x, quatOld.y, quatOld.z, cxOld, cyOld, residualOld+1.0, &configurationParmsList); //initial parameters
	configurationParmsList.push_back(confOld); //put the best parms to the list.

	//flip the orientation
	Quaternion quatFlip(quatOld.getEuler().alt(), quatOld.getEuler().az(), quatOld.getEuler().phi()+PI);
	configParm confFlip(quatFlip.w, quatFlip.x, quatFlip.y, quatFlip.z, cxOld, cyOld, residualOld+1.0, &configurationParmsList); //initial parameters
	configurationParmsList.push_back(confFlip); //put the best parms to the list.


	//handle the extral parms
	float  *extrCx = parm->extrCx;
	float  *extrCy = parm->extrCy;
	float  *extrAlt= parm->extrAlt;
	float  *extrAz = parm->extrAz;
	float  *extrPhi= parm->extrPhi;
	double *extrQw = parm->extrQw;
	double *extrQx = parm->extrQx;
	double *extrQy = parm->extrQy;
	double *extrQz = parm->extrQz;
	Quaternion *extrQ = parm->extrQ;
	float  *extrResidual = parm->extrResidual;
	int totalExtrNum = parm->totalExtrNum;
	for (int i=0; i<totalExtrNum; i++) {
		if(configurationParmsList.size() == totalNumOfConfigurations) break;
		configParm confExtr(extrQw[i], extrQx[i], extrQy[i], extrQz[i], extrCx[i], extrCy[i], extrResidual[i]+1.0, &configurationParmsList); //extral initial parameters
		configurationParmsList.push_back(confExtr); //put the extral parms to the list.

		//flip the orientation
		Quaternion quatExtrFlip(extrAlt[i], extrAz[i], extrPhi[i]+PI);
		configParm confExtrFlip(quatExtrFlip.w, quatExtrFlip.x, quatExtrFlip.y, quatExtrFlip.z, extrCx[i], extrCy[i], extrResidual[i]+1.0, &configurationParmsList); //initial parameters
		configurationParmsList.push_back(confExtrFlip); //put the best parms to the list.

	}

	//If the list is not long enough (<totalNumOfConfigurations), just copy the existed elements
	int lengthOfConfList = configurationParmsList.size();
	list<configParm>::iterator iterCopyFrom=configurationParmsList.begin(); //find the starting pointer
	for (int i=lengthOfConfList; i< totalNumOfConfigurations; i++) { //make copy until the list full
		//configParm confCopy = *iterCopyFrom;
		configurationParmsList.push_back(*iterCopyFrom); //copy to the end of the list.
		iterCopyFrom++;
	}


	//save the best one
	float bestResidual= residualOld;
	float bestCx = cxOld, bestCy = cyOld, deltaResidual;
	Quaternion bestQuat(quatOld);

	//current using parameters
	configParm confNew = confOld, confTmp;
	list<configParm>::iterator itr;
	float cx, cy, residual;
	Quaternion quat(parm->iniQ);

	int acceptThisNewConf = 1;
	float ranStep;
	double s, theta1, theta2;

	int   iterationOfRandomJump = parm->numOfRandomJump;  //50  is default
	int   iterationOfFastShrink = parm->numOfFastShrink;  //150 is default

	int maxIteration = parm-> maxNumOfIteration;

	//preset controll parameters
	float maxXShift = parm->centerSearchRange;
	float maxYShift = parm->centerSearchRange;
	double maxRange = parm->orientationSearchRange; //max range in Rad
	double cosMaxRange = cos(maxRange/2.0);

	float stepX = maxXShift;  //step sizes on x, y directions
	float stepY = maxYShift;
	double range = maxRange/2.0;


	int totalNumIteration = 0, totalNumIterationBest = 0;
	double totalResidual = 0, totalResidual2 = 0;
	float MINSTEP = 0.35, MINRANGE = 0.25;
	float MAXSTEP = 10.0, MAXRANGE = 7.0;

	for(int iter=0; iter<maxIteration; iter++) {
		//if(iter < (maxIteration-iterationOfRandomJump)) {}
                if(iter ==  iterationOfRandomJump/4) {
                        float factor = 2.0/3.0; 
			if (stepX > MAXSTEP) stepX *= factor; if (stepY > MAXSTEP) stepY *= factor; if (range > MAXRANGE) range *= factor;
                }
                if(iter ==  iterationOfRandomJump*2/4) {
                        float factor = 2.0/3.0;
                        if (stepX > MAXSTEP) stepX *= factor; if (stepY > MAXSTEP) stepY *= factor; if (range > MAXRANGE) range *= factor;
                }
                if(iter ==  iterationOfRandomJump*3/4) {
                        float factor = 2.0/3.0;
                        if (stepX > MAXSTEP) stepX *= factor; if (stepY > MAXSTEP) stepY *= factor; if (range > MAXRANGE) range *= factor;
                }
		if(iter ==  iterationOfRandomJump) {
			float factor = 2.5/3.0; stepX *= factor; stepY *= factor; range *= factor;
			if (stepX < MINSTEP) stepX = MINSTEP; if (stepY < MINSTEP) stepY = MINSTEP; if (range < MINRANGE) range = MINRANGE;
		}
		if(iter == (maxIteration-iterationOfFastShrink + iterationOfRandomJump)/2) {
			float factor = 1.5/2.0; stepX *= factor; stepY *= factor; range *= factor;
			if (stepX < MINSTEP) stepX = MINSTEP; if (stepY < MINSTEP) stepY = MINSTEP; if (range < MINRANGE) range = MINRANGE;
		}
		if(iter == (maxIteration-iterationOfFastShrink)) {
			float factor = 1.5/2.0; stepX *= factor; stepY *= factor; range *= factor;
			if (stepX < MINSTEP) stepX = MINSTEP; if (stepY < MINSTEP) stepY = MINSTEP; if (range < MINRANGE) range = MINRANGE;
		}
		if(iter == (maxIteration-iterationOfFastShrink/3*2)) {
			float factor = 1.0/2.0; stepX *= factor; stepY *= factor; range *= factor;
			if (stepX < MINSTEP) stepX = MINSTEP; if (stepY < MINSTEP) stepY = MINSTEP; if (range < MINRANGE) range = MINRANGE;
		}
		if(iter == (maxIteration-iterationOfFastShrink/3)) {
			float factor = 1.0/2.0; stepX *= factor; stepY *= factor; range *= factor;
			if (stepX < MINSTEP) stepX = MINSTEP; if (stepY < MINSTEP) stepY = MINSTEP; if (range < MINRANGE) range = MINRANGE;
		}

		double cosRange  = cos(range);
		double sMin = cosRange*cosRange;
		double sRange = 1 - sMin;

		int countRedoSameVariable = 0;
		for(int variables=0; variables<5; variables++) {
			if((variables>=3)&&(iter>iterationOfRandomJump)&&(iter<(maxIteration-iterationOfFastShrink))&&(ran() < 0.007)) continue; //skip some cx, cy coarse search, since the residual is very sensitive to center, it is very easy to locate center
			if(configurationParmsList.size() == totalNumOfConfigurations) {
				configurationParmsList.sort();  //work on the bad one, so we need to sort
				int snList;
				if(iter < (maxIteration-iterationOfFastShrink)) snList = ran()*numOfConfToBeUpdated + 1; //randomly pick one conf from the last "numOfConfToBeUpdated" confs.
				else        snList = sqrt(ran())*numOfConfToBeUpdated + 1; //randomly pick one conf from the last "numOfConfToBeUpdated" confs.
				list<configParm>::iterator iterConf=configurationParmsList.end(); //find the ending pointer
				for(int dum=0; dum<snList; dum++) iterConf--; //find the pointer we picked
				confOld = *iterConf; //point to a random element of the list
				if(iter < (maxIteration-iterationOfFastShrink)) {
					configurationParmsList.erase(iterConf); //delete the picked one, since it will be move to the last one
					configurationParmsList.push_back(confOld);//put the selected one to back for replacing if new conf is adopted
				}
				confOld.setResidual(configurationParmsList.back().residual); //set the compared residual to the one to be updated (self or last one)
			}

			quatIni.setQuaternion(confOld.wAnc, confOld.xAnc, confOld.yAnc, confOld.zAnc);  //restore the parameters of the ancestor
			cxIni = confOld.cxAnc; cyIni = confOld.cyAnc;

			quatOld.setQuaternion(confOld.w, confOld.x, confOld.y, confOld.z);  //restore the parameters
			cxOld = confOld.cx; cyOld = confOld.cy;
			residualOld = confOld.residual;


			cx = cxOld;
			cy = cyOld;
			quat.setQuaternion(quatOld);
			int countRedo = 0, maxRedoNum = 150;
			double dev;
			if(variables < 3) {
				redoQ:
					if(countRedo++ > maxRedoNum) continue; //to break dead lock

					s      = sRange*ran() + sMin;    //make sure sqrt(s) > cosRange
					theta1 =  twoPI*ran();
					theta2 =  range*(ran()-0.5)*2;           //make sure Theta2 < range

					int inRange = quat.genUniformQuaternionInARange(s, theta1, theta2, cosRange); //return w to check maximum axes deviation
					if(!inRange) goto redoQ;
					quat.multQuaternion(quatOld); //get trail ratation == quat * quatOld [ q(qold X qold*)q* = (q qold) X (q qold)* ]
					dev = quat.dotQuaternion(quatIni); //find the deviation of this Q to the initial one
					if(fabs(dev) < cosMaxRange)  goto redoQ;
			} else if (variables < 4) {
				redoX:
					if(countRedo++ > maxRedoNum) continue; //to break dead lock
					ranStep = 2.0*ran()-1.0;
					cx = cxOld + stepX*ranStep;// cx=10;
					if (fabs(cx-cxIni)>maxXShift) goto redoX;
			} else if (variables < 5) {
				redoY:
					if(countRedo++ > maxRedoNum) continue; //to break dead lock
					ranStep = 2.0*ran()-1.0;
					cy = cyOld + stepY*ranStep; //cy=10;
					if (fabs(cy-cyIni)>maxYShift) goto redoY;
			}

			//printf("dev, dx, dy  %f  %f %f : %f  %f %f\n", cosMaxRange, cos(maxRange/2.0), quat.dotQuaternion(quatIni), acos(quat.dotQuaternion(quatIni))*2/3.14*180,fabs(cx-cxIni), fabs(cy-cyIni));

			Euler thisEuler = quat.getEuler(); //convert to Euler angles
			parm->alt = thisEuler.alt();
			parm->az  = thisEuler.az();
			parm->phi = thisEuler.phi();
			parm->cx  = cx;
			parm->cy  = cy;

			residual = phaseResidualMultiProjectionWithCenterShift(parm);

			//if(iter < (maxIteration-iterationOfFastShrink)) {
			if(iter < (iterationOfRandomJump/2)) {
				totalNumIteration++;
				totalResidual += residual;
				totalResidual2 += (residual*residual);
			}

			acceptThisNewConf = 1;
			confNew.setConfigParm(quat.w, quat.x, quat.y, quat.z, cx, cy, residual, &configurationParmsList); //create a temple object for all parameters of the current configuration
			confNew.setAncestorConfigParm(confOld); //copy the parameters of its ancestor

			if(configurationParmsList.size() == totalNumOfConfigurations) {
				if(((iter > (iterationOfRandomJump/2))&&(iter <= iterationOfRandomJump)&&(exp(-(residual - residualOld)/0.3*(1+2.0*iter/(1.0+iterationOfRandomJump)))<ran())) ||
					((iter > iterationOfRandomJump)&&(iter <= maxIteration-iterationOfFastShrink)&&(exp(-(residual - residualOld)/0.004*(1+4.0*(iter-iterationOfRandomJump)/(1.0+maxIteration-iterationOfRandomJump-iterationOfFastShrink)))<ran())) ||
					((iter > maxIteration-iterationOfFastShrink)&&(exp(-(residual - residualOld)/0.0001)<ran()))) {
					acceptThisNewConf = 0;
				} else {
					configurationParmsList.pop_back();    //delete the last element of the list
					if(parm->verbose > 2) {printf("*%1d",iter/250); fflush(stdout);}
				}
			}

			if((acceptThisNewConf)||(iter < 2*totalNumOfConfigurations)) { //accept any configuration if iter <30 to make starting points are different from the read in parameters, but it still in range
				if(configurationParmsList.size() == totalNumOfConfigurations) configurationParmsList.pop_back(); 
				configurationParmsList.push_back(confNew);
				}


			//save the best one
			if((bestResidual > residual)&&(iter > iterationOfRandomJump/2)) { //save the best one
					if(parm->verbose > 12) {
						printf("####%1d####",iter/250); 
						printf("\n%7.2f  %7.2f  %7.2f  %7.2f  %7.2f  %7.2f  %7.2f\n", cx, cy, parm->alt*180/3.14, parm->az*180/3.14, parm->phi*180/3.14, residual);
						fflush(stdout);
					}
				bestQuat.setQuaternion(quat); bestCx = cx; bestCy = cy; bestResidual = residual;
				totalNumIterationBest = totalNumIteration;
				totalIgnoredCommonLines = parm->totalIgnoredCommonLines;
			}
		}//end variables
	}//end iteration

	totalResidual /= totalNumIteration;
	totalResidual2 /= totalNumIteration;
	float sigmaOfThisRun = sqrt(totalResidual2 -totalResidual*totalResidual);
	RATIO = (totalResidual-bestResidual)/sigmaOfThisRun;
	printf("sigma = %7.5f\n",sigmaOfThisRun);

	//////////////////////////////////////////////////////////////////////////
	///////////////// check if it pass all criteria  /////////////////////////
	//////////////////////////////////////////////////////////////////////////
	float modifiedZScoreCriterion = 6.5;
	//if (sigmaOfThisRun < 0.03) modifiedZScoreCriterion = zScoreCriterion-6.5+(4100*sigmaOfThisRun*sigmaOfThisRun-340*sigmaOfThisRun+10);
	//else modifiedZScoreCriterion = zScoreCriterion-6.5+4.0*exp(-4.0*sigmaOfThisRun);
	modifiedZScoreCriterion = zScoreCriterion;


	if(parm->verbose > 3) printf("\nAverage residual = %8.5f, ratio = %8.5f, diff = %8.5f, best = %8.5f\n", totalResidual, (totalResidual-bestResidual)/sigmaOfThisRun, totalResidual - bestResidual,bestResidual);
	if(parm->verbose > 3) printf("Total ignored common Lines :: %d\n", totalIgnoredCommonLines);

	//just output the calculated results, the common parameter will be copied in upper level function
	ccmlOutputParm finalResult;
	finalResult.setCalculatedCenterAndOrientation(bestCx, bestCy, bestQuat);
	finalResult.totalIgnoredCommonLines = totalIgnoredCommonLines;
	finalResult.zScore = RATIO;
	finalResult.zScoreCriterion = modifiedZScoreCriterion;
	finalResult.sigma = sigmaOfThisRun;
	finalResult.residual = bestResidual;
	finalResult.sym=parm->sym;
	return finalResult;

} //end crossCommonLinerefinement




ccmlOutputParm EMData::crossCommonLineCoarseSearch(ccmlParm *parm) {
	int    totalIgnoredCommonLines = 0;
	float  RATIO;
	double zScoreCriterion = parm->zScoreCriterion;
	double residualCriterion = parm->residualCriterion;
	double solutionCenterDiffCriterion = parm->solutionCenterDiffCriterion;
	double solutionOrientationDiffCriterion = parm->solutionOrientationDiffCriterion;

	//preset orientation and center parameters, construct a parameter object and paramenter object list
	Quaternion quatIni(parm->iniQ);
	Quaternion quatOld(parm->iniQ);
	//printf("Output Ini Q :: %8.4f %8.4f %8.4f %8.4f\n", quatIni.w,quatIni.x,quatIni.y,quatIni.z);
	float cxIni = parm->iniCx;
	float cyIni = parm->iniCy;
	float cxOld = parm->iniCx;
	float cyOld = parm->iniCy;
	float residualOld = 999.9;
	list< configParm > configurationParmsList;
	configParm confOld(quatOld.w, quatOld.x, quatOld.y, quatOld.z, cxOld, cyOld, residualOld, &configurationParmsList); //initial parameterd
	confOld.setAncestorConfigParm(quatOld.w, quatOld.x, quatOld.y, quatOld.z, cxOld, cyOld, residualOld); //set initial values as the each one's starting best parameters, then it will refine around the temporary the best parameters

	//save the best one
	float bestResidual= residualOld;
	float bestCx = cxOld, bestCy = cyOld, deltaResidual;
	Quaternion bestQuat(quatOld);

	//current using parameters
	list< configParm > bestConfigurationParmsList;
	configParm confNew = confOld, confTmp;
	list<configParm>::iterator itr;
	float cx, cy, residual;
	Quaternion quat(parm->iniQ);
	
	int acceptThisNewConf = 1;
	float ranStep;
	double s, theta1, theta2;

	//preset controll parameters
	float stepX = parm->centerSearchRange;
	float stepY = parm->centerSearchRange;  //step sizes on x, y directions
	float maxShift = parm->centerSearchRange;
	float maxXShift = parm->centerSearchRange;
	float maxYShift = parm->centerSearchRange;
	double maxRange = parm->orientationSearchRange; //max range in Rad

	int   iterationOfRandomJump = parm->numOfRandomJump;  //50  is default
	int   iterationOfFastShrink = parm->numOfFastShrink;  //150 is default

	//shrinked controll parameters
	float stepXS = stepX;
	float stepYS = stepY;
	float maxShiftS = maxShift;
	float maxXShiftS = maxXShift;
	float maxYShiftS = maxYShift;
	double maxRangeS  = maxRange;

	int totalNumOfConfigurations = parm-> numOfStartConfigurations; //in general it should be total # of variables + 1
	int numOfConfToBeUpdated = parm-> numOfStartConfigurations;
	int maxIteration = parm-> maxNumOfIteration;


	int totalNumIteration = 0, totalNumIterationBest = 0;
	double totalResidual = 0, totalResidual2 = 0;
	int copyBestList = 1;
	
	float CN_OR = 1.0;

	for(int iter=0; iter<maxIteration; iter++) {
/*
		if(iter < (iterationOfRandomJump/2)) {}
		else if(iter < iterationOfRandomJump) {stepX = stepXS/2; stepY = stepYS/2; maxRange = 15.0/180.0*PI;}
		else if(iter < (maxIteration-iterationOfFastShrink-iterationOfRandomJump)/3.0+iterationOfRandomJump) {stepX = 17.0; stepY = 17.0; maxRange = 8.0/180.0*PI;}
		else if(iter < (maxIteration-iterationOfFastShrink-iterationOfRandomJump)/2.0+iterationOfRandomJump) {stepX = 10.0; stepY = 10.0; maxRange = 6.0/180.0*PI;}
		else if(iter < (maxIteration-iterationOfFastShrink-iterationOfRandomJump)/1.5+iterationOfRandomJump) {stepX = 5.0; stepY = 5.0; maxRange = 4.0/180.0*PI;}
		else if(iter < (maxIteration-iterationOfFastShrink)) {stepX = 3.0; stepY = 3.0; maxRange = 3.0/180.0*PI;}
		else if(iter < (maxIteration-iterationOfFastShrink/2.0)) {stepX = 1.0; stepY = 1.0; maxRange = 2.0/180.0*PI;}
		else if(iter < (maxIteration-iterationOfFastShrink/4.0)) {stepX = 0.5; stepY = 0.5; maxRange = 1.0/180.0*PI;}
		else if(iter <  maxIteration     ) {stepX = 0.25; stepY = 0.25; maxRange = 0.5/180.0*PI;}

		if(iter < (iterationOfRandomJump/2)) {}
		else if(iter < iterationOfRandomJump) {stepX = stepXS; stepY = stepYS; cxIni = bestCx; cyIni = bestCy; maxXShift = maxXShiftS; maxYShift = maxYShiftS;}
		else if(iter < (maxIteration-iterationOfFastShrink)) {stepX = 7.0; stepY = 7.0; cxIni = bestCx; cyIni = bestCy; maxXShift=25.0; maxYShift=25.0;maxRange = 13.0/180.0*PI;}
		else if(iter < (maxIteration-iterationOfFastShrink/1.5)) {stepX = 7.0; stepY = 7.0; cxIni = bestCx; cyIni = bestCy; maxXShift=15.0; maxYShift=15.0;maxRange = 6.0/180.0*PI;}
		else if(iter < (maxIteration-iterationOfFastShrink/3.0)) {stepX = 1.0; stepY = 1.0; cxIni = bestCx; cyIni = bestCy; maxXShift=3.0; maxYShift=3.0; maxRange = 3.0/180.0*PI;}
		else if(iter < (maxIteration-iterationOfFastShrink/6.0)) {stepX = 0.5; stepY = 0.5; cxIni = bestCx; cyIni = bestCy; maxXShift=1.5; maxYShift=1.5; maxRange = 1.0/180.0*PI;}
		else if(iter <  maxIteration     ) {stepX = 0.25; stepY = 0.25; cxIni = bestCx; cyIni = bestCy; maxXShift=0.7; maxYShift=0.7; maxRange = 0.5/180.0*PI;}
*/
/*
                if(iter < (iterationOfRandomJump/14)) {}
		else if(iter < iterationOfRandomJump) {stepX = stepXS/2; stepY = stepYS/2; cxIni = bestCx; cyIni = bestCy; maxShift = maxShiftS/2; }
                //else if(iter < iterationOfRandomJump) {stepX = stepXS; stepY = stepYS; cxIni = bestCx; cyIni = bestCy; maxXShift = maxXShiftS; maxYShift = maxYShiftS;}
                else if(iter < (maxIteration-iterationOfFastShrink)) {stepX = 7.0; stepY = 7.0; cxIni = bestCx; cyIni = bestCy; maxXShift=10.0; maxShift=10.0;}
                else if(iter < (maxIteration-iterationOfFastShrink/3)) {stepX = 2.0; stepY = 2.0; cxIni = bestCx; cyIni = bestCy; maxXShift=6.0; maxShift=6.0; maxRange = 3.0/180.0*PI;}
                else if(iter < (maxIteration-iterationOfFastShrink/1.5)) {stepX = 0.5; stepY = 0.5; cxIni = bestCx; cyIni = bestCy; maxXShift=1.5; maxShift=1.5; maxRange = 1.0/180.0*PI;}
                else if(iter <  maxIteration     ) {stepX = 0.25; stepY = 0.25; cxIni = bestCx; cyIni = bestCy; maxXShift=0.7; maxShift=0.7; maxRange = 0.5/180.0*PI;}
*/

		if(iter < (iterationOfRandomJump/14.0)) {}
		else if(iter < (iterationOfRandomJump/2.0)) {stepX = stepXS/2; stepY = stepYS/2;  maxShift = maxShiftS/2; maxRange = maxRangeS;}
		//else if(iter < iterationOfRandomJump/1.0) {stepX = 15.0; stepY = 15.0; maxShift = 20; maxRange = 29.0/180.0*PI;}
		else if(iter < iterationOfRandomJump/1.0) {stepX = 7.0; stepY = 7.0; maxShift = 10; maxRange = 20.0/180.0*PI;}
		else if(iter < (maxIteration-iterationOfFastShrink-iterationOfRandomJump)/1.0+iterationOfRandomJump) {stepX = 5.0; stepY = 5.0; maxShift = 8; maxRange = 10.0/180.0*PI;}
		//else if(iter < (maxIteration-iterationOfFastShrink-iterationOfRandomJump)/1.5+iterationOfRandomJump) {stepX = 4.0; stepY = 4.0; maxShift = 6; maxRange = 15.0/180.0*PI;}
		//else if(iter < (maxIteration-iterationOfFastShrink)) {stepX = 3.0; stepY = 3.0; maxShift = 5; maxRange = 10.0/180.0*PI;}
		else if(iter < (maxIteration-iterationOfFastShrink/1.5)) {stepX = 3.0; stepY = 3.0; maxShift=6.0; maxRange = 4.0/180.0*PI;}
		else if(iter < (maxIteration-iterationOfFastShrink/4.0)) {stepX = 2.0; stepY = 2.0; maxShift=4.0; maxRange = 2.0/180.0*PI;}
		else if(iter <  maxIteration     ) {stepX = 0.25; stepY = 0.25; maxShift=1.5; maxRange = 0.5/180.0*PI;}
		
		//if(iter > (iterationOfRandomJump/2.0)) {stepX /= 2.0; stepY /= 2.0; maxShift /= 2.0;}
		
		double cosMaxRange = cos(maxRange/2.0);
		double range = maxRange/2.0;
		double cosRange  = cos(range);
		double sMin = cosRange*cosRange;
		double sRange = 1 - sMin;
		
		int countRedoSameVariable = 0;
		for(int variables=0; variables<5; variables++) {
			if ((variables >= 3)&&(iter>iterationOfRandomJump/2)) CN_OR = 3.0 +4.0*(iter/(1.0+maxIteration)); //to make residual drop for the center change
			else CN_OR = 1.0;
			
			if((variables>=3)&&(iter>iterationOfRandomJump)&&(iter<(maxIteration-iterationOfFastShrink))&&(ran() < 0.15)) continue; //skip some cx, cy coarse search, since the residual is very sensitive to center, it is very easy to locate center
			if(configurationParmsList.size() == totalNumOfConfigurations) {
				configurationParmsList.sort();  //work on the bad one, so we need to sort
				//configurationParmsList.back().setConfigParm(bestQuat.w, bestQuat.x, bestQuat.y, bestQuat.z, bestCx, bestCy, configurationParmsList.back().residual, &configurationParmsList); //copy current paramters as best parms of this configuration, then it will refine around these parms next time
				//configurationParmsList.back().setAncestorConfigParm(bestQuat.w, bestQuat.x, bestQuat.y, bestQuat.z, bestCx, bestCy, bestResidual); //copy current paramters as best parms of this configuration, then it will refine around these parms next time
				int snList;
				if(iter < (maxIteration-iterationOfFastShrink)) snList = pow(ran(),1.0f)*numOfConfToBeUpdated + 1; //randomly pick one conf from the last "numOfConfToBeUpdated" confs.
				else        snList = sqrt(ran())*numOfConfToBeUpdated + 1; //randomly pick one conf from the last "numOfConfToBeUpdated" confs.
				if (snList > numOfConfToBeUpdated) {
					printf("Got a snList larger than numOfConfToBeUpdated (%d)\n", snList);
					snList=numOfConfToBeUpdated;
				}
				list<configParm>::iterator iterConf=configurationParmsList.end(); //find the ending pointer
				for(int dum=0; dum<snList; dum++) iterConf--; //find the pointer we picked
				/*
				(iterConf->numOfFail)++; //count fail times, once it success, a new conf will replace this one, and the fail times will be set to 0
				if ((iterConf->numOfFail)> 20) {
					iterConf->numOfFail = 0;
					iterConf->setConfigParm(bestQuat.w, bestQuat.x, bestQuat.y, bestQuat.z, bestCx, bestCy, bestResidual, &configurationParmsList); //copy current paramters as best parms of this configuration, then it will refine around these parms next time
					iterConf->setAncestorConfigParm(bestQuat.w, bestQuat.x, bestQuat.y, bestQuat.z, bestCx, bestCy, bestResidual); //copy current paramters as best parms of this configuration, then it will refine around these parms next time
				}*/

				confOld = *iterConf; //point to a random element of the list
				if(iter < (maxIteration-iterationOfFastShrink)) {
					configurationParmsList.erase(iterConf); //delete the picked one, since it will be move to the last one
					configurationParmsList.push_back(confOld);//put the selected one to back for replacing if new conf is adopted
				}
				confOld.setResidual(configurationParmsList.back().residual); //set the compared residual to the one to be updated (self or last one)
			}


			quatIni.setQuaternion(confOld.wAnc, confOld.xAnc, confOld.yAnc, confOld.zAnc);  //restore the parameters of the ancestor
			cxIni = confOld.cxAnc; cyIni = confOld.cyAnc;
			//quatIni.setQuaternion(bestQuat); cxIni = bestCx; cyIni = bestCy;
				
			quatOld.setQuaternion(confOld.w, confOld.x, confOld.y, confOld.z);  //restore the parameters
			cxOld = confOld.cx; cyOld = confOld.cy;
			residualOld = confOld.residual;


			quat.setQuaternion(quatOld);
			cx = cxOld; cy = cyOld;
			double dev;
			int countRedo = 0, maxRedoNum = 1500;
			if(variables < 3) {
				redoQ:
					if(countRedo++ > maxRedoNum) continue; //to break dead lock

					s      = sRange*ran() + sMin;    //make sure sqrt(s) > cosRange
					theta1 =  twoPI*ran();
					theta2 =  range*(ran()-0.5)*2;           //make sure Theta2 < range

					int inRange = quat.genUniformQuaternionInARange(s, theta1, theta2, cosRange); //return w to check maximum axes deviation
					if(!inRange) goto redoQ;
					quat.multQuaternion(quatOld); //get trail ratation == quat * quatOld [ q(qold X qold*)q* = (q qold) X (q qold)* ]
					//dev = quat.dotQuaternion(quatIni); //find the deviation of this Q to the initial one
					//if(fabs(dev) < cosMaxRange)  {quat.setQuaternion(quatOld); goto redoQ;}
			} else if (variables < 4) {
				redoX:
					if(countRedo++ > maxRedoNum) continue; //to break dead lock
					ranStep = 2.0*ran()-1.0;
					cx = cxOld + stepX*ranStep;// cx=10;
					if (fabs(cx-cxIni)>maxShift) goto redoX;
			} else if (variables < 5) {
				redoY:
					if(countRedo++ > maxRedoNum) continue; //to break dead lock
					ranStep = 2.0*ran()-1.0;
					cy = cyOld + stepY*ranStep; //cy=10;
					if (fabs(cy-cyIni)>maxShift) goto redoY;
			}


			Euler thisEuler = quat.getEuler(); //convert to Euler angles
			parm->alt = thisEuler.alt();
			parm->az  = thisEuler.az();
			parm->phi = thisEuler.phi();
			parm->cx  = cx;
			parm->cy  = cy;
			
			residual = phaseResidualMultiProjectionWithCenterShift(parm);
			//printf("residual = %f\n",residual);
			
			//if(iter < (maxIteration-iterationOfFastShrink)) {
			if(iter < (iterationOfRandomJump/2)) {
				totalNumIteration++;
				totalResidual += residual;
				totalResidual2 += (residual*residual);
			}

			acceptThisNewConf = 1;
			confNew.setConfigParm(quat.w, quat.x, quat.y, quat.z, cx, cy, residual, &configurationParmsList); //create a temple object for all parameters of the current configuration

			if(configurationParmsList.size() == totalNumOfConfigurations) {
				//if(((iter <= iterationOfRandomJump)&&(exp(-(residual - residualOld)/0.1*iter)<ran()))||((iter > iterationOfRandomJump)&&(residual > residualOld))) {
				//if(((iter > iterationOfRandomJump)&&(residual > residualOld))) {
				
				if(((iter > (iterationOfRandomJump/14))&&(iter <= iterationOfRandomJump)&&(exp(-(residual - residualOld)/0.03*CN_OR*(1+2.0*iter/(1.0+iterationOfRandomJump)))<ran())) ||
					((iter > iterationOfRandomJump)&&(iter <= maxIteration-iterationOfFastShrink)&&(exp(-(residual - residualOld)/0.006*CN_OR*(1+4.0*(iter-iterationOfRandomJump)/(1.0+maxIteration-iterationOfRandomJump-iterationOfFastShrink)))<ran())) ||
					((iter > maxIteration-iterationOfFastShrink)&&(exp(-(residual - residualOld)/0.0001)<ran()))) {
					acceptThisNewConf = 0;
				} else {
					configurationParmsList.pop_back();    //delete the last element of the list
					if(parm->verbose > 1) {printf("*%1d",iter/250); fflush(stdout);}
				}
			}

			if(acceptThisNewConf) {
				if (confOld.residualAnc > residual)
					confNew.setAncestorConfigParm(quat.w, quat.x, quat.y, quat.z, cx, cy, residual); //copy current paramters as best parms of this configuration, then it will refine around these parms next time
				else 	{
					confNew.setAncestorConfigParm(confOld); //if it does not get better, keep old best one
					confNew.numOfFail = confOld.numOfFail;
				}
				
				configurationParmsList.push_back(confNew);
			}

			//save the best one
			if(bestResidual > residual) { //save the best one
				bestQuat.setQuaternion(quat); bestCx = cx; bestCy = cy; bestResidual = residual;
				totalNumIterationBest = totalNumIteration;
				totalIgnoredCommonLines = parm->totalIgnoredCommonLines;
				if(copyBestList) { //save the best to list for the first 50 iterations, then treat the best list as starting confs
					if(bestConfigurationParmsList.size() == totalNumOfConfigurations) bestConfigurationParmsList.pop_back(); //when list full, delete the last one which is the worst one
					bestConfigurationParmsList.push_front(confNew);
					if(iter > iterationOfRandomJump) {configurationParmsList = bestConfigurationParmsList; copyBestList = 0;} //just copy once
				}
			}
		}//end variables
	}//end iteration

	totalResidual /= totalNumIteration;
	totalResidual2 /= totalNumIteration;
	float sigmaOfThisRun = sqrt(totalResidual2 -totalResidual*totalResidual);
	RATIO = (totalResidual-bestResidual)/sigmaOfThisRun;
	printf("sigma = %7.5f\n",sigmaOfThisRun);

	//////////////////////////////////////////////////////////////////////////
	///////////////// check if it pass all criteria  /////////////////////////
	//////////////////////////////////////////////////////////////////////////
	float modifiedZScoreCriterion = 6.5;
	//if (sigmaOfThisRun < 0.03) modifiedZScoreCriterion = zScoreCriterion-6.5+(4100*sigmaOfThisRun*sigmaOfThisRun-340*sigmaOfThisRun+10);
	//else modifiedZScoreCriterion = zScoreCriterion-6.5+4.0*exp(-4.0*sigmaOfThisRun);
	modifiedZScoreCriterion = zScoreCriterion;


	if(parm->verbose > 3) printf("\nAverage residual = %8.5f, ratio = %8.5f, diff = %8.5f, best = %8.5f\n", totalResidual, (totalResidual-bestResidual)/sigmaOfThisRun, totalResidual - bestResidual,bestResidual);
	if(parm->verbose > 3) printf("Total ignored common Lines :: %d\n", totalIgnoredCommonLines);

	//just output the calculated results, the common parameter will be copied in upper level function
	ccmlOutputParm finalResult;
	finalResult.setCalculatedCenterAndOrientation(bestCx, bestCy, bestQuat);
	finalResult.totalIgnoredCommonLines = totalIgnoredCommonLines;
	finalResult.zScore = RATIO;
	finalResult.zScoreCriterion = modifiedZScoreCriterion;
	finalResult.sigma = sigmaOfThisRun;
	finalResult.residual = bestResidual;
	finalResult.sym=parm->sym;
	return finalResult;

} //end crossCommonLineCoarseSearch




float EMData::phaseResidualMultiProjectionWithCenterShift(ccmlParm *parm) {
	float *amplitude1 = parm->amplitude1;
	float *amplitude2 = parm->amplitude2;
	float *phase1 = parm->phase1;
	float *phase1Shift = parm->phase1Shift;
	float *phase2 = parm->phase2;
	float dx = parm->cx - xSize()/2;  //calculate center shift, here CX is the current absolute center position
	float dy = parm->cy - ySize()/2;


	int numRefs = parm->numOfRefImages;
	float *weightAmpRefs = parm->weightAmpRefs;
	int numPoints = parm->numPoints;

	float residual=0;
	float totalIgnoredCommonLines = 0;
	for (int numRef=0; numRef<numRefs; numRef++) {
		parm->numRef = numRef;
		//parm->refFFT =  parm->refsFFT[numRef];
		parm->refEuler =  parm->refsEuler + numRef;
		//printf("Euler 3 : %f  %f   %f\n",parm->refEuler->alt(),parm->refEuler->az(),parm->refEuler->phi());
		parm->weightAmpRef = &weightAmpRefs[numRef*numPoints];  //find the corresponding weight for this ref image

		GetCommonLineValueWithSymetry(parm);

		phaseChangeByCenterShift(parm, dx, dy, phase1, phase1Shift);
		residual += phaseResidual(parm, amplitude1, amplitude2, phase1Shift, phase2);
		//printf("residual = %f\n",residual);

		for(int i=0; i<(parm->numSym); i++) totalIgnoredCommonLines += (parm->ignoreThisSym[i]);
	}
	residual /= ((numRefs*parm->numSym-totalIgnoredCommonLines)/parm->numSym);
	parm->totalIgnoredCommonLines = totalIgnoredCommonLines;
	return residual;
}




void EMData::GetCommonLineValueWithSymetry(ccmlParm *parm) {
	//get the needed parameters
	float *thisPFT = parm->thisPFT;
	float *refsPFT = parm->refsPFT;

	int    numPoints = parm->numPoints;

	float *amplitude1 = parm->amplitude1;
	float *amplitude2 = parm->amplitude2;
	float *phase1 = parm->phase1;
	float *phase2 = parm->phase2;

	int totalPFTPoints = parm->totalPFTPoints;
	int numRef = parm->numRef;
	float PFTSTEP = parm->pftStepSize;

	//define vector, pointers, etc
	Vect thisNormal, refNormal, theCommonLine, thisCommonLine, refCommonLine;
	float theta1, theta2;
	float *amp1, *amp2, *phs1, *phs2;

	//refNormal = parm->refFFT->getEuler()->GetNormal();
	refNormal = parm->refEuler->GetNormal();

	Euler thisEuler = Euler(parm->alt, parm->az, parm->phi);
	thisEuler.setSym(getEuler()->getSym());

	int numSym = 0;
	for (Euler e = thisEuler.FirstSym(); e.valid(); e = thisEuler.NextSym(), numSym++) {
		parm->ignoreThisSym[numSym] = 0;

		thisNormal =  e.GetNormal();            //get normal from the euler angles

		theCommonLine = thisNormal.CrossP(refNormal);  // the common line is the cross product of the two normals
		if(theCommonLine.length() < 0.5) {  //0.17 = sin 10 degree
			parm->ignoreThisSym[numSym] = 1;
			continue;  //if the angle of the two normal line less than 10 degree, it will be ignored
		}

		thisCommonLine =  e.TransformXYZtoXYZprime (theCommonLine);  // map the 3D common line to 2D line of the Fourier plan
		//refCommonLine = refFFT->getEuler()->TransformXYZtoXYZprime (theCommonLine);
		refCommonLine = parm->refEuler->TransformXYZtoXYZprime (theCommonLine);

		//the common line angle in each FFT image
		theta1=atan2(thisCommonLine.Y(), thisCommonLine.X());
		theta2=atan2(refCommonLine.Y(), refCommonLine.X());
		theta1=fmod((theta1+2*twoPI), twoPI);
		theta2=fmod((theta2+2*twoPI), twoPI);

		//save thisCommonLine angle in parm
		parm->theta[numSym] = theta1;

		//find the start point to store the common line amplitude amd phase for each symetry
		amp1=amplitude1 + numSym*numPoints;
		amp2=amplitude2 + numSym*numPoints;
		phs1=phase1 + numSym*numPoints;
		phs2=phase2 + numSym*numPoints;

		//select mode to do scaling or not
		if(parm->scalingMode==0) {

			int theta1Index = theta1*180.0/PI/PFTSTEP+0.5;  //conjugate problem??????????????
			int amp1OffsetAddress = (2*theta1Index+0)*numPoints;
			int phs1OffsetAddress = (2*theta1Index+1)*numPoints;
			memcpy(amp1, &thisPFT[amp1OffsetAddress],sizeof(float)*numPoints);
			memcpy(phs1, &thisPFT[phs1OffsetAddress],sizeof(float)*numPoints);
			//for (int i=0;i<numPoints; i++) printf("%d  %f   %f \n", i,amp1[i],phs1[i]);

			int theta2Index = theta2*180.0/PI/PFTSTEP+0.5;
			int amp2OffsetAddress = (2*theta2Index+0)*numPoints + totalPFTPoints*2*numRef;
			int phs2OffsetAddress = (2*theta2Index+1)*numPoints + totalPFTPoints*2*numRef;
			memcpy(amp2, &refsPFT[amp2OffsetAddress],sizeof(float)*numPoints);
			memcpy(phs2, &refsPFT[phs2OffsetAddress],sizeof(float)*numPoints);
			//for (int i=0;i<numPoints; i++) printf("%d  %d  %f   %f \n", numRef, i,amp2[i],phs2[i]);

		} else {
			//thisFFT->GetCommonLineValueWithScale(parm, theta1, amp1, phs1);  //get the amplitudes and phases of this common line
			//refFFT->GetCommonLineValueWithScale(parm, theta2, amp2, phs2);  //get the amplitudes and phases of the other
		}  //end if parm
		//exit(0);
	}  //end for Euler e
}  
//end function GetCommonLineValueWithSymetry



void EMData::GetCommonLineValueNoScale(ccmlParm *parm, float theta, float *amplitude, float *phase) {
	//get parameters
	int RMin = parm->RMin;
	float deltaR = parm->deltaR;
	int numPoints = parm->numPoints;
	int overSampleScale = parm->FFTOverSampleScale;

	//this function to get a commonline without considering any scaling problem
	//get both values of amplitude/real and phase/imaginary
	//"theta" is the angle of common line
	//"RMin" is the starting position in R direction
	//"deltaR" is the step in R direction
	//"numPoints" is total number to be calculated in R direction
	//"value1" is the output for amplitude which is converted from the interproted real and image
	//"value2" is the output for phase which is converted from the interproted real and image


	float xx=0, yy=0;
	int x=0, y=0;
	float R;
	int j, k;
	int conjugate = 1;
	float real, image;
	//float sinTheta, cosTheta;

	double sinTheta = sin(theta), cosTheta = cos(theta);
	//if costheta<0, x<0. we neet to convert to its conjugete due to only positive X direction data are stored.
	if(cosTheta < 0) {sinTheta = -sinTheta; cosTheta = -cosTheta; conjugate = -1;}



	for(j = 0; j < numPoints; j++) {
		R =   RMin+j*deltaR; //get the R length
		R *= overSampleScale;  //enlarge R by scale factor, since the FFT enlarged by  scale factor
		xx = R*cosTheta;
		yy = R*sinTheta;

		x=(int)floor(xx);
		y=(int)floor(yy);

		k = 2*x + (y + ny/2)*nx;  //the first one is real value
		real = bilin(rdata[k],rdata[k+2],rdata[k+2+nx],rdata[k+nx],xx-x,yy-y);

		k++;  //the second one is imaginary value
		image = bilin(rdata[k],rdata[k+2],rdata[k+2+nx],rdata[k+nx],xx-x,yy-y)*conjugate;   //conjugate is considered


		amplitude[j] = hypot(real, image);  //convert to amplitude
		phase[j] = atan2(image, real);  //convert to phase
	}  //end j

}



void EMData::GetCommonLineValueWithScale(ccmlParm *parm, float theta, float *amplitude, float *phase) {
	int RMin = parm->RMin;
	int numPoints = parm->numPoints;
	int overSampleScale = parm->FFTOverSampleScale;
	float deltaR = parm->deltaR;

	//this function does consider scaling problem to get a commonline
	//get both values of amplitude/real and phase/imaginary
	//"parm"  is a parameter pointer which points to scaling prameters
	//"theta" is the angle of common line
	//"RMin" is the starting position in R direction
	//"deltaR" is the step in R direction
	//"numPoints" is total number to be calculated in R direction
	//"value1" is the output for amplitude which is converted from the interproted real and image
	//"value2" is the output for phase which is converted from the interproted real and image


	float scaleTheta = 0, scaleX = 1.0, scaleY = 1.0;

	scaleTheta = parm->scaleTheta;
	scaleX = parm->scale;  //suppose the long axis is 1.0 for the elliptical scaling
	scaleY = parm->scale * sqrt(1.0 + parm->scaleE*parm->scaleE);
	////// from undistorted image (x,y) transform to distorted image (x',y') by the following matrix
	////////////////  | a11 = a cos(theta)^2 + b sin(theta)^2,  a12 = (b-a)sin(theta)*cos(theta) |
	////////////////  | a21 = (b-a)sin(theta)*cos(theta), a22 = a sin(theta)^2 + b cos(theta)^2  |
	/////////////////// a = scaleX, b = scaleY

	float sinTheta = sin(scaleTheta), cosTheta = cos(scaleTheta);
	float sinTheta2 = sinTheta*sinTheta, cosTheta2 = cosTheta*cosTheta;
	float a = scaleX, b = scaleY;
	float a11 = a*cosTheta2 + b*sinTheta2, a12 = (b-a)*sinTheta*cosTheta;
	float a21 = a12,                       a22 = a*sinTheta2 + b*cosTheta2;

	float xxx=0, yyy=0;
	float xx=0, yy=0;
	int x=0, y=0;
	float R;
	int j, k;
	int conjugate = 1;
	float real, image;


	getDataRO();   //get the fft data pointer and hold the data

	sinTheta = sin(theta); cosTheta = cos(theta);
	//if costheta<0, x<0. we neet to convert to its conjugete due to only positive X direction data are stored.
	if(cosTheta < 0) {sinTheta = -sinTheta; cosTheta = -cosTheta; conjugate = -1;}
	for(j = 0; j < numPoints; j++) {
		R   = RMin+j*deltaR; //get the R length
		R  *= overSampleScale;
		xxx = R*cosTheta;
		yyy = R*sinTheta;

		//do transform
		xx = a11*xxx + a12*yy;
		yy = a12*xxx + a22*yy;

		x=(int)floor(xx);
		y=(int)floor(yy);

		k = 2*x + (y + ny/2)*nx;  //the first one is real value
		real = bilin(rdata[k],rdata[k+2],rdata[k+2+nx],rdata[k+nx],xx-x,yy-y);  //conjugate is considered

		k++;  //the second one is imaginary value
		image = bilin(rdata[k],rdata[k+2],rdata[k+2+nx],rdata[k+nx],xx-x,yy-y)*conjugate;   //conjugate is considered


		amplitude[j] = hypot(real, image);  //convert to amplitude

		phase[j] = atan2(image, real);  //convert to phase
	}  //end j

	doneData();     //release the data
}



void EMData::phaseChangeByCenterShift(ccmlParm *parm, float cx, float cy, float *phaseIn, float *phaseOut) {

	//pass needed parameters
	int   RMin = parm->RMin;
	int   numPoints = parm->numPoints;
	int   numSym = parm->numSym;
	float *theta = parm->theta;
	float deltaR = parm->deltaR;

	//define new parameters
	//dx, dy:  center shift in pixel on x, y axes for real image
	//"theta" is the angle of common line
	//RMin, deltaR:  starting point, step length in Fourier space respectively, the unit is pixel
	//phase shift = -2 PI (Rx cx + Ry cy)

	int ii;
	float R; //spatial frequency in pixel
	float deltaPhase; //the changed phase
	float cosTheta, sinTheta;

	for(int i=0; i<numSym; i++) {
		if(parm->ignoreThisSym[i]) continue;
		ii = i*numPoints; //memory offset for different symmetry
		sinTheta = sin(theta[i]);
		cosTheta = cos(theta[i]);
		for(int j=0; j<numPoints; j++) {
			R = RMin + j*deltaR;
			deltaPhase = twoPI*(cx*R*cosTheta/nx + cy*R*sinTheta/ny);
			phaseOut[j+ii] = phaseIn[j+ii] + deltaPhase;
			//printf("center shift :: %f   %f :: %f   %f\n",phaseIn[j+ii], phaseOut[j+ii], cx, cy);
			}
		}
}  // end function PhaseChangeByCenterShift


float EMData::phaseResidual(ccmlParm *parm, float *amplitude1, float *amplitude2, float *phase1, float *phase2) {
	//pass needed parameters
	int numPoints = parm->numPoints;
	int numSym = parm->numSym;

	//define new parameters
	float residual = 0;
	float *amp1, *amp2, *phs1, *phs2;

	for(int i=0; i<numSym; i++) {
		if(parm->ignoreThisSym[i]) continue;
		amp1 = amplitude1 + i*numPoints;
		amp2 = amplitude2 + i*numPoints;
		phs1 = phase1 + i*numPoints;
		phs2 = phase2 + i*numPoints;
		residual += onePairCommonLinesPhaseResidualMode(parm, amp1, amp2, phs1, phs2);
	} //end for i<numSym
	return (residual/numSym);
} //end phaseResidual



float EMData::onePairCommonLinesPhaseResidualMode(ccmlParm *parm, float *amplitude1, float *amplitude2, float *phase1, float *phase2) {
	//pass needed parameters
	int   residualMode = parm->residualMode;
	int   numPoints = parm->numPoints;
	float *weightSNR = parm->weightSNR;
	float *weightSNRSqrt = parm->weightSNRSqrt;
	float *weightAmp1 = parm->weightAmpRaw;
	float *weightAmp2 = parm->weightAmpRef;
	float os2=parm->FFTOverSampleScale*parm->FFTOverSampleScale;

	//normalize amplitudes
	{ //make a scope, which will not let the variables mix with other scope's variables
		float avgAmp1 = 0, avgAmp2 = 0;
		for(int k=0; k<numPoints; k++) {
			amplitude1[k] /= weightAmp1[k]; amplitude2[k] /= weightAmp2[k]; //flating the amplitude
			avgAmp1 += amplitude1[k];       avgAmp2 += amplitude2[k];       //find average
		}
		avgAmp1 /= numPoints;       avgAmp2 /= numPoints;
		for(int k=0; k<numPoints; k++) {
			amplitude1[k] /= avgAmp1; amplitude2[k] /= avgAmp2;  //normalize the amplitude, make the two comparing amps have the same order of amplitude
		}
	}




	float weight[3000],weightSqrt[3000];
	if (parm->weightMode == 1) { // Xiangan's original ad hoc weighting scheme
		for(int k=0; k<numPoints; k++) {
			//weight[k]=exp(-pow(k/(0.58*numPoints+1.0),4.0)); weightSqrt[k]=sqrt(weight[k]);
			//if(k<40) weight[k]=1.0-k/3.0/(numPoints+1.0); 
			//else weight[k]=0;
			weight[k]=(1.0-k/2.0/(numPoints+1.0))*weightSNR[k];
			weightSqrt[k]=sqrt(weight[k]);
	
		//printf("the final weight :    %f\n",weightSNR[k]);
			//	weight[k]=1.0-k/2.0/numPoints; weightSqrt[k]=sqrt(weight[k]);
			//	if(amplitude1[k]>0.8 || amplitude2[k]>0.8) {weight[k]=0; weightSqrt[k]=0; actualPoint--;};
				//if(amplitude1[k]<0.02) {weight[k]=0; weightSqrt[k]=0; };
				}
				
		//	float *weight = weightSNR;
		//	float *weightSqrt = weightSNRSqrt;
	}
	else {
		for(int k=0; k<numPoints; k++) {
			weight[k] = weightSNR[k];	// stupid copy, will clean later
			weightSqrt[k] = weightSNRSqrt[k];
		}
	}
	
	float residual=0;
	switch(residualMode) { //select mode
		case 0: {
			//amplitude correlation p=sigma((r1i-avg(r1))*(r2i-avg(r2)))/sqrt(sigma(r1i-avg(r1)^2)*sigma(r2i-avg(r2))^2)
			//normal return value is 0--1, but it could be -1 --- 1
			float corr=0;  //correlation coefficient
			float avgAmp1=0,avgAmp2=0;
			float sigmaXY=0, sigmaXX=0, sigmaYY=0;
			float dAmp1, dAmp2, sumWeight=0;
			float corrCos=0;

			for(int k=0; k<numPoints; k++) {
				avgAmp1 += amplitude1[k];
				avgAmp2 += amplitude2[k];
			}
			avgAmp1 /= numPoints;          avgAmp2 /= numPoints;


			sigmaXY=0; sigmaXX=0; sigmaYY=0; sumWeight=0;
			for(int k=0; k<numPoints; k++) {
				dAmp1 = amplitude1[k]-avgAmp1; dAmp2 = amplitude2[k]-avgAmp2;
				sigmaXY += dAmp1*dAmp2*weight[k];
				sigmaXX += dAmp1*dAmp1*weight[k];
				sigmaYY += dAmp2*dAmp2*weight[k];
	//			sumWeight += weight[k];
	//			corrCos += cos(phase1[k] - phase2[k]);
	//			printf("%4d --> amp1 = %f, amp2 = %f   ----  %f     %f  \n",k, dAmp1, dAmp2, weightAmp1[k], weightAmp2[k]);
				}

			if((sigmaXX==0)&&(sigmaYY==0)) corr = 1.0;
			else if ((sigmaXX==0)||(sigmaYY==0)) corr = 0;
			else corr = sigmaXY / sqrt(sigmaXX*sigmaYY);

			residual = (1.0-corr)/2.0;
			break;
			}
		case 1: {
			float corr=0, phaseDiff=0, sumWeight=0;
			for(int k=0; k<numPoints; k++) {
				phaseDiff = fmod(fabs(phase1[k] - phase2[k]), twoPI);
				if(phaseDiff > PI) phaseDiff = twoPI - phaseDiff;
				corr += weight[k]*phaseDiff;
				sumWeight += weight[k];
			}
			corr /= (sumWeight*PI);
			residual = corr;
			break;
			}
		case 2: {
			float corr=0, sumWeight=0;
			for(int k=0; k<numPoints; k++) {
				corr += (weight[k]*cos(phase1[k] - phase2[k]));
				sumWeight += weight[k];
				//printf("%f  %f  %f  %f\n",amplitude1[k], phase1[k]*180.0/PI, amplitude2[k], phase2[k]*180.0/PI);
			}
			corr /= sumWeight;
			residual = (1.0-corr)/2.0;
			//printf("sumWeight = %f, residual =%f\n",sumWeight, residual); exit(0);
			break;
			}
		case 3: {
			//normalized dot product r1*r2cos(phase1-phase2)/sqrt(sigma(r1^2)*sigma(r2^2))
			//normal return value is -1 to 1
			float corr=0;  //correlation coefficient
			float amp1, amp2;
			float amp11=0, amp12=0, amp22=0;


			for(int k=0; k<numPoints; k++) {
				//amplitude correlation p=sigma((r1i-avg(r1))*(r2i-avg(r2)))/sqrt(sigma(r1i-avg(r1)^2)*sigma(r2i-avg(r2))^2)
				//normal return value is 0--1, but it could be -1 --- 1
				amp1 = amplitude1[k]; amp2 = amplitude2[k];
				amp12 += (weight[k]*amp1*amp2*cos(phase1[k] - phase2[k]));
				amp11 += (weight[k]*amp1*amp1);
				amp22 += (weight[k]*amp2*amp2);
			}

			corr = amp12 / sqrt(amp11*amp22);

			residual = (1.0-corr)/2.0;
			break;
			}
		case 4: {
			//combined amplidue difference [ln(amp1)-ln(amp2)]and phase difference [1-cos(p1-p2)]
			//normal return value is 0 to infinite (0 is the best)

			float  sumWeight=0;
			residual=0;
			for(int k=0; k<numPoints; k++) {
				residual += (weight[k]*fabs(log(amplitude1[k]) - log(amplitude2[k]))*(1.0 - cos(phase1[k]-phase2[k]))/2.0);
	//			residual += (weight[k]*fabs(log(amplitude1[k]) - log(amplitude2[k])));
				sumWeight += weight[k];
			}

			residual /= sumWeight;
			break;
			}
		case 5: {
			float amp1, amp2, amp11, amp22, amp12;
			float deltaVect=0, sumAmp1Amp2=0, sumWeight=0;

			residual=0; sumAmp1Amp2=0; sumWeight=0;
			for(int k=0; k<numPoints; k++) {
				amp1  = amplitude1[k]; amp2  = amplitude2[k];
				amp11 = amp1*amp1; amp22 = amp2*amp2; amp12 = amp1*amp2;

				deltaVect = sqrt(amp11 + amp22 - 2*amp12*cos(phase1[k] - phase2[k]));
	//			deltaVect /= (amp1 + amp2);
				residual    += weight[k]*deltaVect;
				sumAmp1Amp2 += weight[k]*(amp1 + amp2);
				sumWeight   += weight[k];
			}

	//		residual /= sumWeight;
			residual /= sumAmp1Amp2;
			break;
			}
		case 6: {
			//normalized dot product r1*r2cos(phase1-phase2)/sqrt(sigma(r1^2)*sigma(r2^2))
			//normal return value is -1 to 1
			float phaseDiff=0, corr=0;  //correlation coefficient
			float amp1, amp2;
			float amp11=0, amp12=0, amp22=0;


			for(int k=0; k<numPoints; k++) {
				phaseDiff = fmod(fabs(phase1[k] - phase2[k]), twoPI);
				if(phaseDiff > PI) phaseDiff = twoPI - phaseDiff;
				phaseDiff = 1 - 2*phaseDiff/PI;
				amp1 = amplitude1[k]; amp2 = amplitude2[k];
				amp12 += (weight[k]*amp1*amp2*phaseDiff);
				amp11 += (weight[k]*amp1*amp1);
				amp22 += (weight[k]*amp2*amp2);
			}

			corr = amp12 / sqrt(amp11*amp22);
	//			printf("\n\n");exit(0);

			residual = (1.0-corr)/2;
			break;
			}
		default:
			fprintf(stderr,"ERROR: residualMode %d does NOT exist in function EMData::onePairCommonLinesPhaseResidualMode()\n", residualMode);
			return 99999.99;
		}//end switch(mode)
	return residual;
}


//Do over sample FFT will insert more samples between two integer spatial frequencies.
//It means the spatial frequecy could be fractional pixel number, the minimal interval is 1/scale
EMData *EMData::doOverSampleFFT(int scale) {
	if(scale<1) { printf("The EMData::doOverSampleFFT(int scale) can not accept scale<1\n"); return NULL; }
	int w = scale*nx;
	int h = scale*ny;
	int x0 = nx/2 - w/2;
	int y0 = ny/2 - h/2;
	EMData* temp = clip(x0, y0, w, h);
	EMData *tempFFT = temp->doFFT();
	tempFFT->toCorner();
	temp->gimmeFFT();
        delete temp;
	return tempFFT;
} // end doOverSampleFFT(int scale)



EMData *EMData::doOverSampleFFTShift(int scale, int xx, int yy) {
	if(scale<1) { printf("The EMData::doOverSampleFFT(int scale) can not accept scale<1\n"); return NULL; }
	int w = scale*nx;
	int h = scale*ny;
	int x0 = -xx + nx/2 - w/2;
	int y0 = -yy + ny/2 - h/2;
	EMData* temp = clip(x0, y0, w, h);
	EMData *tempFFT = temp->doFFT();
	tempFFT->toCorner();
	temp->gimmeFFT();
        delete temp;
	return tempFFT;
} // end doOverSampleFFT(int scale)





void EMData::addAsMask(EMData* mask2) // add another mask to current mask, overlapping conflict area set to 0
{
float *d1=getData(), *d2=mask2->getDataRO();
for(int i=0; i<nx*ny*nz; i++){
	if(d2[i]){
		if(d1[i] && d1[i]!=d2[i]) d1[i]=0;
		else d1[i]=d2[i];
	}
}
mask2->doneData();
doneData();
update();
}


void EMData::get_bounding_box(float thresh, int* xmin, int* ymin, int* zmin, int* xmax, int* ymax, int* zmax)
{
	float* dp = getDataRO();
	*xmin = nx; *ymin = ny; *zmin = nz;
	*xmax = 0;  *ymax = 0; *zmax = 0;
	for(int k=0; k<nz; k++) {
		for(int j=0; j<ny; j++) {
			for(int i=0; i<nx; i++) {
				if(dp[k*nx*ny + j*nx + i] >= thresh) {
					if(k>*zmax) *zmax = k;
					else if (k<*zmin) *zmin = k;
					if(j>*ymax) *ymax = j;
					else if (j<*ymin) *ymin = j;
					if(i>*xmax) *xmax = i;
					else if (i<*xmin) *xmin = i;
				}
			}
		}
	}
}

float EMData::ncccmp(EMData *noisy)	{// normalized correlation coefficient comparision (-1->1, larger better), ignore the 0-value pixels, only use non-zero pixels of this
float ret=0,*dx,*dy;
int i,n=nx*ny*nz;
float m=1.0,b=0.0;

// First we must determine the proper normalization for 'this' for optimal subtraction
dx=getDataRO();
dy=noisy->getDataRO();

lsqfit(n,dx,dy,&m,&b,2);

float sumab=0, suma2=0, sumb2=0;
ret=0;

for (int i=0; i<nx*ny*nz; i++) {
	if (dx[i]) {
		float t1 = dx[i] * m + b;
		sumab += t1 * dy[i];
		suma2 += t1*t1;
		sumb2 += dy[i] * dy[i];
	}
}
doneData();
noisy->doneData();

if(suma2 && sumb2) ret = sumab / sqrt(suma2 * sumb2);
else ret = -1.0;	// at least one map is empty, give it worst score

return ret;
}

#include <gsl/gsl_linalg.h>
// find a least square fit of plane to the image pixel values. i.e. find the gradient
EMData* EMData::leastSquarePlaneFit()
{
if (zSize()>1) {
	fprintf(stderr, "ERROR: EMData::leastSquarePlaneFit() only supports 2D images\n");
	exit(1);
}
float* d=getDataRO();
int count = 0;
for(int i=0; i<nx*ny; i++) {
	if(d[i]) count++;
}
if(count<3) {
	fprintf(stderr, "ERROR: EMData::leastSquarePlaneFit() requires at least 3 nonzero pixels\n");
	exit(1);
}
// Allocate the working space
gsl_vector *S=gsl_vector_calloc(3);
gsl_matrix *A=gsl_matrix_calloc(count,3);
gsl_matrix *V=gsl_matrix_calloc(3,3);

double m[3] = {0, 0, 0};
int index=0;
for(int j=0; j<ny; j++){
	for(int i=0; i<nx; i++){
		int ij=j*nx+i;
		if(d[ij]) {
			m[0]+=i;	// x
			m[1]+=j;	// y
			m[2]+=d[ij];	// z
			//printf("index=%d/%d\ti,j=%d,%d\tval=%g\txm,ym,zm=%g,%g,%g\n", \
			//        index,count,i,j,d[ij],xm/(index+1),ym/(index+1),zm/(index+1));
			index++;
		}
	}
}

for(int i=0; i<3; i++) m[i]/=count;	// compute center of the plane

index=0;
for(int j=0; j<ny; j++){
	for(int i=0; i<nx; i++){
		int ij=j*nx+i;
		if(d[ij]) {
			//printf("index=%d/%d\ti,j=%d,%d\tval=%g\n",index,count,i,j,d[index]);
			gsl_matrix_set(A,index,0,i-m[0]);
			gsl_matrix_set(A,index,1,j-m[1]);
			gsl_matrix_set(A,index,2,d[ij]-m[2]);
			index++;
		}
	}
}
doneData();

// SVD decomposition and use the V vector associated with smallest singular value as the plan normal
gsl_linalg_SV_decomp_jacobi(A, V, S);

double n[3];
for(int i=0; i<3; i++) n[i] = gsl_matrix_get(V, i, 2);

#ifdef DEBUG
printf("S=%g,%g,%g\n",gsl_vector_get(S,0), gsl_vector_get(S,1), gsl_vector_get(S,2));
printf("V[0,:]=%g,%g,%g\n",gsl_matrix_get(V,0,0), gsl_matrix_get(V,0,1),gsl_matrix_get(V,0,2));
printf("V[1,:]=%g,%g,%g\n",gsl_matrix_get(V,1,0), gsl_matrix_get(V,1,1),gsl_matrix_get(V,1,2));
printf("V[2,:]=%g,%g,%g\n",gsl_matrix_get(V,2,0), gsl_matrix_get(V,2,1),gsl_matrix_get(V,2,2));
printf("Fitted plane: p0=%g,%g,%g\tn=%g,%g,%g\n",m[0],m[1],m[2],n[0],n[1],n[2]);
#endif

EMData *ret = new EMData();
ret->setSize(nx, ny);

for(int j=0; j<nx; j++){
	for(int i=0; i<ny; i++){
		ret->setValueAt(i,j,0,-((i-m[0])*n[0]+(j-m[1])*n[1])/n[2]+m[2]);
	}
}
return ret;
}




