#ifndef _emhdf2_h_
#define _emhdf2_h_

#ifdef HDF5
#include <hdf5.h>
#include <string>

#include "EMData.h"
#include "emobject.h"

using std::string;

class EMhdf2 {
public:
	EMhdf2(const char* hdf_filename, const char* rw_mode="rw");
    ~EMhdf2();
    
    /** test whether this hdf file is new HDF5 or old HDF5 implementation
     * 
     * @param hdf_filename file name
     * @return 0 for new HDF5 file, -1 for old-style HDF5 file
     * */
    static int init_test(const char * hdf_filename);
    
    EMDataHeader * read_header(int image_index, Euler& euler);
    
    int read_data(float* data, int image_index = 0);
    
    int erase_header(int image_index);
    
    int write_header(const EMDataHeader & header, int image_index, const Euler& euler);
    
    int write_data(const float* const data, int image_index = 0);
    
    /** get the number of images in this file*/
    int get_num_dataset();
    
private:
	hid_t file;
	hid_t group;
	hid_t accprop;
	hid_t simple_space;
	string filename;   
	bool initialized;
	const char * rwmode;
	
	int write_attr(hid_t loc, const char *name, EMObject obj);
	EMObject read_attr(hid_t attr);
	
	void init();
};

#endif	//HDF5
#endif	//_emhdf2_h_