// CompVol.C   Copyright 2003   Shunming Fang		1/02/03
// This object contains index table(ny*nz array of int pointers) for a
//  3d EMdata. The real image data can be accessed by index.
// This objects can be used to do projection  
// efficiently in memory
#include <stdio.h>
#include <string.h>
//#include <iostream>
#include "VolTable.h"
#include "mrc.h"
#include "portable_fileio.h"
#include "Euler.h"
#include "byteorder.h"

VolTable::VolTable()
{
    nx = 0;
    ny= 0;
    nz = 0;
    r_ = 0;
    xmax= 0;
    rdata = 0;
    nrow = 0;
    npixel = 0;
}

VolTable::~VolTable()
{
    
    //  for (int i=0; i<ny * nz; i++) delete []  xmax[i];
    delete [] xmax;
    delete [] rdata;
}

void VolTable::setSize(int x, int y, int z)
{
    int i;
    nx = x;
    ny = y;
    nz = z;
    if(z==1)  // it is 2d image
    {
	cx = x/2;
	cy = y/2;
	cz = 0;
	if(nx<ny)
	    r_ = (nx/2-1);
	else
	    r_ = (ny/2-1);
    }
    else //it is 3d image
    {

	cx = x/2;
	cy = y/2;
	cz = z/2;
	if((nx<=ny)&&(nx<=nz))
	{
	    r_ = (nx/2-1);
	}
	else if((ny<=nx)&&(ny<=nz))
        {
	    r_ = (ny/2-1);
        }
        else
	    r_ = (nz/2-1);
    }
    //printf("ny*nz %d\n", ny*nz);
    xmax =(int*) new int[ny*nz*5];
    // printf("xmax:%d\n", xmax);
    // for(i = 0; i<ny*nz; )
    // {
    //	xmax[i] = new int[5];
	//	int j=0;
	//	for(j = 0; j<5;)
	//	{
	//   printf("xmax[%d][%d]: %d", i, j, xmax[i][j]);
	//    j=j+1;
	//	}
    //	i = i+1;
	//printf("i: %d", i);
    // }

}


void VolTable::zero()
{
    int i;
    int j;
    int ny_nz = ny*nz;
    // printf("xmax: %d\n", xmax);
    // printf("xmax+1: %d\n",xmax+1);
    // printf("this was call::ny_nz: %d\n", ny_nz);
    for (i=0; i<ny*nz*5;i++)
    {
	//	for(j=0; j<5; )
	//	{
	//    xmax[i][j] = 0;
	//    j=j+1;
	    // printf("j: %d", j);
	// }
	//	i =i+1;
	//printf("i: %d", i);
	//	}
	//	memset(xmax[i],0,sizeof(int)*5);
	//i= i+1;
	//	printf("i: %d", i);
	xmax[i] =0;
	
    }
    
}

float VolTable::setRowAndPixel(int ri,int dcx,int dcy,int dcz)
{
    int i, j, k;
    
    float dm, sum_1,sum_2,dist;
    int r;
    int first = 1;

    if(ri == 0) // use default radius
    {
	r = r_*r_;
    }
    else
    {
	//	if(ri>r_) //radius > default, partial info lost
	//	{
	//    printf("WARNING! radius is greater than box, Information will be possibly lost!\n");
	//    r = r_*r_;
	//	}
	//	else
        {
	    r_ = ri;
	    r = ri*ri;
	}
    }
    zero();
    //    nrow = 0;

    if(dcx != 0 || dcy != 0 || dcz != 0)
    {
	//	if(dcx <0 || dcy <0 || dcz<0||dcx >=nx ||dcy>=ny ||dcz >=nz)
	//	{
	//    printf("WARNING! the cent point is out of the box\n");
	//	}
        cx = dcx;
        cy = dcy;
        cz = dcz;
    }
    for(i = 0; i<nz; i++)
    {
	dm = i- cz;
        sum_1 = dm *dm;
        for( j = 0; j<ny; j++)
        {
	    dm = j - cy;
	    sum_2 = sum_1 + dm *dm;
	    first = 1;
		//   pTable = getPterAt(j, i);
	    for( k = 0; k < nx; k++)
	    {
		dm = k - cx;
	        dist = sum_2 + dm*dm;
	        if(first)
	        {
		    if(dist > r)
		        continue;
		    else
		    {
		        first = 0;
			npixel +=1;
			nrow +=1;
		        xmax[(nrow-1)*5+0] = k;
		        xmax[(nrow-1)*5+1] = j;
			xmax[(nrow-1)*5+2] = i;
			xmax[(nrow-1)*5+3] = npixel;
			xmax[(nrow-1)*5+4] = npixel;
		    }
		}
	        else
	        {
		    if(dist > r)
		        break;
		    else
		    {
			npixel +=1;
			xmax[(nrow-1)*5+4] = npixel;
		    }
		}

	    }

	}

    }
    return r;

}


void VolTable::setVolTable(EMData* data_in,int dcx,int dcy,int dcz,int ri)
{
    //    int cx,cy,cz;
    int i, j, k;
    float dm, sum_1,sum_2,dist;
    int r;
    int first = 1;
    float *din;

    if(data_in == 0)//no real EM data
    {
	printf("WARNING:No EMANData was given\n");
	return;
    }

    if(ri <= 0) // use default radius
    {
	//	printf("ri = 0 and r_ is %d\n", r_);
	r = r_*r_;

    }
    else
    {
       
	r_ = ri;
	r = ri*ri;
    }
    // printf("zero() will call,  %d, %d\n", sizeof(int), sizeof(char));
    zero();
    // printf("zero() was call, %d\n", sizeof(int));

    if(dcx != 0 || dcy != 0 || dcz != 0)
    {
        cx = dcx;
        cy = dcy;
        cz = dcz;
    }
    //volTable.setSize(nx,ny,nz);
    for(i = 0; i<nz; i++)
    {
	dm = i- cz;
        sum_1 = dm *dm;
        for( j = 0; j<ny; j++)
        {
	    dm = j - cy;
	    sum_2 = sum_1 + dm *dm;
	    first = 1;
		//   pTable = getPterAt(j, i);
	    for( k = 0; k < nx; k++)
	    {
		dm = k - cx;
	        dist = sum_2 + dm*dm;
	        if(first)
	        {
		    if(dist > r)
		        continue;
		    else
		    {
		        first = 0;
			npixel +=1;
			nrow +=1;
		        xmax[(nrow-1)*5+0] = k;
		        xmax[(nrow-1)*5+1] = j;
			xmax[(nrow-1)*5+2] = i;
			xmax[(nrow-1)*5+3] = npixel;
			xmax[(nrow-1)*5+4] = npixel;
		    }
		}
	        else
	        {
		    if(dist > r)
		        break;
		    else
		    {
			npixel +=1;
			xmax[(nrow-1)*5+4] = npixel;
		    }
		}
	    }
	}
    }
    rdata = new float[npixel];
    din = data_in->getDataRO();
    k = 0;
    for(i = 0; i<nrow; i++)
    {
	for(j = 0; j< (xmax[i*5+4]-xmax[i*5+3]+1);j++)
	{
	    rdata[k] = din[xmax[i*5+0]+j +xmax[i*5+1]*nx +xmax[i*5+2]*ny*nx];
	    ++k;
	}
    }
    data_in->doneData();
     
}

void VolTable::setTableFromMRC(char* fspec, int ri)
{
     FILE *in;
     int i,j,k;
     float dm, sum_1,sum_2,dist;
     int r = 0;
     int pixels = 0;
     int first = 1;
     int ord=0;	// 1 if the file order is MSB first
     int mord=0; // 1 if the machine order is MSB first
     float f,a,p;
     char s[800],*m2,w;
     mrcH* mrch;
     unsigned char *cbuf,temp;
     short *sbuf;
     float *fbuf;
     
     
     int startz = 0;
     static char *str[] = { "char", "short", "float", "short complex", "float complex" };
     static char *str2[] = { "" , "byte reversed" , "" };
     static char *str3[] = { "" , "compressed" };

#ifdef DEBUG
     printf("MRC open '%s'\n",fspec);
#endif
     //      mrch=new mrcH;
     if (fspec==NULL)
     {
	 printf("No image file.Please specify one.\n");
	 return;
     }
     else
     {
	 in=fopen(fspec,"rb");
	 if (in==NULL)
	 {
	     printf("No such image file.\n");
	     return ;
	 }
     }
     mrch = new mrcH;

     if (!fread(mrch,sizeof(mrcH),1,in))
     {
	 delete mrch;
	 return ;
     }

     mrch->labels[0][79]=0;
     if (ByteOrder::is_machine_big_endian()) {
	 mord=1;
     }
     else {
	 mord=0;
     }

     m2=(char *)mrch;
     if (m2[0]==0 && m2[1]==0)
	 ord=1;
     if (mord ^ ord)
     {

	 for (i=0; i<(27+MRC_USER)*4; i+=4)
	 {
	     w=m2[i]; m2[i]=m2[i+3]; m2[i+3]=w;
	     w=m2[i+1]; m2[i+1]=m2[i+2]; m2[i+2]=w;
	 }
     }

     if (mrch->ny>0xffff || mrch->nz>0xffff || mrch->mode>4 || mrch->nlabl>11)
     {
	 //	 setSize(10,10,1);
	 printf("Invalid MRC file\n");
	 // zero();
	 delete mrch;
	 return ;
     }

  //printf("ord=%d mord=%d\n",ord,mord);

     if (mrch->ny==1)
     {
	 printf("This appears to be a 1d file. I only read 2d files.\n");
	 delete mrch;
	 return ;
     }
     if (mrch->nz==1)
	 sprintf(s,"Read a %s %dx%d 2D %s MRC File\n",
			   str2[ord+mord],nx,ny,str[mrch->mode]);
     else sprintf(s,"Read a %dx%dx%d 3D MRC File\n",nx,ny,nz);

     setSize(mrch->nx,mrch->ny,mrch->nz);//set the size for image
					 //object
     //  if(mrch->xorigin != 0 || mrch->yorigin != 0)
     //{
     // zorigin = (mrch->nz)/2;
     // }
     if (mrch->xlen==0) mrch->xlen=1.0;
     if (mrch->ylen==0) mrch->ylen=1.0;
     if (mrch->zlen==0) mrch->zlen=1.0;
     r= setRowAndPixel(ri);
     if(1)
     {
	 portable_fseek(in,sizeof(mrcH)+mrch->nsymbt,SEEK_SET);
	 switch(mrch->mode)
	 {
	 case MODE_char:
	     rdata = new float[npixel];
	     cbuf = new unsigned char[nx];
	     for(i = 0; i<nz; i++)
	     {
		 dm = i- cz;
		 sum_1 = dm *dm;
		 for( j = 0; j<ny; j++)
		 {
		     dm = j - cy;
		     sum_2 = sum_1 + dm *dm;
		     first = 1;
		     //	portable_fseek(in,startz*nx*sizeof(char),SEEK_CUR);
		     // startz = 1;
		     if ((fread(cbuf,sizeof(unsigned char),nx,in))!=nx)
		     {
		         sprintf(s,"Incomplete data read %d blocks\n",i*ny+j);
			 error(s,ERR_WARN);
		      }

		     for( k = 0; k < nx; k++)
		     {
			 dm = k - cx;
			 dist = sum_2 + dm*dm;
			 if(first)
			 {
			     if(dist > r)
			         continue;
			     else
			     {
				 first = 0;
				 // if ((fread(cbuf,sizeof(char),nx,in))!=nx)
				 //{
				 //  sprintf(s,"Incomplete data read %d/%d blocks\n",i,ny*nz);
				 //  error(s,ERR_WARN);
			         //}
				 rdata[pixels] =(float)cbuf[k]/100-1.28;
			         pixels +=1;
				 //				 startz=0;
			     }
			 }
			 else
			 {
			     if(dist > r)
			         break;
			     else
			     {
			         rdata[pixels] =(float)cbuf[k]/100-1.28;
			         pixels +=1;
			     }
			 }
		     }
		 }
	     }
	     delete [] cbuf;
	     break;
	 case MODE_short:

#ifdef DEBUG
	     printf("Short file read\n");
#endif
	     rdata = new float[npixel];
	     sbuf = new short[nx];
	     cbuf = (unsigned char*)sbuf;
	     for(i = 0; i<nz; i++)
	     {
		 dm = i- cz;
		 sum_1 = dm *dm;
		 for( j = 0; j<ny; j++)
		 {
		     dm = j - cy;
		     sum_2 = sum_1 + dm *dm;
		     first = 1;
		     // portable_fseek(in,startz*nx*sizeof(short),SEEK_CUR);
		     // startz = 1;
		     if((fread(cbuf,sizeof(short),nx,in))!=nx)
		     {
		         sprintf(s,"Incomplete data read %d blocks\n",i*ny+j);
			 error(s,ERR_WARN);
		     }
		     for( k = 0; k < nx; k++)
		     {
			 dm = k - cx;
			 dist = sum_2 + dm*dm;
			 if(first)
			 {
			     if(dist > r)
			         continue;
			     else
			     {
				 first = 0;
				 //if ((fread(cbuf,sizeof(short),nx,in))!=nx)
				 //{
				 //  sprintf(s,"Incomplete data read %d/%d blocks\n",i,ny*nz);
				 //  error(s,ERR_WARN);
				 // }
				 if ((ord+mord)&1)
				 {
				     temp=cbuf[k*2];
				     cbuf[k*2]=cbuf[k*2+1];
				     cbuf[k*2+1]=temp;
				     rdata[pixels]=(float)(sbuf[k]);
				 } 
				 else
				     rdata[pixels]=(float)(sbuf[k]); 
			         pixels +=1;
				 // startz =0;
			     }
			 }
			 else
			 {
			     if(dist > r)
			         break;
			     else
			     {
				if ((ord+mord)&1)
				 {
				     temp=cbuf[k*2];
				     cbuf[k*2]=cbuf[k*2+1];
				     cbuf[k*2+1]=temp;
				     rdata[pixels]=(float)(sbuf[k]);
				 } 
				 else
				     rdata[pixels]=(float)(sbuf[k]); 
			   	 pixels +=1;
			     }
			 }
		     }
		 }
	     }
	     ;
	     delete [] sbuf;
#ifdef DEBUG
	     printf("Short read complete\n");
#endif
	     break;
	 case MODE_float:
	     // portable_fseek(in,nz*nx*ny*sizeof(float),SEEK_CUR);
	     rdata = new float[npixel];
	     fbuf = new float[nx];
	     cbuf = (unsigned char*) fbuf;
	     for(i = 0; i<nz; i++)
	     {
		 dm = i- cz;
		 sum_1 = dm *dm;
		 for( j = 0; j<ny; j++)
		 {
		     dm = j - cy;
		     sum_2 = sum_1 + dm *dm;
		     first = 1;
		     // portable_fseek(in,sizeof(mrcH)+mrch->nsymbt+startz*nx*sizeof(float),SEEK_SET);
		     // portable_fseek(in,startz*nx*sizeof(float),SEEK_CUR);
		     // startz = 1;
		     if ((fread(cbuf,sizeof(float),nx,in))!=nx)
		     {
		         sprintf(s,"Incomplete data read %d blocks\n",i*ny+j);
			 error(s,ERR_WARN);
				     
		     }
		     for( k = 0; k < nx; k++)
		     {
			 dm = k - cx;
			 dist = sum_2 + dm*dm;
			 if(first)
			 {
			     if(dist > r)
			         continue;
			     else
			     {
				 first = 0;
				 // if ((fread(cbuf,sizeof(float),nx,in))!=nx)
				 //{
				 //  sprintf(s,"Incomplete data read %d/%d blocks\n",i,i*ny+j);
				 //  error(s,ERR_WARN);
				     
				 // }
				 if (ord+mord==1)
				 {
				      temp=cbuf[k*4];
				      cbuf[k*4]=cbuf[k*4+3];
				      cbuf[k*4+3]=temp;
				      temp=cbuf[k*4+1];
				      cbuf[k*4+1]=cbuf[k*4+2];
				      cbuf[k*4+2]=temp;
				 }
				 rdata[pixels] = fbuf[k];
			         pixels +=1;
				 // startz = 0;
			     }
			 }
			 else
			 {
			     if(dist > r)
			         break;
			     else
			     {
				  if (ord+mord==1)
				  {
				      temp=cbuf[k*4];
				      cbuf[k*4]=cbuf[k*4+3];
				      cbuf[k*4+3]=temp;
				      temp=cbuf[k*4+1];
				      cbuf[k*4+1]=cbuf[k*4+2];
				      cbuf[k*4+2]=temp;
				  }
			         rdata[pixels] = fbuf[k];
			         pixels +=1;
			     }
			 }
		     }
		 }
	     }
	     delete [] fbuf;
	     
	     break;
	 default: 
	     printf("Bad MRC file\n"); 
	     delete mrch;
	     return ;
	 }
     }

     delete mrch;
     return ;
}

EMData * VolTable::project3d(float alt,float az,float phi)
{

    Euler eu_Ang;
    int i, j, k;// nn;
    float xb, yb;
    // int cx, cy, cz;
    int iqx,iqy, iloc;
    float dipx, dipy, dipy1m, dipx1m;
    float *dm;
    float *dat;
    // float *din;
    float trueData;
    int *pInt;
    // int    sx,sy,sz;
    EMData* ret;
    //if(!vol_table)
    // {

    //  vol_table = new VolTable;
	 //	 dataTable = (VolTable*)vol_table;
    //if(fsp)
    //{
    //    vol_table->setTableFromMRC(fsp,ri);
    //}
    //else
    //{
    //    vol_table->setSize(nx, ny, nz);
    //    vol_table->setVolTable(this,dcx, dcy, dcz, ri);
    //}
    // }
    //   else
    //vol_table = (VolTable*)vol_table;
    // cx = vol_table->getCXSize();
    // cy = vol_table->getCYSize();
    // cz = vol_table->getCZSize();
    // sx = vol_table->getXSize();
    // sy = vol_table->getYSize();
    // sz = vol_table->getZSize();
    eu_Ang.setAngle(alt, az, phi);
    ret = new EMData;
    dm = eu_Ang.mat();
    ret->setSize(nx,ny,1);
    ret->zero();
    dat = ret->getData();
    //  din = getData();
    //   nn = getNoOfRow();
    //    if((cx+r_)> nx ||(cx-r_)<0 || (cy+r_)> ny ||(cy-r_)<0)
    if((2*r_+1) >nx || (2*r_+1)>ny)// need to boundary check
    {
	for(i = 0; i<nrow; i++)
	{
	    // pInt = getPterAt(i);
	    xb = (xmax[i*5+0]-cx)*dm[0] +(xmax[i*5+1]-cy)*dm[1] + (xmax[i*5+2]-cz)*dm[2] +cx;
	    yb = (xmax[i*5+0]-cx)*dm[3] +(xmax[i*5+1]-cy)*dm[4] + (xmax[i*5+2]-cz)*dm[5] +cy;
	    for(k = xmax[i*5+3]-1; k < xmax[i*5+4]; k++)
	    {
		iqx =(int) floor(xb);
		if(iqx >(nx-1))//check boundary
		{
		    xb = xb + dm[0];
		    yb = yb + dm[3];
		    continue;
		}
		iqy =(int) floor(yb);
		if(iqy >(ny-1)) //check boundary
		{
		    xb = xb + dm[0];
		    yb = yb + dm[3];
		    continue;
		}
		    
		trueData = rdata[k];
		dipx = (xb - iqx);
		dipy = (yb - iqy) * trueData;
		dipy1m = (trueData - dipy);
		dipx1m = (1.0 - dipx);
		iloc = iqy*nx + iqx;
		dat[iloc] = dat[iloc] + dipx1m*dipy1m;
		dat[iloc+1] = dat[iloc+1] + dipx*dipy1m;
		dat[iloc+nx] = dat[iloc+nx] + dipx1m*dipy;
		dat[iloc+nx+1] = dat[iloc+nx+1]+ dipx*dipy;
		xb = xb + dm[0];
		yb = yb + dm[3];
	    }
	}

    }
    else
    {
	for(i = 0; i<nrow; i++)
	{
	    // pInt = getPterAt(i);
	    xb = (xmax[i*5+0]-cx)*dm[0] +(xmax[i*5+1]-cy)*dm[1] + (xmax[i*5+2]-cz)*dm[2] +cx;
	    yb = (xmax[i*5+0]-cx)*dm[3] +(xmax[i*5+1]-cy)*dm[4] + (xmax[i*5+2]-cz)*dm[5] +cy;
	    for(k = xmax[i*5+3]-1; k < xmax[i*5+4]; k++)
	    {
		iqx =(int) floor(xb);
		iqy =(int) floor(yb);
		trueData = rdata[k];
		dipx = (xb - iqx);
		dipy = (yb - iqy) * trueData;
		dipy1m = (trueData - dipy);
		dipx1m = (1.0 - dipx);
		iloc = iqy*nx + iqx;
		dat[iloc] = dat[iloc] + dipx1m*dipy1m;
		dat[iloc+1] = dat[iloc+1] + dipx*dipy1m;
		dat[iloc+nx] = dat[iloc+nx] + dipx1m*dipy;
		dat[iloc+nx+1] = dat[iloc+nx+1]+ dipx*dipy;
		xb = xb + dm[0];
		yb = yb + dm[3];
	    }
	}
    }
	
    //   doneData();
    ret->doneData();
    ret->update();
    return ret;
}
