#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#ifdef _WIN32
 #include "UnixToWin.h"
#else
 #include <unistd.h>
 #include <sys/wait.h>
 #include <sys/times.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
#endif

#include "util.h"
#include <sys/stat.h>
#include <sys/types.h>

#include <signal.h>

//#include "SHMEM.h"
#include <errno.h>
#include "portable_fileio.h"
#include "byteorder.h"

int FSsendlocal(int command, int datalen,const char *fsp,const char *data,int fileno=0);

int p_fetch;

void error(char *msg,int pri) {
switch (pri) {
case ERR_EXIT:
	printf("Fatal Error: %s\n",msg);
	break;
case ERR_CRIT:
case ERR_ERROR:
	printf("Error: %s\n",msg);
	break;
case ERR_WARN:
	printf("Warning: %s\n",msg);
	break;
case ERR_MSG:
case ERR_INFO:
	printf("Note: %s\n",msg);
	break;
case ERR_DEBUG:
	printf("Debug: %s\n",msg);
	break;
}
if (pri==ERR_EXIT) exit(1);
}

void error(int pri,char *msg) {
	error(msg,pri);
}

// almost guarentees a valid temporary name in the current directory
const char *tmpnam2() {
static char ret[80];

sprintf(ret,"tmp.%d",(int)frand(0,100000000.0));
if (!access(ret,F_OK)) sprintf(ret,"tmp.%d",(int)frand(0,100000000.0));
//if (access(ret,W_OK)) { printf("%s\n",ret); error(ERR_EXIT,"Cannot open temporary file for writing"); }

return ret;
}

// This will lock a file. If it's already locked it will wait 30 seconds
// then remove the old lock.
int file_lock_wait(FILE *file) {

#ifdef FILELOCK

#ifndef _WIN32			// WIN32 #ifndef statement start
#ifndef FILESERVER

if (!file) error(ERR_EXIT,"Tried to lock NULL file");

int fdes = fileno(file);
 

struct flock fl;
fl.l_type = F_WRLCK;
fl.l_whence = SEEK_SET;
fl.l_start = 0;
fl.l_len = 0;
fl.l_pid = getpid();
#ifdef SGI
fl.l_sysid = getpid();
#endif

if (fcntl(fdes,F_SETLKW,&fl)==-1) {
	printf("Error: file locking error (%d)! NFS problem?\n",errno);
	int i;
	for (i=0; i<5; i++) if (fcntl(fdes,F_SETLKW,&fl)!=-1) break; else sleep(1);
	if (i==5) error(ERR_EXIT,"Fatal file locking error");
}
return 0;

#endif
#endif				// WIN32 #ifndef statement stop
#endif				//  #ifdef FILELOCK ends here
return -1;													// now returns -1`to indicate nothing happened
}

int file_lock(FILE *file) {

#ifndef FILESERVER

#ifndef _WIN32			// WIN32 #ifndef statement start
#ifndef FILESERVER
	
if (!file) error(ERR_EXIT,"Tried to lock NULL file");

int fdes = fileno(file);
 
struct flock fl = { F_WRLCK, SEEK_SET, 0, 0, getpid() };

return fcntl(fdes,F_SETLK,&fl);

#endif
#endif														// WIN32 #ifndef statement stop
#endif				//  #ifdef FILELOCK ends here
return -1;													// now returns -1`to indicate nothing happened
}

// deprecated, no longer necessary
void file_unlock(FILE *file) {

}

long file_size(FILE* file) {
	long current = portable_ftell(file);
	portable_fseek(file,0,SEEK_END);
	long size = portable_ftell(file);
	portable_fseek(file,current,SEEK_SET);
	return size;
}


// used for parsing special delimited HTML files for generating
// custom sets of instructions
void addHTMLSection(char *sourcefile,FILE *out,char *secid) {
FILE *in;
int c;
char s[20],*H;
int i;

H=EMDIR();

in=fopen(sourcefile,"rb");
if (!in) return;

// rely on buffered IO to make this reasonably efficient
// first find the correct section
s[0]=0;
while (strcmp(s,secid)) {
//	printf("'%s' != '%s'\n",s,secid);
	while ((c=getc(in))!='$' && c!=EOF) ;
	if (c==EOF) return;

	i=0;
	while ((c=getc(in))>' ' && i<10) { s[i]=c; i++; }
	s[i]=0;
}

// dump the section to the output stream
while ((c=getc(in))!='$' && c!=EOF) {
	if (c=='@') {
		c=getc(in);
		if (c=='E') fprintf(out,"%s",H);
		else if (c=='L') {
			char *cw=getcwd(NULL,0);
			if (cw) {
				fprintf(out,"%s",cw);
				free(cw);
			}
		}
		else putc(c,out);
	}
	else putc(c,out);
}

fclose(in);

}

// This returns the path to the EMAN directory
char *EMDIRC = NULL;	//DON'T USE THIS

char *EMDIR() {
if (EMDIRC) return EMDIRC;

if (getenv("EMANDIR")) {
	EMDIRC=(char *)malloc(strlen(getenv("EMANDIR"))+1);
	strcpy(EMDIRC,getenv("EMANDIR"));
	if (access(EMDIRC,F_OK)) error(ERR_EXIT,"The location specified by $EMANDIR does not exist");
}
else {
#ifdef _WIN32
	EMDIRC="C:\EMAN";
	if (access(EMDIRC,F_OK)) error(ERR_EXIT,"C:\EMAN does not exist, please install your EMAN to this location");
#else
	if (getenv("HOME")) { 
		EMDIRC=(char *)malloc(strlen(getenv("HOME"))+6);
		strcpy(EMDIRC,getenv("HOME"));
		strcat(EMDIRC,"/EMAN");
	}
	else {
		EMDIRC="/usr/local/EMAN/";
	}
	if (access(EMDIRC,F_OK)) error(ERR_EXIT,"Neither $HOME/EMAN or /usr/local/EMAN exist, please set EMANDIR to point to your EMAN installation");
#endif
}
return EMDIRC;
}

// All applications should call this before parsing the command line
void appinit(int argc,char *argv[]) {

srandom(time(0));

if (argc>1 && (strcmp(argv[1],"help")==0 || strcmp(argv[1],"-help")==0 ||
strcmp(argv[1],"--help")==0)) {
	FILE *rh;
	char s[256],*t;
	t=strrchr(argv[0],'/');
	if (t) t++; else t=argv[0];

/*	if (rh=DFopenr("remotehelp")) {
		fclose(rh);
		sprintf(s,"netscape -remote 'openURL(http://ncmi.bcm.tmc.edu/~stevel/EMAN/doc/progs/%s.html,new-window)'",t);
	}
	else sprintf(s,"netscape -remote 'openFile(%s/doc/progs/%s.html,new-window)'",EMDIR(),t);

	if (run(s)) {
		printf("Sorry, netscape doesn't appear to be running on your remote display.\nCheck the $DISPLAY variable.\n");
		
	}*/
	sprintf(s,"QHelp %s/doc/progs/%s.html",EMDIR(),t);
	run(s);

	appusage(argv[0]);
	exit(0);
}

#ifdef FILESERVER
FILE *in = DFopenr(".key");
if (!in) {
	in=DFopenw(".key");
	fprintf(in,"%d\n",random());
	fclose(in);
	in=DFopenr(".key");
}
fscanf(in,"%d",&FSKEY);
fclose(in);

for (int i=1; i<argc; i++) {
	if (strncmp("usefs=",argv[i],6)==0) sscanf(&argv[i][6],"%hu,%u",&FSPORT,&FSHOST);
}
if (FSHOST) {
	FSSOCK=sock_open(FSHOST,FSPORT);
	int i=0;
	while (FSSOCK < 0 && i < 10)
	{
		sleep(1);
		FSSOCK=sock_open(FSHOST,FSPORT);
		i++;
	        if (i>1) {
        	        char s[120];
			gethostname(s,119);
			printf("%s:Retry socket %d\n",s,i);
        	}
	}
	if (FSSOCK<0) { printf("%hu,%x\n",FSPORT,FSHOST); error(ERR_EXIT,"No socket to fileserver"); }
	int one=1;

//	setsockopt(FSSOCK,SOL_TCP,TCP_NODELAY,&one,sizeof(int));

}
else FSSOCK=0;
#endif


#ifdef EMVERD
printf("EMAN %sdbg.\nRun '%s help' for detailed help.\n",EMVER,argv[0]);
#else
printf("EMAN %s.\nRun '%s help' for detailed help.\n",EMVER,argv[0]);
#ifdef FILESERVER
if (FSHOST) printf("\nUsing Fileserver, %x\n",FSHOST);
fflush(stdout);
#endif	//FILESERVER
#endif

}

void appusage(char *argv0) {
char s[2048],*t;

t=strrchr(argv0,'/');
if (t) t++; else t=argv0;

sprintf(s,"%s/doc/usage/%s.usg",EMDIR(),t);

 FILE* in = fopen(s, "rb");
 if (!in) {
	 fprintf(stderr, "Error: cannot open file '%s'\n", s);
	 exit(1);
 }

 char line[4096];
 while(fgets(line, 4096, in)) {
	 fprintf(stderr, "%s", line);
 }
 fclose(in);

exit(1);
}

int  LOGbegin(int argc,char *argv[],int ppid) {
FILE *out,*in;
time_t tm=time(0);
int ref=getpid(),fl=0;
char *t;
//char *c=(char *)getcwd(NULL,1024);
char *c=getenv("PWD");

if (!c) { c=(char *)getcwd(NULL,1024); fl=1; }

// write the actual log entry
if (strrchr(argv[0],'/')) t=strrchr(argv[0],'/')+1;
else t=argv[0];

char s[2048];	//perhaps dangerous
sprintf(s,"%d\t%d\t%d\t%d\t%s",ref,tm,LOG_BEGIN,ppid?ppid:getppid(),t);
for (int i=1; i<argc; i++) sprintf(s+strlen(s)," %s",argv[i]);
sprintf(s+strlen(s),"\n");

#ifdef FILESERVER
if (FSHOST) { 
	FSsend(FS_TEXT,strlen(s)+1, ".emanlog", s);
	return ref;		// for remote logging, we skip the dirlist 
}
#endif

out=fopen(".emanlog","ab");
if (!out) return 0;
file_lock_wait(out);
fprintf(out,"%s",s);
file_unlock(out);
fclose(out);

// add the current directory to the directory list in ~/.eman
in=DFopenr("dirlist");
if (in) {
	char s[1024];
	int f=0;
	while (fscanf(in," %1023s",s)==1) {
		if (strcmp(s,c)==0) { f=1; break;}
	}
	fclose (in);
	if (!f) in=NULL;
}

if (!in) {
	out=DFopena("dirlist");
	if (out) {
		fprintf(out,"%s\n",c);
		fclose(out);
	}
}
if (fl) free(c);

return ref;
}

void LOG(int ref,char *file,int mode,char *text) {
FILE *out;
time_t tm=time(0);

if (!text) text="";
if (!file) file="-";

// if we reference a file in a remote directory, put a link in to the
// current directory
if (mode>=10 && mode<=19) {
	if (strrchr(file,'/')) {
		char *f2=(char *)malloc(strlen(file)+8);
		char *c=(char *)getcwd(NULL,1024);
		strcpy(f2,file);
		strcpy(strrchr(f2,'/')+1,".emanlog");
		out=fopen(f2,"ab");
		if (out) {
			fprintf(out,"%d\t%d\t%d\t%s\t%s\n",ref,tm,mode+10,strrchr(file,'/')+1,c);
			fclose(out);
		}
		free(f2);
		free(c);
	}
}

char s[2048];
sprintf(s,"%d\t%d\t%d\t%s\t%s\n",ref,tm,mode,file,text);

#ifdef FILESERVER
if (FSHOST) { 
	FSsend(FS_TEXT,strlen(s)+1, ".emanlog", s);



	return;		// for remote logging, we skip the dirlist 
}
#endif

out=fopen(".emanlog","ab");

if (out) {
	file_lock_wait(out);
	fprintf(out,"%s",s);
	file_unlock(out);
	fclose(out);
}

}

void LOGerror(int ref,int err,char *text) {
FILE *out;
time_t tm=time(0);

if (!text) text="";

char s[2048];
sprintf(s,"%d\t%d\t%d\t%d\t%s\n",ref,tm,LOG_ERROR,err,text);

#ifdef FILESERVER
if (FSHOST) { 
	FSsend(FS_TEXT,strlen(s)+1, ".emanlog", s);
	return;		// for remote logging, we skip the dirlist 
}
#endif

out=fopen(".emanlog","ab");
if (!out) return;
file_lock_wait(out);

fprintf(out,"%s",s);

file_unlock(out);
fclose(out);
}


// return a random number between lo and hi
float frand(float lo, float hi)
{
static int f=1;

if (f) { srandom(time(0)); f=0; }

return ((float)random()/2147483647.0*(hi-lo)+lo);
}

// interpolate x,y at x=xl, x must increase
float interpolate(float xl,int n,float *x,float *y) {
int i,st;
st=(int)floor(n*(xl-x[0])/(x[n-1]-x[0]));
if (st>n-2) st=n-2;

// now we set i such that st straddles x[i], x[i+1]
if (x[st]<=xl && x[st+1]>=xl) i=st;
else if (x[st]>xl) { for (i=st; i>=0 && x[i]>xl; i--); }
else { for (i=st; i<n-1 && x[i]<xl; i++) ; i--; }

// outside range, return terminal value
if (x[i]>xl) { printf("Range exceeded %f, %f to %f\n",xl,x[i],x[i+1]); return y[0]; }
if (x[i+1]<xl) { printf("Range exceeded %f, %f to %f\n",xl,x[i],x[i+1]); return y[n-1]; }

float f=(xl-x[i])/(x[i+1]-x[i]);
return f*y[i+1]+(1-f)*y[i];
}


// this will interpolate a peak location. v3 is considered to be at x=0
// The other points are nearest neighbors
// returns 0 if not well defined
float p5peak(float v1, float v2,float v3, float v4,float v5) {
float r;

if (v2==v4) return 0;

if (v2<v4) {
	r=(2*v4-v5-v3)/(v3+v4-v2-v5);
	if (r<0||r>1.0) return 0;
//printf("%f\n",r);
	return r;
}

r=(2*v2-v1-v3)/(v4+v1-v3-v2);
if (r<-1.0 || r>0) return 0;
//printf("%f\n",r);
return r;
}

float grand(float mean,float sigma)
{
float x,y,r,f;

do {
	x=frand(-1.0,1.0);
	y=frand(-1.0,1.0);
	r=x*x+y*y;
} while (r>1.0||r==0);
f=sqrt(-2.0*log(r)/r);

return x*f*sigma+mean;
}


FILE *DFopenr(const char *name) {
char s[MAXPATHLEN];

char *home=NULL;
#ifdef WIN32
home = "USERPROFILE";
#else
home = "HOME";
#endif

if (!name) { printf("Attempt to open null file\n"); return NULL; }
if (!getenv(home)) { printf("$%s not set!\n", home); exit(1); }
sprintf(s,"%s/.eman/%s",getenv(home),name);

return fopen(s,"rb");
}

FILE *DFopena(const char *name) {
char s[MAXPATHLEN];
FILE *ret;

char *home=NULL;
#ifdef WIN32
home = "USERPROFILE";
#else
home = "HOME";
#endif

if (!name) { printf("Attempt to open null file\n"); return NULL; }
if (!getenv(home)) { printf("$%s not set!\n", home); exit(1); }
sprintf(s,"%s/.eman",getenv(home));
if (access(s,F_OK)) mkdir(s,0xffff);

sprintf(s,"%s/.eman/%s",getenv(home),name);
ret=fopen(s,"ab");

return ret;
}

FILE *DFopenw(const char *name) {
char s[MAXPATHLEN];
FILE *ret;

char *home=NULL;
#ifdef WIN32
home = "USERPROFILE";
#else
home = "HOME";
#endif

if (!name) { printf("Attempt to open null file\n"); return NULL; }
if (!getenv(home)) { printf("$%s not set!\n", home); exit(1); }
sprintf(s,"%s/.eman",getenv(home));
if (access(s,F_OK)) mkdir(s,0xffff);

sprintf(s,"%s/.eman/%s",getenv(home),name);
ret=fopen(s,"wb");
chmod(s,00600);

return ret;
}

char *DFdir() {
char *s = (char *)malloc(MAXPATHLEN);

char *home=NULL;
#ifdef WIN32
home = "USERPROFILE";
#else
home = "HOME";
#endif

sprintf(s,"%s/.eman",getenv(home));
if (access(s,F_OK)) mkdir(s,0xffff);

sprintf(s,"%s/.eman",getenv(home));

return s;
}


char *readTextFile(const char *fsp, off_t *size) {

#ifdef FILESERVER
if (FSHOST&&FSSOCK) {
	int sock=FSSOCK;
	char path[MAXPATHLEN];
	getcwd(path,MAXPATHLEN-1);
	FSCOMMAND com;
	com.datalen=0;  // serialIn auto allocate memory, no need to tell server to allocate
	com.command=FS_READFILE;
	com.key=FSKEY;
	com.fileno=0;
	com.filelen=strlen(fsp);
	com.pathlen=strlen(path);
	com.headonly=0;
	writesock(sock,(char *)&com,sizeof(com));
	writesock(sock,(char *)path,strlen(path));
	writesock(sock,(char *)fsp,strlen(fsp));
	readsock(sock,(char *)size,sizeof(off_t));
	if (*size<1) return NULL;
	char *ret = (char *) malloc(*size+1);
	readsock(sock,ret,*size);
	return ret;
}
#endif

	
struct stat st;
*size=-1;
if (stat(fsp,&st)) return NULL;
*size=st.st_size;

FILE *in = fopen(fsp,"r");
if (!in) return NULL;
char *buf = (char *)malloc(*size+1);
fread(buf,*size,1,in);
fclose(in);
return buf;
}

// These routines are for approximate benchmarking of large code
// blocks. They are only accurate to 1 tic each time they are
// called, so the code blocks should each execute for a substantial
// fraction of a second
int *BM_USER = NULL;
int *BM_SYS = NULL;
int *BM_USERS = NULL;
int *BM_SYSS = NULL;

void benchmark_start(int tag) {
if (!BM_USER) BM_USER=(int *)calloc(1000,sizeof(int));
if (!BM_SYS) BM_SYS=(int *)calloc(1000,sizeof(int));
if (!BM_USERS) BM_USERS=(int *)calloc(1000,sizeof(int));
if (!BM_SYSS) BM_SYSS=(int *)calloc(1000,sizeof(int));

if (tag>999||tag<0) return;

#ifndef _WIN32												// WIN32 #ifndef statement start
struct tms T;
times(&T);
BM_USERS[tag]=T.tms_utime;
BM_SYSS[tag]=T.tms_stime;
#endif														// WIN32 #ifndef statement end
}

void benchmark_stop(int tag) {
if (tag>999||tag<0) return;

#ifndef _WIN32												// WIN32 #ifndef statement start
struct tms T;
times(&T);
BM_USER[tag]+=T.tms_utime-BM_USERS[tag];
BM_SYS[tag]+=T.tms_stime-BM_SYSS[tag];
#endif														// WIN32 #ifndef statement end
}

void benchmark_report() {
int i;
#ifndef _WIN32												// WIN32 #ifndef statement start
float tps = sysconf(_SC_CLK_TCK);

printf("Benchmark (%1.0f)\nTag\tUser\tSys\n",tps);
int tu=0,ts=0;
for (i=0; i<1000; i++) { tu+=BM_USER[i]; ts+=BM_SYS[i]; }
for (i=0; i<1000; i++) {
	if (BM_USER[i]>0 || BM_SYS[i]>0) 
		printf("%d\t%1.2f\t%1.2f\t%1.1f%%\t%1.1f%%\n",i,BM_USER[i]/tps,BM_SYS[i]/tps,BM_USER[i]*100.0/tu,BM_SYS[i]*100.0/ts);
}
#else
	printf("Benchmark not yet supported on this release of EMAN  \n");
#endif														// WIN32 #ifndef statement end
}

void roteulertb(float *alt,float *az,float *phi, float dx,float dy)
{
float mx1[3][3],mx2[3][3],mx3[3][3];
float a,t,c,s,x,y;
int i,j,k;

x=dy/hypot(dx,dy);	// axis
y=-dx/hypot(dx,dy);
a=-hypot(dx,dy)/100.0;	// angle
t=1.0-cos(a);
c=cos(a);
s=sin(a);


mx1[0][0]=(cos(*phi)*cos(*az)-cos(*alt)*sin(*az)*sin(*phi));
mx1[0][1]=-(sin(*phi)*cos(*az)+cos(*alt)*sin(*az)*cos(*phi));
mx1[0][2]=sin(*alt)*sin(*az);
mx1[1][0]=(cos(*phi)*sin(*az)+cos(*alt)*cos(*az)*sin(*phi));
mx1[1][1]=(sin(*phi)*sin(*az)+cos(*alt)*cos(*az)*cos(*phi));
mx1[1][2]=-sin(*alt)*cos(*az);
mx1[2][0]=sin(*alt)*sin(*phi);
mx1[2][1]=sin(*alt)*cos(*phi);
mx1[2][2]=cos(*alt);

mx2[0][0]=t*x*x+c;
mx2[0][1]=t*x*y;
mx2[0][2]=-s*y;
mx2[1][0]=t*x*y;
mx2[1][1]=t*y*y+c;
mx2[1][2]=s*x;
mx2[2][0]=s*y;
mx2[2][1]=-s*x;
mx2[2][2]=c;

for (i=0; i<3; i++) {
	for (j=0; j<3; j++) {
		mx3[i][j]=0;
		for (k=0; k<3; k++) {
			mx3[i][j]+=mx1[i][k]*mx2[k][j];
		}
	}
}
	

if (fabs(*alt)<.0001) {
	*phi=0;
	if (fabs(mx3[1][1])>1.0) *az=0;	// not a good thing
	else *az=acos(mx3[1][1]);
}
else {
	*az=atan2(mx3[0][2],-mx3[1][2]);
	*phi=atan2(mx3[2][0],mx3[2][1]);
}

*alt=atan2(sqrt(mx3[2][0]*mx3[2][0]+mx3[2][1]*mx3[2][1]),mx3[2][2]);


if (az!=az || phi!=phi) {
printf("%f %f %f\n",mx1[2][0]*mx2[0][2]+mx1[2][1]*mx2[1][2]+mx1[2][2]*mx2[2][2],
	mx1[2][0]*mx2[0][0]+mx1[2][1]*mx2[1][0]+mx1[2][2]*mx2[2][0],
	mx1[0][0]*mx2[0][2]+mx1[0][1]*mx2[1][2]+mx1[0][2]*mx2[2][2]);
az=phi=0;
}

return;
}

void swapdouble(char* d, int n)
{
    unsigned char s;
    int p = sizeof(double);
    
    for (int i = 0; i < n; i++, d+=p) {
	s = d[0]; d[0] = d[7]; d[7] = s;
	s = d[1]; d[1] = d[6]; d[6] = s;
	s = d[2]; d[2] = d[5]; d[5] = s;
	s = d[3]; d[3] = d[4]; d[4] = s;
    }
}


void swapint(char *d,int n)
{
int i;
unsigned char s;

for (i=0; i<n; i++,d+=4) {
	s=d[0]; d[0]=d[3]; d[3]=s;
	s=d[1]; d[1]=d[2]; d[2]=s;
}
}

void swapshort(char *d,int n)
{
int i;
unsigned char s;

for (i=0; i<n; i++,d+=2) {
	s=d[0]; d[0]=d[1]; d[1]=s;
}
}

int npfa(int low) { return low; }

// finds fft sizes with good primes. needs some work ...
int bestfftsize(int low) {
static char *valid = NULL;	//array containing valid sizes <1024 for speed

if (!valid) {
	valid=(char *)calloc(8192,1);
	
	for (float i2=1; i2<13.0; i2+=1.0) {
		for (float i3=0; i3<8.0; i3+=1.0) {
			for (float i5=0; i5<6.0; i5+=1.0) {
				for (float i7=0; i7<5.0; i7+=1.0) {
					float f=pow((float)2.0,i2)*pow((float)3.0,i3)*pow((float)5.0,i5)*pow((float)7.0,i7);
					if (f>8191.0) continue; 
					int n=(int)f;
					valid[n]=1;
				}
			}
		}
	}
}

for (int i=low; i<8192; i++) if (valid[i]) return i;

error(ERR_EXIT,"Sorry, can only find good fft sizes up to 8192 right now.");
return 1;
}

// The new fft routine can handle any size (although odd will make other things crash)
int isvalidfft(int low) { return 1; }

// the new fft library can do any size, so these are obsolete
/*
int npfa(int low)
{
while (!isvalidfft(low)) low++;
return low;
}

int isvalidfft(int low)
{
int v[] = { 2,3,5,7,11,13,17,19 };
int i;

if (low&1) return 0;
low/=2;
while (low!=1) {
        for (i=0; i<8; i++) {
                if (low%v[i]==0) { low/=v[i]; break; }
        }
        if (i==8) return 0;
}
return 1;
}
*/

// least squares fit returns slope, m, and intercept, b
// if ign0 = 1, ignores all zero values for x and y
// if ign0 = 2, ignores all zero values for x only
void lsqfit(int n,float *dx,float *dy,float *m,float *b,int ign0) {
double S=0,Sx=0,Sy=0,Sxx=0,Sxy=0;
int i;
for (i=0; i<n; i++) {
	if (ign0==2&&dx[i]==0) continue;
	else if (ign0==1&&(dx[i]==0||dy[i]==0)) continue;
	double y=dy[i];
	double x;
	if (dx) x=dx[i]; else x=i;
	Sx+=x;
	Sy+=y;
	Sxx+=x*x;
	Sxy+=x*y;
	S++;
}

double del=S*Sxx-Sx*Sx;
if (del==0) del=.0000001;
*b=(Sxx*Sy-Sx*Sxy)/del;
*m=(S*Sxy-Sx*Sy)/del;
}

int split(const char* str, vector<string>& results, const char* delim, bool empties)
{
	char* pstr = const_cast<char*>(str);
	char* r = NULL;
	if (delim) r = strstr(pstr, delim);
	else {
		r = NULL;
		for(int i=0; i<strlen(pstr); i++) {
			if(isspace(pstr[i])) {
				r = &pstr[i];
				break;
			}
		}
	}
	int dlen = 1;
	if (delim) dlen = int(strlen(delim));	// Cast used to avoid compiler warning

	while( r != NULL )
	{
		char* cp = new char[(r-pstr)+1];
		memcpy(cp, pstr, (r-pstr));
		cp[(r-pstr)] = '\0';

		if( strlen(cp) > 0 || empties )
		{
			string s(cp);
			results.push_back(s);
		}

		delete[] cp;
		pstr = r + dlen;
		if (delim) r = strstr(pstr, delim);
		else {
			r = NULL;
			for(int i=0; i<strlen(pstr); i++) {
				if(isspace(pstr[i])) {
					r = &pstr[i];
					break;
				}
			}
		}
	}

	if( strlen(pstr) > 0 || empties )
	{
		results.push_back(string(pstr));
	}

	return int(results.size()); // Cast used to avoid compiler warning
}

void save_data(float *x, float *y, int n, const char *fsp)
{
FILE *out;
int i;

out=fopen(fsp,"wb");
if (out==NULL) return;
for (i=0; i<n; i++) fprintf(out,"%g\t%g\n",x[i],y[i]);
fclose(out);
return;
}

void save_data(float x0,float dx, float *y, int n, const char *fsp)
{
FILE *out;
int i;

out=fopen(fsp,"wb");
if (out==NULL) return;
for (i=0; i<n; i++) fprintf(out,"%g\t%g\n",x0+dx*i,y[i]);
fclose(out);
return;
}

// substring search (broken in early linux libc, and missing on SGI)
char *memmem(char *haystack,int hsize,char *needle,int nsize) {
int i;
for (i=0; i<hsize-nsize+1; i++) {
	if (haystack[i]==needle[0]) {
		int j;
		for (j=1; j<nsize; j++) if (haystack[i+j]!=needle[j]) break;
		if (j==nsize) return haystack+i;
	}
}
return NULL;
}


// basically a version of 'system'
int run(char *s) {
#ifndef _WIN32																// WIN32 #ifndef statement start															

pid_t f=fork();
int stat;

#ifdef ALPHA
if (!f) execl("/bin/zsh","/bin/zsh","-c",s,0);	// temporary fix
#else
if (!f) execl("/bin/sh","/bin/sh","-c",s,0);
#endif
//printf("%d exit\n",waitpid(f,NULL,0));
waitpid(f,&stat,0);

if (WIFEXITED(stat)) return WEXITSTATUS(stat);

return 256;
#else
 system(s);
#endif
// WIN32 #ifndef statement start
return -1;																	// now returns -1`to indicate nothing happened
}

// No shared memory for now
/*
inline void *Smalloc(int size,int shared) {
if (shared) return DMEM.malloc(size);
else return malloc(size);
}

inline void Sfree(void *ptr, int shared) {
if (shared) DMEM.free(ptr);
else free(ptr);
}

inline void *Srealloc(void *ptr, int size, int shared) {
if (shared) return DMEM.realloc(ptr,size);
else return realloc(ptr,size)
}
*/

unsigned long resolv(char *host)
{
struct hostent *he;
unsigned long addr;

if (!host) { 
	host=(char *)malloc(128);
	gethostname(host,127);
	he=gethostbyname(host);
	free(host);
}
else he=gethostbyname(host);
if (he==NULL) return(0);
memcpy(&addr,he->h_addr,4);
return(ntohl(addr));
}

int sock_open(unsigned long addr,unsigned short port)
{
int s,p;
sockaddr_in sa;

s=socket(AF_INET,SOCK_STREAM,6);
if (s==-1) return (-1);

memset((char *)&sa, 0,sizeof(sockaddr_in));
sa.sin_family=AF_INET;
sa.sin_port=htons(port);
sa.sin_addr.s_addr=htonl(addr);
if (connect(s,(sockaddr*)&sa,sizeof(sockaddr_in))) { close(s); return(-2); }

return(s);
}

int writesock(int sock,const char *data,int len,int almtime) {
int rd = 0,r;

while (rd<len) {
	r=0;
#ifndef _WIN32
	alarm(almtime);		// if it takes more than 120 seconds to write DIE
	while (r==0) {
		r=write(sock,data+rd,len-rd);
		if (r==0) { struct timespec ts= {0,1000000}; nanosleep(&ts,NULL); }
	}
	alarm(0);
	if (r<0) { printf("ERROR: No write to socket (%d)\n",errno); return -1; }
	rd+=r;
#endif	//_WIN32

}

return rd;
}

int readsock(int sock,char *data,int len,int almtime) {
int rd = 0,r;

while (rd<len) {
	r=0;
#ifndef _WIN32
	alarm(almtime);		// if we don't get anything within 60 seconds, DIE	
	while (r==0) {
		r=read(sock,data+rd,len-rd);
		if (r==0) { struct timespec ts= {0,1000000}; nanosleep(&ts,NULL); }
	}
	alarm(0);
	if (r<0) { printf("Error: no data on socket (%d)\n",errno); return -1; }
	rd+=r;
#endif	//_WIN32
}

return rd;
}

int readsocknz(int sock,char *data,int len,int almtime) {
int rd = 0,r;

while (rd<len) {
	r=0;
#ifndef _WIN32
	alarm(almtime);	
	while (r==0) {
		r=read(sock,data+rd,len-rd);
		if (r==0 && rd==0) { alarm(0); return 0; }
		if (r==0) { struct timespec ts= {0,1000000}; nanosleep(&ts,NULL); }
	}
	alarm(0);
	if (r<0) { printf("Error: no data on socket (%d)\n",errno); return -1; }
	rd+=r;
#endif	//_WIN32
}

return rd;
}

#ifdef FILESERVER

unsigned short FSPORT=0;
unsigned int FSHOST=0;
unsigned int FSKEY=0;
int FSSOCK=0;

// Send a command to the fileserver, if datalen==-1, null terminated string
// command is FS_DONE, FS_TEXT, FS_BIN or FS_IMAGIC
// (for FS_BIN, the first 4 bytes are the file location)
int FSsend(int command, int datalen,const char *fsp,const char *data,int fileno) {
int sock=-1,n=0;

if (!FSHOST||!FSSOCK) return FSsendlocal(command,datalen,fsp,data,fileno);
char path[MAXPATHLEN];
getcwd(path,MAXPATHLEN-1);

/*
while (sock<0 && n<10) {
	sleep(1);
	sock=sock_open(FSHOST,FSPORT);
	n++;
	if (n>1) {
		char s[120];
		gethostname(s,119);
		printf("%s:Retry socket %d\n",s,n);
	}
}

if (sock<0) { printf("%hu,%x  (%d,%d)\n",FSPORT,FSHOST,sock,errno); error(ERR_EXIT,"No socket to fileserver after 10 tries"); }
*/
sock=FSSOCK;

FSCOMMAND com;
if (datalen==-1) com.datalen=strlen(data);
else com.datalen=datalen;
com.command=command;
com.key=FSKEY;
com.pathlen=strlen(path);
com.filelen=strlen(fsp);
com.fileno=fileno;
//int zero=0,one=1;
//setsockopt(sock,SOL_TCP,TCP_CORK,&one,sizeof(int));
writesock(sock,(char *)&com,sizeof(com));
writesock(sock,(char *)path,strlen(path));
writesock(sock,(char *)fsp,strlen(fsp));
writesock(sock,(char *)data,com.datalen);
char ack[4] = {0,0,0,0};
readsock(sock,ack,4);
if (strncmp(ack,"ACK",3)!=0) error(ERR_EXIT,"No ACK on FSsend");
//setsockopt(sock,SOL_TCP,TCP_CORK,&zero,sizeof(int));
}
#else
int FSsend(int command, int datalen,const char *fsp,const char *data,int fileno) {
return FSsendlocal(command,datalen,fsp,data,fileno);
}
#endif

int FSsendlocal(int command, int datalen,const char *fsp,const char *data,int fileno) {
FILE *out;
int *loc = (int *) data;

switch(command) {
case FS_BIN:
	if (*loc>=0) {
		out=fopen(fsp,"rb+");
		if (!out) out=fopen(fsp,"wb");
		file_lock_wait(out);
		int *loc = (int *)data;
		portable_fseek(out,*loc,SEEK_SET);
	}
	else {					// -1 appends, -2 truncates
		if (*loc==-1) out=fopen(fsp,"ab");
		else out=fopen(fsp,"wb");
		file_lock_wait(out);
	}
	fwrite(data+4,datalen-4,1,out);
	fclose(out);
	break;
case FS_LST:
	if (access(fsp,F_OK)||!fileno) {
		out=fopen(fsp,"wb");
		fprintf(out,"#LST\n");
	}
	else out=fopen(fsp,"ab");
	file_lock_wait(out);
	fprintf(out,"%s",data);
	fclose(out);
	break;
case FS_TEXT:
	out=fopen(fsp,"ab");
	file_lock_wait(out);
	fprintf(out,"%s",data);
	fclose(out);
	break;
}
return 0;
}

bool is_little_endian() {	// check if this machine is little endian
	int one=1;
	char* bytep=(char*)(&one);
	if(bytep[0]==1 && bytep[1]==0 && bytep[2]==0 && bytep[3]==0) return true;
	else	return false; 
}

bool is_big_endian() {		// check if this machine is big endian
	return ! is_little_endian();
}

int generate_machine_stamp() {	// output a machine stamp in MRC/CCP4/HDF convention. this only output the ieee_big_endian (ox44440000) and ieee_little_endian (0x11110000) and ignoring other specific types
	int stamp;
	char* p=(char*)(&stamp);
	if(!is_little_endian()){
		p[0]=0x11;	p[1]=0x11;	p[2]=0;	p[3]=0;	
	}
	else {
		p[0]=0;	p[1]=0;	p[2]=0x44;	p[3]=0x44;	
	}
	return stamp;
}

bool check_byte_order(char* small_num_addr) {
    bool right_order = true;

    if (small_num_addr) {
	int byte0 = small_num_addr[0];
	int byte1 = small_num_addr[1];

	if (ByteOrder::is_machine_big_endian()) {
	    if (byte0 != 0 || byte1 != 0) {
		right_order = false;
	    }
	}
	else {
	    if (byte0 == 0 && byte1 == 0) {
		right_order = false;
	    }
	}
    }
    return right_order;
}
	
char* file_basename(const char* filename) { // /abs/path/filename.ext -> filename
	char *ret=(char*)malloc(strlen(filename)+1);
	char *tmp=(char*)malloc(strlen(filename)+1);
	strcpy(tmp,filename);
	char* id0=strrchr(tmp,'/');
	if (!id0) id0=tmp;
	else id0=&id0[1];
	strcpy(ret,id0);
	char* id1=strchr(ret,'.');
	if(id1) ret[id1-ret]=0;
	free(tmp);
	return ret;
}

bool check_file_type(const char* filename, const char* file_ext) {
    if (filename && file_ext) {
	int file_strlen = strlen(filename);
	int ext_strlen = strlen(file_ext);
	if (file_strlen > ext_strlen &&
	    strcmp(&filename[file_strlen - ext_strlen], file_ext) == 0) {
	    return true;
	}
    }
    return false;
}
    

FILE* open_image_file(const char* filename, bool *is_zipped) {
    FILE* in = 0;

    if (!filename) {
	in = stdin;
    }
    else if (filename && is_zipped) {
	char unzip_cmd[MAXPATHLEN];
	
	if (check_file_type(filename, ".gz") || check_file_type(filename, ".Z")) {
	    sprintf(unzip_cmd, "zcat %s", filename);
	    *is_zipped = true;
	}
	else if (check_file_type(filename, ".gz") || check_file_type(filename, ".bz2")) {
	    sprintf(unzip_cmd, "bzcat %s", filename);
	    *is_zipped = true;
	}
	else {
	    *is_zipped = false;
	}
	
	if (*is_zipped) {
	    in = popen(unzip_cmd, "rb");
	}
	else {
	    in = fopen(filename, "rb");
	}
	if (!in) {
	    printf("Error: cannot open file: '%s'\n", filename);
	}
    }
    return in;
}

const char* get_imagetype_name(ImageType t)
{
    switch(t) {   
    case IMAGE_MRC:
	return "mrc";
    case IMAGE_SPIDER:
	return "spider";
    case IMAGE_IMAGIC:
	return "img";
    case IMAGE_PGM:
	return "pgm";
    case IMAGE_LST:
	return "lst";
    case IMAGE_PIF:
	return "pif";
    case IMAGE_PNG:
	return "png";
    case IMAGE_HDF:
	return "hdf5";
    case IMAGE_DM3:
	return "dm3";
    case IMAGE_TIF:
	return "tif";
    case IMAGE_VTK:
	return "vtk";
    case IMAGE_SAL:
	return "sal";
    case IMAGE_ICOS:
	return "map";
    case IMAGE_IMAGIC3D:
	return "img3d";
    case IMAGE_EMIM:
	return "emim";
    case IMAGE_SINGLE_SPIDER:
	return "single_spider";
    case IMAGE_GATAN:
	return "dm2";
    case IMAGE_AMIRA:
	return "amira";
    case IMAGE_XPLOR:
	return "xplor";
    case IMAGE_ANY:
	return "any";
    }
    return "unknown";
}
