#include "PyList.h"
#include "PyPickle.h"
#include "Euler.h"
#include "EMData.h"
#include "Vect.h"
#include "Cylinder.h"
#include <boost/python/detail/api_placeholder.hpp>

python::tuple Euler_pickle_suite::getinitargs(Euler& e)
{
    return python::make_tuple(e.alt(), e.az(), e.phi());
}

python::tuple Quater_pickle_suite::getinitargs(Quater& q)
{
    return python::make_tuple(q.nx, q.ny, q.nz, q.Q);
}

python::tuple Vect_pickle_suite::getinitargs(Vect& v)
{
    return python::make_tuple(v.X(), v.Y(), v.Z());
}

python::tuple Cylinder_pickle_suite::getinitargs(Cylinder& c)
{
    return python::make_tuple(c.Center(), c.Axis(), c.Length(), c.Radius(), c.Weight());
}


python::tuple EMData_pickle_suite::getinitargs(EMData& em)
{
    return python::make_tuple();
}

enum EMDataState {
    EMDATA_DICT,
    EMDATA_FLAGS,
    //EMDATA_DATA,
    EMDATA_NX,
    EMDATA_NY,
    EMDATA_NZ,
    EMDATA_DX,
    EMDATA_DY,
    EMDATA_DZ,
    EMDATA_EULER,
    EMDATA_NIMG,
    EMDATA_PIXEL,
    // EMDATA_CTF,
    EMDATA_MIN,
    EMDATA_MAX,
    EMDATA_MINLOC,
    EMDATA_MAXLOC,
    EMDATA_MEAN,
    EMDATA_SIGMA,
    EMDATA_SUMSQ,
    EMDATA_MEAN2,
    EMDATA_SIGMA2,
    EMDATA_NUM_STATES
};

python::tuple EMData_pickle_suite::getstate(python::object em_obj)
{
    EMData& em = python::extract<EMData&>(em_obj)();

    python::tuple result(EMDATA_NUM_STATES);
    
    result[EMDATA_DICT] = em_obj.attr("__dict__");
    
    result[EMDATA_FLAGS] = em.getFlags();

    /*
    python::object num_array = em.get_num_array();
    result[EMDATA_DATA] = num_array;
    */
    result[EMDATA_NX] = em.xSize();
    result[EMDATA_NY] = em.ySize();
    result[EMDATA_NZ] = em.zSize();
    result[EMDATA_DX] = em.Dx();
    result[EMDATA_DY] = em.Dy();
    result[EMDATA_DZ] = em.Dz();

    Euler* euler = em.getEuler();
    result[EMDATA_EULER] = *euler;

    result[EMDATA_NIMG] = em.NImg();
    result[EMDATA_PIXEL] = em.Pixel();
    /*
    PyList ctf_list = em.py_getCTF();
    result[EMDATA_CTF] = ctf_list;
    */
    result[EMDATA_MIN] = em.Min();
    result[EMDATA_MAX] = em.Max();
    result[EMDATA_MINLOC] = em.MinLoc();
    result[EMDATA_MAXLOC] = em.MaxLoc();
    result[EMDATA_MEAN] = em.Mean();
    result[EMDATA_SIGMA] = em.Sigma();
    result[EMDATA_SUMSQ] = em.SumSq();
    result[EMDATA_MEAN2] = em.Mean2();
    result[EMDATA_SIGMA2] = em.Sigma2();
    
    return result;
}

void EMData_pickle_suite::setstate(python::object em_obj, python::tuple state)
{
    EMData& em = python::extract<EMData&>(em_obj)();

    if (python::len(state) != EMDATA_NUM_STATES) {
	PyErr_SetObject(PyExc_ValueError,
			("expected %d-item tuple in call to __setstate__; got %s"
			 % (EMDATA_NUM_STATES, state)).ptr());
	python::throw_error_already_set();
    }
    
    python::dict mydict = python::extract<python::dict>(em_obj.attr("__dict__"))();
    mydict.update(state[EMDATA_DICT]);

    int flags = python::extract<int>(state[EMDATA_FLAGS]);
    em.setFlags(flags);

    int nx = python::extract<int>(state[EMDATA_NX]);
    int ny = python::extract<int>(state[EMDATA_NY]);
    int nz = python::extract<int>(state[EMDATA_NZ]);
    em.setSize(nx, ny, nz);
    /*
    python::numeric::array& num_array = python::extract<python::numeric::array>(state[EMDATA_DATA]);
    em.set_num_array(num_array);
    */
    float dx = python::extract<float>(state[EMDATA_DX]);
    float dy = python::extract<float>(state[EMDATA_DY]);
    float dz = python::extract<float>(state[EMDATA_DZ]);
    em.setTAlign(dx, dy, dz);

    Euler euler = python::extract<Euler>(state[EMDATA_EULER]);    
    em.setRAlign(&euler);
    
    int nimg = python::extract<int>(state[EMDATA_NIMG]);
    em.setNImg(nimg);

    float pixel = python::extract<float>(state[EMDATA_PIXEL]);
    em.setPixel(pixel);

    /*
    PyList ctf_list = python::extract<PyList>(state[EMDATA_CTF]);    
    float* ctf_array = new float[python::len(ctf_list)];
    list2array(ctf_list, ctf_array);
    em.setCTF(ctf_array);
    delete [] ctf_array;
    ctf_array = 0;
    */
    
    float min = python::extract<float>(state[EMDATA_MIN]);
    em.setMin(min);

    float max = python::extract<float>(state[EMDATA_MAX]);
    em.setMax(max);

    int minloc = python::extract<int>(state[EMDATA_MINLOC]);
    em.setMinLoc(minloc);

    int maxloc = python::extract<int>(state[EMDATA_MAXLOC]);    
    em.setMaxLoc(maxloc);

    float mean = python::extract<float>(state[EMDATA_MEAN]);
    em.setMean(mean);

    float sigma = python::extract<float>(state[EMDATA_SIGMA]);
    em.setSigma(sigma);
    
    float sumsq = python::extract<float>(state[EMDATA_SUMSQ]);
    em.setSumSq(sumsq);
    
    float mean2 = python::extract<float>(state[EMDATA_MEAN2]);
    em.setMean2(mean2);

    float sigma2 = python::extract<float>(state[EMDATA_SIGMA2]);
    em.setSigma2(sigma2);
}

bool EMData_pickle_suite::getstate_manages_dict()
{
    return true;
}


enum ObjectStat_State {
    OBJSTAT_DICT,
    OBJSTAT_XC,
    OBJSTAT_YC,
    OBJSTAT_ZC,
    OBJSTAT_AXIS,
    OBJSTAT_MASS,
    OBJSTAT_VOLUME,
    OBJSTAT_LEN,
    OBJSTAT_RAD,
    OBJSTAT_ASPECTRATIO,
    OBJSTAT_MOMENTS,
    OBJSTAT_EIGENVALUE,
    OBJSTAT_EIGENVECTOR,
    OBJSTAT_NUM_STATES
};

python::tuple ObjectStat_pickle_suite::getinitargs(ObjectStat& o)
{
    return python::make_tuple();
}

python::tuple ObjectStat_pickle_suite::getstate(python::object os_obj)
{
    ObjectStat& os = python::extract<ObjectStat&>(os_obj)();
    python::tuple result(OBJSTAT_NUM_STATES);

    result[OBJSTAT_DICT] = os_obj.attr("__dict__");
    
    result[OBJSTAT_XC] = os.xc;
    result[OBJSTAT_YC] = os.yc;
    result[OBJSTAT_ZC] = os.zc;

    result[OBJSTAT_AXIS] = os.axis;
    
    result[OBJSTAT_MASS] = os.mass;
    result[OBJSTAT_VOLUME] = os.volume;
    result[OBJSTAT_LEN] = os.len;
    result[OBJSTAT_RAD] = os.rad;

    PyList asp_list = PyList();
    array2list(os.aspectratio, asp_list, 3);
    result[OBJSTAT_ASPECTRATIO] = asp_list;

    PyList mom_list = PyList();
    array2list((float*)os.moments, mom_list, 9);
    result[OBJSTAT_MOMENTS] = mom_list;

    PyList evalue_list = PyList();
    array2list(os.eigenvalue, evalue_list, 3);    
    result[OBJSTAT_EIGENVALUE] = evalue_list;

    PyList evector_list = PyList();
    array2list((float*)os.eigenvector, evector_list, 9);
    result[OBJSTAT_EIGENVECTOR] = evector_list;
    
    return result;
}

void ObjectStat_pickle_suite::setstate(python::object os_obj, python::tuple state)
{
    ObjectStat& os = python::extract<ObjectStat&>(os_obj)();
    
    if (python::len(state) != OBJSTAT_NUM_STATES) {
	PyErr_SetObject(PyExc_ValueError,
			("expected %d-item tuple in call to __setstate__; got %s"
			 % (OBJSTAT_NUM_STATES, state)).ptr());
	python::throw_error_already_set();
    }
    
    python::dict mydict = python::extract<python::dict>(os_obj.attr("__dict__"))();
    mydict.update(state[OBJSTAT_DICT]);

    os.xc = python::extract<float>(state[OBJSTAT_XC]);
    os.yc = python::extract<float>(state[OBJSTAT_YC]);
    os.zc = python::extract<float>(state[OBJSTAT_ZC]);

    os.axis = python::extract<Vect>(state[OBJSTAT_AXIS]);
    
    os.mass = python::extract<float>(state[OBJSTAT_MASS]);
    os.volume = python::extract<float>(state[OBJSTAT_VOLUME]);
    os.len = python::extract<float>(state[OBJSTAT_LEN]);
    os.rad = python::extract<float>(state[OBJSTAT_RAD]);

    PyList asp_list = python::extract<PyList>(state[OBJSTAT_ASPECTRATIO]);
    list2array(asp_list, os.aspectratio);
    
    PyList mom_list = python::extract<PyList>(state[OBJSTAT_MOMENTS]);
    list2array(mom_list, (float*)os.moments);
    
    PyList evalue_list = python::extract<PyList>(state[OBJSTAT_EIGENVALUE]);
    list2array(evalue_list, os.eigenvalue);
    
    PyList evector_list = python::extract<PyList>(state[OBJSTAT_EIGENVECTOR]);
    list2array(evector_list, (float*)os.eigenvector);
}



bool ObjectStat_pickle_suite::getstate_manages_dict()
{
    return true;
}

#if 0
python::tuple Helix_pickle_suite::getinitargs(Helix& h)
{
    return python::make_tuple();
}


python::tuple Helix_pickle_suite::getstate(python::object h_obj)
{
    return python::make_tuple();
}


bool Helix_pickle_suite::getstate_manages_dict()
{
    return true;
}


python::tuple HHH_pickle_suite::getinitargs(HHH& h)
{
    return python::make_tuple();
}

bool HHH_pickle_suite::getstate_manages_dict()
{
    return true;
}

#endif
