#include "ImHist.h"
#include <qstringlist.h>

ImHist::ImHist(QWidget* parent,const char* name):QWidget( parent, name ) {
qpm=new QPixmap(size());
qp=new QPainter;

setBackgroundMode(NoBackground);
qp->begin(qpm);
qp->fillRect(0,0,size().width(),size().height(),white);
qp->setPen(black);
qp->drawText(0,0,size().width(),size().height(),AlignCenter,"No Data");
qp->end();

mind=0;
maxd=1.0;
sel=1.0e20;
range0=range1=0;
stat=0;
}

ImHist::~ImHist() {
delete qpm;
delete qp;
}

void ImHist::resizeEvent ( QResizeEvent *qe ) {
qpm->resize(size().width(),size().height());
qp->begin(qpm);
qp->fillRect(0,0,size().width(),size().height(),white);
qp->setPen(black);
qp->drawText(0,0,size().width(),size().height(),AlignCenter,"No Data");
qp->end();
//update();
}

void ImHist::paintEvent( QPaintEvent *pe ) {
int w=width()-2,h=height()-2;
QPixmap qpm2(size());

//printf("paint %f %f\n",range0,range1);
qp->begin(&qpm2);
qp->fillRect(pe->rect(),black);
if (range0!=range1) {
	qp->fillRect(w*(range0-mind)/(maxd-mind)+1,1,
		w*(range1-range0)/(maxd-mind),h,QColor(0,32,32));
}
bitBlt(&qpm2,0,0,qpm,0,0,width(),height(),XorROP);
//bitBlt( this,pe->rect().topLeft(),qpm,pe->rect(),XorROP);

// Blue line is the mean
qp->setPen(blue);
int x=w*(mean-mind)/(maxd-mind)+1;
qp->drawLine(x,1,x,h);

// Green line is the peak (dCenter) 
qp->setPen(green);
x=w*(peak-mind)/(maxd-mind)+1;
qp->drawLine(x,1,x,h);

// Red line is selected
if (sel>=mind && sel<=maxd) {
	int xl=(sel-mind)*w/(maxd-mind)+1;
	qp->setPen(red);
	qp->drawLine(xl,1,xl,h);
	char s[20];
	sprintf(s,"%1.5g",sel);
	qp->setPen(black);
	qp->drawText(w-84,12,s);
}

if (stat) {
	qp->setPen(black);
	char s[40];
	sprintf(s,"Mean =%1.6g",mean);
	qp->drawText(15,12,s);
	sprintf(s,"Sigma=%1.6g",sigma);
	qp->drawText(15,26,s);
	sprintf(s,"Min/Max=(%1.4g,%1.4g)",mind,maxd);
	qp->drawText(15,40,s);
}

if (misc.length()>0) {
	qp->setPen(black);
	QStringList sl = QStringList::split("\n",misc);
	QString v;
	int i;
	for (v=sl.last(),i=0; !sl.empty(); sl.pop_back(),v=sl.last(),i++) 
		qp->drawText(15,h-i*14-4,v);
}


qp->end();

bitBlt(this,pe->rect().topLeft(),&qpm2,pe->rect());
}

void ImHist::setMiscText(QString s) {
	misc=s;
	this->update();
}

void ImHist::setData(QList<EMData> *im,int n) {
float *hist;
int w=width()-2,h=height()-2;
float max,lmax,lmax2;
int i,f=0;

//printf("draw %d %d\n",w,h);
hist=(float *)calloc(w,sizeof(float));

// If invalid subimage, just average the histograms of all of the images
if (n<0||n>im->count()) {
	for (i=0; i<im->count(); i++) {
		if (im->at(i)->Min()<mind) mind=im->at(i)->Min();
		if (im->at(i)->Max()>maxd) maxd=im->at(i)->Max();
	}

	for (i=0; i<im->count(); i++) {
		im->at(i)->calcHist(hist,w,mind,maxd,i);
	}
}
else {
	mind=im->at(n)->Min();
	maxd=im->at(n)->Max();

	peak=im->at(n)->dCenter();
	mean=im->at(n)->Mean();
	sigma=im->at(n)->Sigma();
	im->at(n)->calcHist(hist,w,mind,maxd,i);
}

qp->begin(qpm);
qp->fillRect(0,0,size().width(),size().height(),white);
qp->setPen(QColor(64,64,64));
qp->drawRect(0,0,size().width(),size().height());
qp->setPen(QColor(120,120,120));

// If over 30% of the histogram is zero, we do a 3 cell running average
/*int z=0;
for (i=0; i<w; i++) if (hist[i]==0) z++;
if (z/(float)w>.3) {
	float *h2 = malloc(w*sizeof(float));
	for (i=1; i<w-1; i++) h2[i]=(hist[i-1]+hist[i]+hist[i+1])/3.0;
	h2[0]=(hist[0]+hist[1])/2.0;
	h2[w-1]=(hist[w-1]+hist[w-2])/2.0;
	free(hist);
	hist=h2;
	f=1;
}*/

//max=0.0001;
//for (i=0; i<w; i++) { if (hist[i]>max) max=hist[i]; }

max=lmax=lmax2=0;
for (i=0; i<w; i++) {
	if (hist[i]>max) { lmax2=lmax; lmax=max; max=hist[i]; }
	else if (hist[i]>lmax) { lmax2=lmax; lmax=hist[i]; }
	else if (hist[i]>lmax2) { lmax2=hist[i]; }
}
if (lmax==0) lmax=max;
if (lmax2==0) lmax2=lmax;
if (lmax2==0) lmax2=lmax=max=1.0;
if (max/lmax2>2.0) max=lmax2*2.0;	  // this will eliminate the central maximum if it's too large

/*if (stat) {
	printf("max=%f\tlmax=%f\tlmax2=%f\n",max,lmax,lmax2);
	for (i=0; i<w; i++) printf("%d\t%f\t%f\n",i,hist[i],hist[i]/max);
}*/

for (i=0; i<w; i++) {
	if (hist[i]>max) {
		qp->setPen(QColor(64,64,32));
		qp->drawLine(i+1,h,i+1,1);
		qp->setPen(QColor(120,120,120));
	}
	else qp->drawLine(i+1,h,i+1,h-h*hist[i]/max+1);
}

if (f) qp->drawText(5,12,"SM");
qp->end();
free(hist);
}

void ImHist::setData(EMData *d) {
float *hist;
int w=width()-2,h=height()-2;
float max,lmax,lmax2;
int i,f=0;

hist=(float *)malloc(w*sizeof(float));

mind=d->Min();
maxd=d->Max();
d->calcHist(hist,w,mind,maxd);

qp->begin(qpm);
qp->fillRect(0,0,size().width(),size().height(),white);
qp->setPen(QColor(64,64,64));
qp->drawRect(0,0,size().width(),size().height());
qp->setPen(QColor(120,120,120));

// If over 30% of the histogram is zero, we do a 3 cell running average
/*int z=0;
for (i=0; i<w; i++) if (hist[i]==0) z++;
if (z/(float)w>.3) {
	float *h2 = malloc(w*sizeof(float));
	for (i=1; i<w-1; i++) h2[i]=(hist[i-1]+hist[i]+hist[i+1])/3.0;
	h2[0]=(hist[0]+hist[1])/2.0;
	h2[w-1]=(hist[w-1]+hist[w-2])/2.0;
	free(hist);
	hist=h2;
	f=1;
}*/

max=lmax=lmax2=0;
for (i=0; i<w; i++) {
	if (hist[i]>max) { lmax2=lmax; lmax=max; max=hist[i]; }
	else if (hist[i]>lmax) { lmax2=lmax; lmax=hist[i]; }
	else if (hist[i]>lmax2) { lmax2=hist[i]; }
}
if (lmax==0) lmax=max;
if (lmax2==0) lmax2=lmax;
if (lmax2==0) lmax2=lmax=max=1.0;
if (max/lmax2>2.0) max=lmax2*2.0;	  // this will eliminate the central maximum if it's too large

for (i=0; i<w; i++) {
	if (hist[i]>max) {
		qp->setPen(QColor(64,64,32));
		qp->drawLine(i+1,h,i+1,1);
		qp->setPen(QColor(120,120,120));
	}
	else qp->drawLine(i+1,h,i+1,h-h*hist[i]/max+1);
}
if (f) qp->drawText(5,12,"SM");
qp->end();
free(hist);
peak=d->dCenter();
mean=d->Mean();
sigma=d->Sigma();
}

void ImHist::setMinMax(float min,float max) {
//printf("minmax  %f %f \n",min,max);
range0=min;
range1=max;
update();
}

void ImHist::mousePressEvent(QMouseEvent *e) {
if ((e->button()&RightButton)) { stat^=1; update(); }

}

void ImHist::mouseReleaseEvent(QMouseEvent *e) {

}

void ImHist::mouseMoveEvent(QMouseEvent *e) {
int w=width()-2,h=height()-2;

if ((e->state()&LeftButton)) {
	sel=(e->x()-1)*(maxd-mind)/w+mind;
	update();
}

}

void ImHist::setSel(float x) {
sel=x;
update();
}

