// helixpitch.C  Copyright 2001 Baylor College of Medicine
// 2/19/01

//N helixpitch
//F Determine the pitch of a helical object
//T X
//1
//P <input file>	Images to be processed
//P <n>	Image number in file
//P [apix=<value>]	Specify A/pixel
//P [out=<filename>]	Output file for correlation plot
//U helixpitch bundle
//D WARNING: Highly experimental
//D This will attempt to determine the pitch of a helix. Actually,
//E it determines the vertical repeat disance.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

main(int argc, char *argv[]) {
int i, imgn;
float keep=10.0;
float apix=1.0;
char *corr="selfcorr.txt";

appinit(argc,argv);

if (argc<3) appusage(argv[0]);

imgn=atoi(argv[2]);
for (i=3; i<argc; i++) {	
	if (strncmp(argv[i],"apix=",5)==0) apix=atof(&argv[i][5]); 
	else if (strncmp(argv[i],"out=",4)==0) corr=&argv[i][4]; 
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

EMData orig;
orig.readImage(argv[1],imgn);
int nx = orig.xSize(),ny = orig.ySize();

FILE *out=fopen(corr,"wb");

int dy,x,y,by=0;
float bd=1.0e12;
for (dy=0; dy<ny; dy++) {
	float df=0,no=0; 
	for (y=0; y<ny; y++) {
		for (x=0; x<nx; x++) {
			if (y+dy>ny-1) continue;
			df+=SQR(orig.valueAt(x,y+dy)-orig.valueAt(x,y));
			no++;
		}
	}
	df/=no;
	if (dy>ny/10 && df<bd) { bd=df; by=dy; }
	fprintf(out,"%1.2f\t%1.5f\n",dy*apix,df);
}
fclose(out);
printf("The optimal repeat distance is %1.2fA (%d pixels) with a correlation value of %1.2f.\n",by*apix,by,bd);
printf("However, %s should be checked manually for confirmation\n",corr);
}
