// eulerproj.C - Steve Ludtke  2/19/98
// This will take a set of footprints from a classes file and another
// set from reprojections of a preliminary model. It then groups each class
// with whichever reprojection it matches best.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

struct DOT { float d; unsigned short i1,i2; };

inline int sgn(float x) { if (x==0) return 0; return x<0?-1:1; }

int dotcomp(const void *a, const void *b) { return sgn(*((float *)b)- *((float *)a)); }

main(int argc, char *argv[]) {
float **f,**cl;
float mm,r,norm1,norm2,eul[3];
float *mean1,*mean2, *sig1,*sig2;
char s[121],s2[121],fsp[121];
int n,n2,nc,bad;
int i,j,k,ii,c,*set,ff=0;
FILE *in,*out;
EMData *dat,*dat2,*dat2a,*dat3;

if (argc<3) { printf("eulerproj.C\nUsage: eulerproj <classes.fp> <projections.fp>\n"); exit(0); }

// first we read the raw data footprints
in=fopen(argv[1],"r");
if (!in) { printf("Cannot open footprints\n"); exit(0); }
fread(s,120,1,in);
fread(&n,4,1,in);
fread(&n2,4,1,in);
f=(float **)malloc(n2*sizeof(float *));

for (i=0; i<n2; i++) {
	f[i]=(float *)malloc(n*sizeof(float));
	fread(f[i],n,sizeof(float),in);
}
fclose(in);

// next we read the projection footprints
in=fopen(argv[2],"r");
if (!in) { printf("Cannot open footprints\n"); exit(0); }
fread(s2,120,1,in);
fread(&nc,4,1,in);
fread(&i,4,1,in);
if (i!=n2) {
	printf("ERROR: Footprints contain different numbers of dimensions!\n");
	exit(1);
}
cl=(float **)malloc(n2*sizeof(float *));

for (i=0; i<n2; i++) {
	cl[i]=(float *)malloc(nc*sizeof(float));
	fread(cl[i],nc,sizeof(float),in);
}
fclose(in);

mean1=(float *)calloc(n2*sizeof(float),1);
sig1=(float *)calloc(n2*sizeof(float),1);
mean2=(float *)calloc(n2*sizeof(float),1);
sig2=(float *)calloc(n2*sizeof(float),1);


for (i=0; i<n2; i++) {
	for (j=0; j<nc; j++) {
		mean2[i]+=cl[i][j];
		sig2[i]+=cl[i][j]*cl[i][j];
	}
	
	for (j=0; j<n; j++) {
		mean1[i]+=f[i][j];
		sig1[i]+=f[i][j]*f[i][j];
	}

	mean1[i]/=(float)n;
	mean2[i]/=(float)nc;
	
	sig1[i]=sig1[i]/(float)n-mean1[i]*mean1[i];
	sig2[i]=sig2[i]/(float)nc-mean2[i]*mean2[i];

	norm2=1.0/sig2[i];		// each footprint parameter is normalized to sigma=1
	norm1=1.0/sig1[i];

//	if (sig2[i]<.01) { norm1/=10.0; norm2/=10.0; }

	// notice that the 'center' from the second set is also used for the first set
	// the 1st set may not have a uniform euler angle distribution, so the center
	// would not be correct.
	for (j=0; j<nc; j++) cl[i][j]=(cl[i][j]-mean2[i])*norm2;
	for (j=0; j<n; j++) f[i][j]=(f[i][j]-mean2[i])*norm1;

	printf("%d. %f %f\t%f %f\n",i,sig1[i],sig2[i],mean1[i],mean2[i]);
}

// next we normalize each individual vector
for (j=0; j<nc; j++) {
	r=0;
	for (i=0; i<n2; i++) r+=cl[i][j]*cl[i][j];
	r=sqrt(r);
	for (i=0; i<n2; i++) cl[i][j]/=r;
}

for (j=0; j<n; j++) {
	r=0;
	for (i=0; i<n2; i++) r+=f[i][j]*f[i][j];
	r=sqrt(r);
	for (i=0; i<n2; i++) f[i][j]/=r;
}

set=(int *)calloc(n,sizeof(int));

/*
// noise will cause significant normalization differences, so we normalize
// each reference vector. May help...
for (i=0; i<nc; i++) {
	r=0;
	for (k=0; k<n2; k++) r+=SQR(cl[k][i]);
	r=sqrt(r);
	for (k=0; k<n2; k++) cl[k][i]/=r; 
}
*/

// now we match up raw images to classes
/*
// maximum dot product criterion (ind. points must be normalized
for (i=0; i<n; i++) {
	mm=-1.0e10;
	for (j=0; j<nc; j++) {
		r=0;
		for (k=0; k<n2; k++) r+=f[k][i]*cl[k][j];
		if (r>mm) { mm=r; set[i]=j; }
	}
}
*/

// minimum distance criterion
for (i=0; i<n; i++) {
	mm=1.0e10;
	for (j=0; j<nc; j++) {
		r=0;
		for (k=0; k<n2; k++) r+=pow(f[k][i]-cl[k][j],2.0);
		if (r<mm) { mm=r; set[i]=j; }
	}
}


// count elements in each class (for display only)
for (i=0; i<nc; i++) {
	k=0;
	for (j=0; j<n; j++) {
		if (set[j]==i) k++; 
	}
	printf("Class %01d. %d members\n",i,k);
}

// write the results. Images are unchanged. The header will have modified eulers
dat=new EMData;
dat2=new EMData;
for (i=0; i<nc; i++) {
	dat->readImage(s2,i);
	eul[0]=dat->alt();
	eul[1]=dat->az();
	eul[2]=dat->phi();
	dat->setRAlign(0,0,0);

	for (j=0; j<n; j++) {
		if (set[j]!=i) continue;
		dat2->readImage(s,j);
		dat2->setRAlign(0,0,0);
		dat2->setTAlign(0,0,0);
		dat2a=dat2->copy();
		dat2a->setParent(NULL);
		dat2a->vFlip();
		dat3=dat2->RTFAlign(dat,dat2a);

		printf("%d: %4.2f,%4.2f\t%5.1f->%5.1f\t%5.1f->%5.1f\t%5.1f->%5.1f\n",j,
			dat3->Dx(),dat3->Dy(),
			dat->alt()*180.0/PI,eul[0]*180.0/PI,
			dat->az()*180.0/PI,eul[1]*180.0/PI,
			dat->phi()*180.0/PI,(eul[2]+dat3->alt())*180.0/PI);

		dat2->setRAlign(eul[0],eul[1],eul[2]+dat3->alt());
		dat2->writeIMAGIC(s,j);
		
		dat->writeIMAGIC("eulasn.hed",-1);
		dat3->writeIMAGIC("eulasn.hed",-1);
		delete dat3;
		delete dat2a;
	}
}

}
