#include <stdlib.h>
#include <stdio.h>
#include "DataSet.h"

DataSet::DataSet() {
title="new";
//initMetaObject();                           // initialize meta object
data=(Column **)malloc(sizeof(Column *));
nx=ny=0;
}

DataSet::~DataSet()
{
free(data);
}

void DataSet::setRC(int x,int y) {
int i;

if (x<0) x=0;
if (y<1) y=1;

for (i=0; i<nx; i++) delete data[i];

data=(Column **)realloc(data,sizeof(Column *)*x);

for (i=0; i<x; i++) {
	data[i]=new Column(i);
	data[i]->setRange(0,1,y);
}

for (i=0; i<x; i++) {
	QObject::connect(data[i],SIGNAL(changed(int)),this,SLOT(colChanged(int)));
}

nx=x;
ny=y;

}

// not optimized, if one column changes, all columns are assumed to require update
void DataSet::colChanged(int col) {
#ifdef DEBUG2
printf("DataSet %p: column change\n",this);
#endif
for (int i=0; i<nx; i++) data[i]->eqChangedQ();
emit changed(); 
}

// for now this will cause all equation columns to recalculate on next use
// regardless of whether the variable actually causes a change
void DataSet::setVar(char *name,float val,bool quiet) {
int i;

for (i=0; i<nx; i++) {
	data[i]->setVar(name,val,TRUE);
	data[i]->eqChangedQ();
}

emit changed();
}

float DataSet::dataAt(int x,int y) {
if (x<0||x>=nx||y<0||y>=ny) return 0;

return data[x]->dataAt(y);
}

void DataSet::setXYfn(float minx, float maxx, int n, char *fofx) {
title=fofx;
setRC(2,n);
data[0]->setRange(minx,maxx,n);
data[1]->setEqn(fofx,n,this);
}

void DataSet::setXYdata(float minx, float maxx, int n) {
title="xydata";
setRC(2,n);
data[0]->setRange(minx,maxx,n);
data[1]->setData((float *)NULL,n);
}

void DataSet::setXYdata(float *xdat, float *ydat, int n) {
title="xydata";
setRC(2,n);
data[0]->setData(xdat,n);
data[1]->setData(ydat,n);
}

void DataSet::setXYparmfn(int n, char *fxofrow, char *fyofrow) {
title=fxofrow;
setRC(2,n);
data[0]->setEqn(fxofrow,n,this);
data[1]->setEqn(fyofrow,n,this);
}

void DataSet::setXYZdata(float *xdat, float *ydat, float *zdat, int n) {
title="XYZdata";
setRC(2,n);
data[0]->setData(xdat,n);
data[1]->setData(ydat,n);
data[2]->setData(zdat,n);
}

void DataSet::setXYZparmfn(int n,char *fxofrow, char *fyofrow, char *fzofrow) {
title=fxofrow;
setRC(2,n);
data[0]->setEqn(fxofrow,n,this);
data[1]->setEqn(fyofrow,n,this);	// Shouldn't really use x, but in case
data[2]->setEqn(fzofrow,n,this);
}


void DataSet::setXYZmxdata(float xmin, float xmax, int nx, float ymin,float ymax, int ny, float *zdata) {
title="3dMx";
setRC(3,nx*ny);
data[0]->setRepCyc(xmin,xmax,ny,1,nx*ny);
data[1]->setRepCyc(ymin,ymax,1,nx,nx*ny);
data[2]->setData(zdata,nx*ny);
}


void DataSet::setXYZmxfn(float xmin, float xmax, int nx, float ymin,float ymax, int ny, char *fofxy) {
title=fofxy;
setRC(3,nx*ny);
data[0]->setRepCyc(xmin,xmax,ny,1,nx*ny);
data[1]->setRepCyc(ymin,ymax,1,nx,nx*ny);
data[2]->setEqn(fofxy,nx*ny,this);
}

int DataSet::readFile(const char *file,char sep,int reclines,int skip) {
FILE *in;
char s[2048];
QString str,sub;
int i,lc,c,row;
int n;		// n is the number of fields in a record

in=fopen(file,"rb");
if (!in) return -1;

for (i=0; i<skip; i++) fgets(s,2047,in);

// Start by counting the number of fields in one record
str="";
for (i=0; i<reclines; i++) {
	fgets(s,2047,in);
	str+=s;
}
str.truncate(str.length()-1);

for (i=n=0; i<str.length(); i++) {
	if (str[i]==sep || str[i]=='\n') n++;
}
rewind(in);
setRC(n,1);

// now we read the file
for (i=0; i<skip; i++) fgets(s,2047,in);

int done=0;
row=0;
while (1) {
	// read 1 record from the file
	str="";
	for (i=0; i<reclines; i++) {
		if (!fgets(s,2047,in)) { done=1; break; }
		str+=s;
	}
	if (done) break;

	// parse the record	
	for (i=lc=c=0; i<n; i++) {
		c=str.find(sep,lc);
		if (c<0) c=2049;
		sub=str.mid(lc,c-lc);
		sub.stripWhiteSpace();
		lc=c+1;
		if (sub[0].isDigit() || (sub[0]=='-' && sub[1].isDigit()) || (sub[0]=='+' && sub[1].isDigit()))
			data[i]->setData(sub.toFloat(),row);
		else data[i]->setData(row,sub);
		row++;
	}

}

fclose(in);

return 0;
}


// reads data files with up to 10 columns
int DataSet::readFile(const char *path) {
FILE *in;
char s[241],t[30];
int i,n,c,d;
float v;
float **dat;

in=fopen(path,"rb");
if (!in) return -1;

title=path;
if (title.findRev('/')!=-1) title.truncate(title.findRev('/'));

// count rows and columns
n=0;
while (fgets(s,240,in)!=NULL) {
	if (s[0]=='#' || s[0]==';') continue;		// skip comments
	if (n==0) c=sscanf(s,"%g%[^.0-9-]%g%[^.0-9-]%g%[^.0-9-]%g%[^.0-9-]%g%[^.0-9-]%g%[^.0-9-]%g%[^.0-9-]%g%[^.0-9-]%g%[^.0-9-]%g",
		&v,t,&v,t,&v,t,&v,t,&v,t,&v,t,&v,t,&v,t,&v,t,&v);
	n++;
}
c=(c+1)/2;
//printf("Data file %d x %d\n",n,c);

setRC(c,n);		// make ourselves have the right size
dat=(float **)malloc(11*sizeof(float *));							// space to read data
for (i=0; i<c+1; i++) dat[i]=(float *)malloc(n*sizeof(float));	// allocate 1 extra scratch column
while (i<11) { dat[i]=dat[c]; i++; }					// extra columns point to scratch space

rewind(in);

// actual data read
n=0;
while (fgets(s,240,in)!=NULL) {
	if (s[0]=='#' || s[0]==';') continue;		// skip comments
	d=sscanf(s,"%g%[^.0-9-]%g%[^.0-9]%g%[^.0-9-]%g%[^.0-9-]%g%[^.0-9-]%g%[^.0-9-]%g%[^.0-9-]%g%[^.0-9-]%g%[^.0-9-]%g",
		&dat[0][n],t,&dat[1][n],t,&dat[2][n],t,&dat[3][n],t,&dat[4][n],t,
		&dat[5][n],t,&dat[6][n],t,&dat[7][n],t,&dat[8][n],t,&dat[9][n]);
	if (d!=c*2) printf("Warning row %d has only %d/%d columns.\n",n,d,c);
	n++;
}

//printf("%d actually read\n",n);

for (i=0; i<c; i++) data[i]->setData(dat[i],n);
free(dat[i]);
free(dat);
fclose(in);

return 0;
}

int DataSet::writeFile(const char *path) {
FILE *out;
int i,j;

out=fopen(path,"wb");
if (!out) return -1;

for (i=0; i<ny; i++) {
	for (j=0; j<nx; j++) {
		fprintf(out,"%3.7g",dataAt(j,i));
		if (j<nx-1) fprintf(out,"\t"); else fprintf(out,"\n");
	}
}
fclose(out);

return 0;
}
