// align3dsymslow.C - Copyright 06/15/98 Steve Ludtke
// Copyright 1998 Baylor College of Medicine

//N align3dsymslow
//F This will rotate a 3D model so its symmetric axes are aligned on the predefined axes
//T LA
//1
//P <input file>	Source file
//P <output file>	Destination file
//P sym=<Cn,Dn>	Apply an n-fold rotational symmetry to each image
//P [mask=<rad>]	Outside mask 
//P [imask=<rad>]	Inner mask applied only for location purposes. Not applied to output.
//U align3dsym threed.0a.mrc threed.0a.mrc sym=c7
//D This program will rotate 3D model so it's rotational symmetries are aligned on
//E the predefined axes. The model MUST be centered properly before this program
//E is used. By default the n-fold symmetric axis is z, and a 2-fold axis (for D symmetry)
//E lies along x.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include "Euler.h"

inline float asub(float x,float y) {
float r;

r=fabs(x-y);
if (r>PI) r=2.0*PI-r;

return r;
}

main(int argc, char *argv[]) {
int i,Ref,csym,nx,dsym=0,nca=0,imask=-1,mask=-1;
EMData *d1,*d2,*m,*p,*p2;
float alt,az,phi,q,r,best,eul[3],eul2[3],beul[3];
FILE *p3d=NULL;

appinit(argc,argv);

if (argc<4) appusage(argv[0]);


if (strncmp(argv[3],"sym=",4)!=0) {
	printf("Please specify sym\n");
	exit(0);
}

if (argc>4 && strncmp(argv[4],"nocalign",8)==0) nca=1;

for (i=4; i<argc; i++) {
	if (strncmp(argv[4],"nocalign",8)==0) nca=1;
	else if (strncmp("imask=",argv[i],6)==0) imask=atoi(&argv[i][6]);
	else if (strncmp("mask=",argv[i],5)==0) mask=atoi(&argv[i][5]);
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}


printf("Read start image\n");
d1=new EMData;
d1->readImage(argv[1],-1);
d1->edgeNormalize();
d1->applyMask(mask,4);
if (imask>0) d1->applyMask(imask,5);

//d1->cmCenter(0);
d1->normalize();
nx=d1->xSize();
//d1=d1->clip(-nx/4,-nx/4,-nx/4,nx+nx/2,nx+nx/2,nx+nx/2);
//d1->toCorner();
d1=d1->copy();
d2=d1->copy();

m=new EMData;
m->setSize(d1->xSize(),d1->ySize(),1);
m->one();
m->applyMask(nx/2,4);

Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);

best=-1.0e15;
beul[0]=beul[1]=beul[2]=0;
phi=0;

if (!nca) {
for (r=15.0; r>=.05; r/=6.0) {
	for (alt=beul[0]-r*6.0; alt<=beul[0]+r*6.0; alt+=r) {
		for (az=beul[1]-r*6.0; az<=beul[1]+r*6.0; az+=r) {
			for (phi=beul[2]-r*6.0; phi<=beul[2]+r*6.0; phi+=r) {
				Euler ort(alt*PI/180.0,az*PI/180.0,phi*PI/180.0),E(0,0,0),a;
				E.setSym(argv[3]+4);

				// we rotate d1 into position
				d1->setRAlign(&ort);
				d1->rotateAndTranslate();

				// then we apply the selected symmetry to it
				q=0;
				a=E.FirstSym();	// this is 0,0,0
				for (a=E.NextSym(); a.valid(); a=E.NextSym()) {
					d2->setRAlign(&a);
					d2->rotateAndTranslate();
					q+=d2->lcmp(d2->Parent());
				}
				printf("\r%1.2f,%1.2f,%1.2f  ",alt,az,phi);
				fflush(stdout);
				if (q>best) { 
					eul[0]=alt; eul[1]=az; eul[2]=phi; best=q; 
					printf("b-> %1.2f,%1.2f,%1.2f   %1.3g    ",alt,az,phi,best);
				}
			}
		}
	}
	beul[0]=eul[0];
	beul[1]=eul[1];
	beul[2]=eul[2];
}
}
printf("\n\nBest is %1.2f,%1.2f,%1.2f\n",beul[0],beul[1],beul[2]);

d1->readImage(argv[1],-1);
d1->edgeNormalize();
d1->applyMask(mask,4);
//d1->cmCenter(0);
d1->setRAlign(beul[0]*PI/180.0,beul[1]*PI/180.0,beul[2]*PI/180.0);
d1->rotateAndTranslate();
d1->writeMRC(argv[2]);
LOG(Ref,argv[2],LOG_OUTFILE,NULL);

LOG(Ref,NULL,LOG_END,NULL);
}
