// classesbymra3.C Copyright 1999 Steve Ludtke
// Copyright 1999 Baylor College of Medicine

//N classesbymra3
//F An experimental version of classesbymra
//T X
//1
//P <particles>	Raw particle filespec	
//P <refrences>	Reference image filespec
//P [] etc...
//D An experimental version of classesbymra.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

main(int argc, char *argv[]) {
int i,j,k,best,x,y,nr,ns,split=0,num=0,denom=1,dodot=0,ver=0,mask=-1,n0,n1;
List *data,*dataref,*toav;
EMData *d,*d2,*df,*dr,*da,*av;
char s[80];
float dot,bestdot,norm,norm2;
FILE *out;

appinit(argc,argv);

if (argc<3) { printf("classesbymra.c\nUsage: classesbymra <unclassed> <refrences> [frac=<0 to d-1>/<d>] [verbose] [dot] \n"); exit(0); }

for (i=3; i<argc; i++) {        
		if (strncmp(argv[i],"split",5)==0) split=1;		// historical compatibility
        else if (strncmp(argv[i],"dot",3)==0) dodot=1;
 		else if (strncmp(argv[i],"num=",4)==0) num=atoi(&argv[i][4]);
		else if (strncmp(argv[i],"denom=",6)==0) denom=atoi(&argv[i][6]);
		else if (strncmp(argv[i],"frac=",5)==0) sscanf(&argv[i][5],"%d/%d",&num,&denom);
		else if (strncmp(argv[i],"mask=",5)==0) mask=atoi(&argv[i][5]);
		else if (strncmp(argv[i],"verbose",7)==0) ver=1;
        else { printf("Unknown parameter '%s'\n",argv[i]); exit(1); }
}

if (num>=denom || num<0) { printf("0 <= num < denom !!!\n"); exit(1); }

// READ Reference Images
nr=fileCount(argv[2]);
dataref=readImages(argv[2],0,nr);		// read all input images
if (dataref==NULL) error("Cannot read input",ERR_EXIT);
printf("Read %d references.\n",dataref->count());

d=(EMData *)dataref->objectAt(0);
x=d->xSize();
y=d->ySize();

for (i=0; i<nr; i++) {
	d=(EMData *)dataref->objectAt(i);
	d->normalize();
}

// prepare cls files for splitting
if (!num) {
	system("/bin/rm cls????.hed cls????.img");
	for (i=0; i<nr; i++) {
		d=(EMData *)dataref->objectAt(i);
		sprintf(s,"cls%04d.hed",i);
		d->writeIMAGIC(s,0);
	}
}

ns=fileCount(argv[1]);
printf("%d images to classify.\n",ns);


d=new EMData;

norm=x*y/1000.0;
norm2=PI/(4.0*x*y);

n0=num*ns/denom;
n1=(num+1)*ns/denom;


// normalizing references
printf("Normalize references\n");
toav=new List;
for (i=n0; i<n0+200 &&i<n1; i++) {
	d->readImage(argv[1],i);
	toav->addObject(d->makeRFP()->copy());
}
av=new EMData;
av->makeAverage(toav);
av->radialAverage();
for (i=0; i<toav->count(); i++) delete (EMData *)toav->objectAt(i);
delete toav;

for (i=0; i<nr; i++) {
	dr=(EMData *)dataref->objectAt(i);
	d2=dr->copy();
	
	dr->radialAverage();
	dr->applyMask(mask,0);
	dr->normalize();
	dr->applyMask(mask,4);
	
	dot=dr->dot(d2);
	dr->multConst(100.0/dot);

	delete d2;
//	printf("Dot %d %f\n",i,dot);
}

printf("processing %d - %d of %d\n",n0,n1,ns);

// align and classify
for (i=n0; i<n1; i++) {
	d->readImage(argv[1],i);
	d->normalize();
	if (d->xSize()!=x ||d->ySize()!=y) { printf("Bad image size %d,%d != %d,%d\n",d->xSize(),d->ySize(),x,y); exit(1); }

	d2=d->copy(0);
	
	best=0;
	bestdot=-1000.0;
	for (j=0; j<nr; j++) {
		dr=(EMData *)dataref->objectAt(j);

		d2->transAlign(dr,1,0,0);
		d2->rotateAndTranslate();
//		dot=(1.0-d2->rotAlignTI(dr))/norm2;
		
		dot=d2->dot(dr);
		
		if (dot>bestdot) { 
			bestdot=dot; 
			best=j;
//printf("better \t%d\t%f\n",best,bestdot/norm);
		}

		if (ver) { putchar('0'+(int)floor(dot/norm/50)); fflush(stdout); }
	}
	if (ver) putchar('\n');

	printf("%d -> %d ( %f ) %c\n",i,best,bestdot/norm,'0'+(int)floor(bestdot/norm/50));

	delete d2;
//	delete df;

	// split into class files
	sprintf(s,"cls%04d.hed",best);
	d->writeIMAGIC(s,-1);
	
}
		
}



