// clsregen.C  8/11/00  Steve Ludtke
// Copyright 2000 Baylor College of Medicine

//N clsregen
//F This program uses particle.log to regenerate cls files for a particular iteration.
//T LC
//
//1
//P <iteration>	The 'refine' iteration number to extract.
//U clsregen 1
//D This program will take the 'particle.log' file, and extract information on
//E the particle classification for a particular iteration of the refine
//E command and regenerate 'cls' files. This can be useful if you wish to run
//E a t-test on a previously completed iteration.
//D Note: This will NOT regenrate the appropriate projections, though it will put in
//E references to proj.hed which will work if it IS generated appropriately. That is,
//E you need to rerun proc3d on the appropriate threed.?a.mrc file.


#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

main(int argc, char *argv[]) {
int targetit=0,i;
appinit(argc,argv);
if (argc<2) appusage(argv[0]);

FILE *in=NULL;

in=fopen("particle.log","rb");
if (!in) error(ERR_EXIT,"particle.log cannot be opened!");
targetit=atoi(argv[1]);

// hunt for the beginning of the last set of classifications with
// the selected iteration number
char s[257],t[257],ll=0;
int n=0,stc=0,ncls=0;
int ptcl,sep,cls,iter;
float tmpf;
while (n++,fgets(s,256,in)) {
	if (s[0]=='P') {
		sscanf(s+1,"%d %f %f %f %d",&ptcl,&tmpf,&tmpf,&tmpf,&iter);
		if (iter==targetit && ptcl>ncls) ncls=ptcl;
	}
	if (s[0]=='C' && ll=='P') {
		sscanf(s+1,"%d %d %s %d %d",&ptcl,&sep,t,&cls,&iter);
		if (iter==targetit) stc=n;
	}
	ll=s[0];
}

rewind(in);
if (stc==0) error(ERR_EXIT,"Requested iteration not present in particle.log");
printf("Iteration %d found at line %d with %d classes\n",targetit,stc,ncls);

// initialize the cls files
for (i=0; i<ncls; i++) {
	sprintf(s,"cls%04d.lst",i);
	FILE *out=fopen(s,"wb");
	if (!out) error(ERR_EXIT,"Cannot open cls file for output.");
	fprintf(out,"#LST\n%d\tproj.hed\n",i);
	fclose(out);
}

// now split up the particles
// very inefficient, should really build up internal arrays then write at once
while (n++,fgets(s,256,in)) {
	if (n<stc) continue;
	if (s[0]=='C') {
		sscanf(s+1,"%d %d %s %d %d",&ptcl,&sep,t,&cls,&iter);
		if (iter!=targetit) continue;
		sprintf(s,"cls%04d.lst",cls);
		FILE *out=fopen(s,"ab");
		if (!out) error(ERR_EXIT,"Cannot open cls file for output.");
		fprintf(out,"%d\tstart.hed\t%s\n",ptcl,t);
		fclose(out);
	}
}
fclose(in);
}
