// eotest.C 1998  Steve Ludtke
// This program splits the data in half and does 2 independent
// reconstructions. This is used to test the resolution of
// a reconstruction.

//N eotest
//F This program performs a 2 way even/odd test to determine the resolution of a reconstruction.
//T H
//1
//P [proc=<min proc>,<maxproc>]	Number of proceesors to use for parallel processing
//P [hard=<maxpr>]	Hard limit for make3d
//P [ctfc=<res in A>]	Performs CTF correction filtered to the specified resolution
//P [ctfcw=<SF file>]	Performs CTF correction with Wiener filtration using a structure factor specified in a file
//P [median]	Specify this when CTF correction is NOT being performed
//P [sym=<c2,c4,etc>]	Imposes symmetry on the model
//P [mask=<rad>]	Applies an outside mask in several stages of refinement
//P [pad=<size>]	Pads the model during Fourier reconstruction
//P [classkeep=<sigma coef>]	Classkeep value for <a href=classalignall.html>classalignall</a>
//P [classiter=<iter>]	Iterations for <a href=classalignall.html>classalignall</a>
//P [precen]	Indicates that raw particles have been accurately precentered
//P [imask=<rad>]	Applies an inside mask in several stages of refinement
//P [xfiles=<a/pix>,<mass in kd>,<ali to>]	Used to generate 3d models that are easy to evaluate
//P [amask=<r>,<thr>,<n iter>]	This applies an automask between steps in a reconstruction. see automask2 in proc3d. REQUIRES xfiles to be used.
//P [3dit=<n it>]	After constructing a model with Fourier reconstruction, this runs several iterations of real-space model 'cleansing' 
//P [3dit2=<n it>]	This is real-space refinement with a special maximum entropy-like feature extraction method 
//P [euler2]	Reassigns Euler angles to class averages using the same step used by refine. ie - 2 will use ang/2 (see <a href=classesrefs.html>classesrefs</a>
//P [fscmp]	Use fsc (frc) to compare images instead of normalized variance
//P [refine]	Refine each alignment to subpixel accuracy
//P [dfilt]	Experimental option which will apply a ~matching filter to the projections during class alignment
//P [dfilt2]	Experimental option which will apply an improved ~matching filter to the projections during class alignment
//P [frm2d[=<rad>,<samp>]]	This will classify using Fast Rotational Matching 2D. rad is the maximum radius of particles; samp is the number of angular sampling points at each radius circle  
//P [refmaskali]	see refine
//P	[iter=<n>]	Select the iteration to do eotest. default to last iteration
//D This program's options parallel <a href=refine.html>refine</a>. 'eotest' options are a
//E subset of refine's options. eotest is used to perform a fairly robust resolution
//E determination on a reconstruction. It is run in the same directory as a completed
//E refine command. Note that the refine command MUST terminate normally. Chances are
//E good that if refine is interrupted in the middle of a run, eotest will not produce
//E valid results. eotest splits each cls file into even and odd halves, then proceeds to
//E build an independent model based on each half of the data. By calculating a Fourier
//E shell correlation between the two resulting maps, a fairly robust resolution determination
//E can be made. For a proper determination, the data should be split into 2, 4, 8, etc sets,
//E then the resolution can be determined by interpolating to the entire data set. The
//E estimate made by comparing 2 halves of the data is actually slightly underestimating
//E the final resolution. Due to the rapid nonlinear falloff in signal to noise ratio
//E at resolution limiting spatial frequencies, this underestimation should be
//E relatively slight in most cases.

#include <stdio.h>
#include <stdlib.h>
//#include <unistd.h>
#include <math.h>
#include <time.h>
#include <glob.h>
#include <ctype.h>
#include "util.h"
#include "EMData.h"

void *cmra(void *fnum);

main(int argc, char *argv[]) {
int i,j, iter=-1;
int restart=0,mask=-1,imask=-1,pad=0,classiter=8,d3r=0,precen=0,refine=0,fscmp=0,rma=0;
int proc=1,median=0,dot=0,mode=2,mra2=0,amask1=0,amask3=0,alito=0,d3it=0,d3it2=0,dfilt=0;
int frm2d=0,ccc=0,size=-1,maxshift=-1;		//ycong
float ang=7.5,keep=1.2,d3s=0,hard=0,range=90.0,ctfc=0,amask2=0.,xfiles=0,appix=0,euler2=0;
float rmaxH=-1.0f;		//ycong
char s[1024],*sym=NULL,*ctfcw=NULL;
FILE *log;
time_t t;

appinit(argc,argv);

log=fopen("refine.log","ab");

if (argc<2) appusage(argv[0]);

int Ref=LOGbegin(argc,argv);
LOG(Ref,"start.hed",LOG_INFILE,NULL);
LOG(Ref,"refine.log",LOG_OUTFILE,NULL);

t=time(0);
if (log) fprintf(log,"******** refine started %s",ctime(&t));
for (i=0; i<argc; i++) fprintf(log,"%s ",argv[i]);
fprintf(log,"\n");

for (i=1; i<argc; i++) {
	if (strncmp("proc=",argv[i],5)==0) proc=atoi(&argv[i][5]);
	else if (strncmp("mask=",argv[i],5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp("imask=",argv[i],6)==0) imask=atoi(&argv[i][6]);
	else if (strncmp("mode=",argv[i],5)==0) mode=atoi(&argv[i][5]);
	else if (strncmp("hard=",argv[i],5)==0) hard=atof(&argv[i][5]);
	else if (strncmp("classkeep=",argv[i],10)==0) keep=atof(&argv[i][10]);
	else if (strncmp("classiter=",argv[i],10)==0) classiter=atoi(&argv[i][10]);
	else if (strncmp("refmaskali",argv[i],10)==0) rma=1;
	else if (strncmp("pad=",argv[i],4)==0) pad=atoi(&argv[i][4]);
	else if (strncmp("d3s=",argv[i],4)==0) d3s=atof(&argv[i][4]);
	else if (strncmp("3dr",argv[i],3)==0) {
		d3r=1;
		if (argv[i][3]=='=') range=atof(&argv[i][4]);
	}
	else if (strncmp("sym=",argv[i],4)==0) sym=&argv[i][4];
	else if (strncmp("precen",argv[i],6)==0) precen=1;
	else if (strncmp("frm2d",argv[i],5)==0) {		//ycong
		frm2d=1;
		if (argv[i][5] == '=') sscanf(argv[i]+6,"%f,%d",&rmaxH,&size);
	}
	else if (strncmp("ccc",argv[i],3)==0) ccc=1;		//ycong
	else if (strncmp("maxshift=",argv[i],9)==0) maxshift=atoi(&argv[i][9]);		//ycong
	else if (strncmp("dfilt2",argv[i],6)==0) dfilt=2;
	else if (strncmp("dfilt",argv[i],5)==0) dfilt=1;
	else if (strncmp("ctfc=",argv[i],5)==0) ctfc=atof(&argv[i][5]);
	else if (strncmp("ctfcw=",argv[i],6)==0) ctfcw=&argv[i][6];
	else if (strncmp("dot",argv[i],3)==0) dot=1;
	else if (strncmp("median",argv[i],6)==0) median=1;
	else if (strncmp("refine",argv[i],6)==0) refine=1;
	else if (strncmp("fscmp",argv[i],5)==0) fscmp=1;
	else if (strncmp("euler2",argv[i],6)==0) {
		if (argv[i][6]!='=') euler2=2.0;
		else { euler2=atof(&argv[i][7]); error(ERR_WARN,"euler2 parameter ignored, 'refine' value used\n"); }
	}
	else if (strncmp("3dit=",argv[i],5)==0) d3it=atoi(&argv[i][5]);
	else if (strncmp("3dit2=",argv[i],6)==0) d3it2=atoi(&argv[i][6]);
	else if (strncmp("xfiles=",argv[i],7)==0) {
		xfiles=atof(&argv[i][7]);
		sscanf(&argv[i][7]," %f , %f , %d",&appix,&xfiles,&alito);
	}
	else if (strncmp("amask=",argv[i],6)==0) {
		sscanf(argv[i]+6,"%d,%f,%d",&amask1,&amask2,&amask3);
	}
	else if (strncmp("iter=",argv[i],5)==0) iter=atoi(&argv[i][5]);
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

t=time(0);
if (log) { fprintf(log,"Beginning T-test %s\n",ctime(&t)); fflush(log); }

//find the last cls.*.tar file and regenerate the last cls*.lst files from it
glob_t globt;
glob("cls.*.tar",NULL,NULL,&globt);
if(globt.gl_pathc) {
	int last_iter=-1, citer, i;
	for(i=0; i<globt.gl_pathc; i++){
		sscanf(globt.gl_pathv[i],"cls.%d.tar", &citer);
		if(iter>0 && citer==iter) { last_iter=iter; break; }
		else if(citer>last_iter)last_iter=citer;
	}
	sprintf(s,"tar xf cls.%d.tar", last_iter); 
	if (log) fprintf(log, "%s\n",s);
	run(s);
}
globfree(&globt);

// align even
sprintf(s,"classalignall even ref1 finalref saverefs keep=%1.2f %d proc=%d mask=%d imask=%d %s%s%s%s%s",
	keep,classiter,proc,mask,imask,median?"median":"",dot?" dot":"",precen?" precen":"",amask1>0?" refmask":"",rma?" refmaskali":" ");
if (ctfc!=0) sprintf(s+strlen(s)," ctfc=%f",ctfc);
if (refine) sprintf(s+strlen(s)," refine");
if (frm2d && rmaxH>0.0 && size>0) sprintf(s+strlen(s)," frm2d=%f,%d",rmaxH,size);
else if (frm2d && rmaxH>0.0) sprintf(s+strlen(s)," frm2d=%f",rmaxH);
else if (frm2d) sprintf(s+strlen(s)," frm2d");
if (maxshift>=0) sprintf(s+strlen(s)," maxshift=%d",maxshift);
if (dfilt==1) sprintf(s+strlen(s)," dfilt");
if (dfilt==2) sprintf(s+strlen(s)," dfilt2");
if (fscmp) sprintf(s+strlen(s)," fsc");
if (ctfcw) sprintf(s+strlen(s)," ctfcw=%s",ctfcw);
if (log) { t=time(0); fprintf(log,"%s   %s",s,ctime(&t)); fflush(log); }
run(s);

rename("classes.img","classes.te.img");
rename("classes.hed","classes.te.hed");
if (log) fprintf(log,"mv classes.hed/img to classes.te.hed/img\n");


// align odd
sprintf(s,"classalignall odd ref1 finalref saverefs keep=%1.2f %d proc=%d mask=%d imask=%d %s%s%s%s%s",
	keep,classiter,proc,mask,imask,median?"median":"",dot?" dot":"",precen?" precen":"",amask1>0?" refmask":"",rma?" refmaskali":" ");
if (ctfc!=0) sprintf(s+strlen(s)," ctfc=%f",ctfc);
if (refine) sprintf(s+strlen(s)," refine");
if (frm2d && rmaxH>0.0 && size>0) sprintf(s+strlen(s)," frm2d=%f,%d",rmaxH,size);
else if (frm2d && rmaxH>0.0) sprintf(s+strlen(s)," frm2d=%f",rmaxH);
else if (frm2d) sprintf(s+strlen(s)," frm2d");
if (maxshift>=0) sprintf(s+strlen(s)," maxshift=%d",maxshift);
if (dfilt==1) sprintf(s+strlen(s)," dfilt");
if (dfilt==2) sprintf(s+strlen(s)," dfilt2");
if (fscmp) sprintf(s+strlen(s)," fsc");
if (ctfcw) sprintf(s+strlen(s)," ctfcw=%s",ctfcw);
if (log) { t=time(0); fprintf(log,"%s   %s",s,ctime(&t)); fflush(log); }
run(s);

rename("classes.img","classes.to.img");
rename("classes.hed","classes.to.hed");
if (log) fprintf(log,"mv classes.hed/img to classes.to.hed/img\n");

// Reassign class eulers if requested
if (euler2>0) {
		const char *t = tmpnam2();
		FILE *out = fopen(t,"wb");
		if (!out) { printf("Cannot open temporary file!\n"); exit(1); }
		
		for (j=0; j<proc; j++) {
			sprintf(s,"eulerproj classes.te.hed proj.e2.hed writeali frac=%d/%d mask=%d %s",j,proc,mask,precen?" precen":"");
			fprintf(out,"%s\n",s);
		}
		fclose(out);
			
		sprintf(s,"runpar proc=%d file=%s",proc,t);
		run(s);
		unlink(t);

		out = fopen(t,"wb");
		if (!out) { printf("Cannot open temporary file!\n"); exit(1); }
		
		for (j=0; j<proc; j++) {
			sprintf(s,"eulerproj classes.to.hed proj.e2.hed writeali frac=%d/%d mask=%d %s",j,proc,mask,precen?" precen":"");
			fprintf(out,"%s\n",s);
		}
		fclose(out);
			
		sprintf(s,"runpar proc=%d file=%s",proc,t);
		run(s);
		unlink(t);
}


// make3d
if (d3r) sprintf(s,"make3dr classes.te.hed out=threed.te.mrc mask=%d range=%f",mask,range);
else sprintf(s,"make3d classes.te.hed out=threed.te.mrc mask=%d",mask);
if (hard) sprintf(s+strlen(s)," hard=%1.2f",hard);
if (d3s) sprintf(s+strlen(s)," keep=%1.2f",d3s);
if (pad) sprintf(s+strlen(s)," pad=%d",pad);
if (sym) sprintf(s+strlen(s)," sym=%s",sym);
if (mode) sprintf(s+strlen(s)," mode=%d",mode);
else sprintf(s+strlen(s)," mode=2");
if (log) { t=time(0); fprintf(log,"%s   %s",s,ctime(&t)); fflush(log); }
run(s);

if (d3r) sprintf(s,"make3dr classes.to.hed out=threed.to.mrc mask=%d range=%f",mask,range);
else sprintf(s,"make3d classes.to.hed out=threed.to.mrc mask=%d",mask);
if (hard) sprintf(s+strlen(s)," hard=%1.2f",hard);
if (d3s) sprintf(s+strlen(s)," keep=%1.2f",d3s);
if (pad) sprintf(s+strlen(s)," pad=%d",pad);
if (sym) sprintf(s+strlen(s)," sym=%s",sym);
if (mode) sprintf(s+strlen(s)," mode=%d",mode);
else sprintf(s+strlen(s)," mode=2");
if (log) { t=time(0); fprintf(log,"%s   %s",s,ctime(&t)); fflush(log); }
run(s);

// clean3d
if (d3it || d3it2) {
	sprintf(s,"clean3d classes.te.hed threed.te.mrc proc=%d 3dit=%d 3dit2=%d",proc,d3it,d3it2);
	if (sym) sprintf(s+strlen(s)," sym=%s",sym);
	run(s);
	if (log) { t=time(0); fprintf(log,"%s   %s",s,ctime(&t)); fflush(log); }

	sprintf(s,"clean3d classes.to.hed threed.to.mrc proc=%d 3dit=%d 3dit2=%d",proc,d3it,d3it2);
	if (sym) sprintf(s+strlen(s)," sym=%s",sym);
	run(s);
	if (log) { t=time(0); fprintf(log,"%s   %s",s,ctime(&t)); fflush(log); }
}

//automask
if (amask1>0 && xfiles) {
	sprintf(s,"volume threed.te.mrc %f set=%f",appix,xfiles);
	if (log) { t=time(0); fprintf(log,"%s   %s",s,ctime(&t)); fflush(log); }
	run(s);
		
	sprintf(s,"proc3d threed.te.mrc threed.te.mrc automask2=%d,%1.3f,%d",amask1,amask2,amask3);
	if (log) { t=time(0); fprintf(log,"%s   %s",s,ctime(&t)); fflush(log); }
	run(s);

	sprintf(s,"volume threed.to.mrc %f set=%f",appix,xfiles);
	if (log) { t=time(0); fprintf(log,"%s   %s",s,ctime(&t)); fflush(log); }
	run(s);
		
	sprintf(s,"proc3d threed.to.mrc threed.to.mrc automask2=%d,%1.3f,%d",amask1,amask2,amask3);
	if (log) { t=time(0); fprintf(log,"%s   %s",s,ctime(&t)); fflush(log); }
	run(s);
}		

// x-files
if (xfiles) {
	sprintf(s,"proc3d threed.te.mrc x.te.mrc clip=%d,%d,%d",mask*2,mask*2,mask*2);
	if (log) { t=time(0); fprintf(log,"%s   %s",s,ctime(&t)); fflush(log); }
	run(s);

	if (i>alito) {
		sprintf(s,"align3d x.%d.mrc x.te.mrc x.te.mrc",alito);
		if (log) { t=time(0); fprintf(log,"%s   %s",s,ctime(&t)); fflush(log); }
		run(s);
	}
	else if (log) fprintf(log,"Alignment skipped\n");
		
	if (xfiles>0) {
		sprintf(s,"volume x.te.mrc %f set=%f",appix,xfiles);
		if (log) { t=time(0); fprintf(log,"%s   %s",s,ctime(&t)); fflush(log); }
		run(s);
	}

	sprintf(s,"proc3d threed.to.mrc x.to.mrc clip=%d,%d,%d",mask*2,mask*2,mask*2);
	if (log) { t=time(0); fprintf(log,"%s   %s",s,ctime(&t)); fflush(log); }
	run(s);

	if (i>alito) {
		sprintf(s,"align3d x.%d.mrc x.to.mrc x.to.mrc",alito);
		if (log) { t=time(0); fprintf(log,"%s   %s",s,ctime(&t)); fflush(log); }
		run(s);
	}
	else if (log) fprintf(log,"Alignment skipped\n");
		
	if (xfiles>0) {
		sprintf(s,"volume x.to.mrc %f set=%f",appix,xfiles);
		if (log) { t=time(0); fprintf(log,"%s   %s",s,ctime(&t)); fflush(log); }
		run(s);
	}

}


run("proc3d threed.te.mrc threed.to.mrc fsc=fsc.eotest");
if (log) { t=time(0); fprintf(log,"%s  proc3d threed.te.mrc threed.to.mrc fsc=fsc.eotest\n",ctime(&t)); }

if (log) { t=time(0); fprintf(log,"%s  EO-test complete\n",ctime(&t)); }
run("rm cls*even*lst cls*odd*lst");
fclose(log);
LOG(Ref,NULL,LOG_END,NULL);
}

void *cmra(void *s)
{
char ss[1024];

strcpy(ss,(char *)s);
run(ss);
return NULL;
}
