// eulerrefine.C 1998 Phil Baldwin

//N eulerrefine
//F This program is used internally by startAny
//T LT
//1
//P <imagefile>	Images to refine
//P <csymnum>	C symmetry number, D symmetry treated like C
//P <NumProcs>	Number of processors to use
//P <Number of Rounds>	Refinement rounds
//P [file=<eulers file,if new>]	Eulers file to use
//D This program is used internally by startAny, and is not usually invoked by itself.

#include <stdio.h>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include <math.h>
#include <time.h>
#include <sys/types.h>
#include <pthread.h>
#include <errno.h>


#define IMODE 0
//             IMODE = 0 means common lines; 1  phase residuals; 2  diff of magnitudes
#define RADIUS 2.5   
// This is the exclusion limit for peak
#define IWRITEer 0
// If 0 means don't write out "qualSquare.img" or "sinecorr.img"
#define MAXCOMPARY 40
#define MAXPARTICLES 1000
#define SKIP 7
#define ISEARCH 3
#define ICOSTILTDMAX 100



struct stuffqualSquareArgs {
	int threadNum;
	int Ntemp;
	int NumProcs;
	int jofinterest;
	int csymnum;
	int CCLxSize;
	int CCLySize;
	int ithetaDmax;
	int icostiltDmax;
};

void* parComLines(void *s);
void* stuffqualSquare(void *s);


//             Globals

int rcom=0;	// real-space common-lines
//             %%       This Program works great ; March 1999   %%
/*
int csymnum;
int ithetaDmax= 360, icostiltDmax= ICOSTILTDMAX; // ithetaDmax should be divisible by 4
int CCLxSize  , CCLySize;	
List *imageList;
EMData *CCLof[MAXPARTICLES];
float *CCLofData[MAXPARTICLES];
float * VecAltqualSquare =  (float *) malloc(icostiltDmax  * sizeof(float)  );
int  * VecAltiphiD       =  ( int *) malloc(icostiltDmax  * sizeof(  int)  );
int  * VecAltithetaD     =  ( int *) malloc(icostiltDmax  * sizeof(  int)  );
*/



/****************************************************************/
main(int argc, char *argv[] ) {


appinit(argc,argv);


int i,j, jofinterest, jofinterestmin, irepeat,iq;
int ithetaD,     iphiD, icostiltD;
int ithetaDbest, iphiDbest, icostiltDbest, iphiDtemp;
int dbaz, bdaz,bdphi, ihollow, itemp;
int Ntemp, m, partSize, Nmax;

int ithread;

float piover= PI/180.,quality;
float thetaDbest, phiDbest, tiltDbest;

float adaz, baaz, adphi, baphi,alpha,beta,gamma;

time_t preAngTime, postAngTime, startTime, endTime;


int csymnum;
int ithetaDmax= 360, icostiltDmax= ICOSTILTDMAX; // ithetaDmax should be divisible by 4
int CCLxSize  , CCLySize;	
EMData *CCLof[MAXPARTICLES];
float * VecAltqualSquare =  (float *) malloc(icostiltDmax  * sizeof(float)  );
int  * VecAltiphiD       =  ( int *) malloc(icostiltDmax  * sizeof(  int)  );
int  * VecAltithetaD     =  ( int *) malloc(icostiltDmax  * sizeof(  int)  );
int alreadyRefinedABit   = 0;


Euler ad, ab, ba, bd;
stuffqualSquareArgs toPass[24];
FILE *tmpAngles; 


//          eulerrefine.parallel.C

//

if (argc<=4 || argc >=10 ) { printf("eulerrefine\n");
	printf("Usage: eulerrefine.parallel <imagefile> \n \
	<beginning particle to angle> <ending particle to angle> <ending anchor value> \
	\n <csymnum><NumProcs><which Round>[file=<eulers file,if new>]\n"); 
//	printf("    if csymnum =4, it will not  even try to read in the eulers file \n"); 
exit(0); }

// 
//   

fflush(stdout);


int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);


startTime = time(0);


		unlink(".lock..tmp.sinecorr.img"); unlink(".lock..tmp.sinecorr.hed"); 
		unlink("z.img"); unlink("z.hed"); 

// *******************   %%    0. Read in the Images   %%  *******************

List *imageList=readImages(argv[1],0,-1);
int N=imageList->count(); 
int MAXCOMPARE=atoi(argv[4]);
csymnum=atoi(argv[5]);
int NumProcs = atoi(argv[6]);// if (NumProcs >10) NumProcs=10;// NumProcs=1;
int whichRound = atoi(argv[7]);

pthread_t *thrd;
thrd = (pthread_t *)malloc(sizeof(pthread_t)*NumProcs);

if (N > MAXPARTICLES ) { printf(" Too Many Particles!!!! \n"); exit(0); }


// pthread_setconcurrency(1.5*NumProcs);


printf("Images have been read \n");

// *******************  %%  0.5 symmetrize  first particle in list %% *******************


EMData *tmpcopy, *tmp;

tmpcopy =((EMData *)imageList->objectAt(0))->copy();
tmp     =((EMData *)imageList->objectAt(0))->copy();

	for (float f=2.0*PI/(float)csymnum; f<(2.0-1.0/(float)csymnum)*PI; f+=2.0*PI/(float)csymnum) {
		tmp->setRAlign(f,0,0);
		tmp->rotateAndTranslate();
		tmpcopy->add(tmp) ;
	}

	tmpcopy->normalize();
	imageList->replaceObjectAt(0,tmpcopy);
	tmpcopy ->writeIMAGIC(argv[1],0);	
//	tmpcopy ->writeIMAGIC("z.img",-1);

printf(" just symmetrized first particle \n  ");

// ***************     %%1. Edge normalize and Create the particles (IM) %%

EMData *IM[MAXPARTICLES];
Euler *Oa = (Euler *) calloc(N,sizeof(Euler));

for (i=0; i<N; i++) {
	EMData *t=(EMData *)imageList->objectAt(i);
	if (i==0) partSize = t->xSize();
	t->edgeNormalize();
	EMData *u = t->clip(-partSize/2,-partSize/2,partSize*2,partSize*2);
	delete t;
	imageList->replaceObjectAt(i,u);
//	delete u;
}
// printf(" just edge normalized  N=%d \n  ",N);fflush(stdout);
 
for (i=0; i<N; i++) {
	((EMData*)imageList->objectAt(i))->toCorner();
	IM[i] = (EMData*)imageList->objectAt(i);
	Oa[i] = *((*IM[i]).getEuler());
	
//		printf("alt = %f, az =%f, phi =%f \n",
//               Oa[i].alt()/piover,Oa[i].az()/piover,Oa[i].phi()/piover);
	}
//	if (Oa[N-1].alt() != 0) alreadyRefinedABit = 1;
	
	
	
	
//  

//printf(" just edge normalized \n  ");fflush(stdout);

//  ******    2. Read in euler file, if necessary   *****

float altnew, aznew, phinew;


if (argc==7) {
	FILE *eulersplt=fopen(argv[8],"rb");
	for (i=0; i < 3; i++){
		fscanf(eulersplt,"%d	%f	%f	%f	\n",&j,&altnew,&aznew,&phinew); // j doesn't do anything here
		while(aznew >= 360./csymnum) aznew +=- 360./csymnum;
		while(phinew >= 360./csymnum) phinew +=- 360./csymnum;
		Oa[i].setAngle(altnew*piover,aznew*piover,phinew*piover);
	}
}

printf(" Finished reading in eulers file, ithetaDmax = %d, csymnum=%d\n",ithetaDmax,csymnum);

	
for (i=0; i<3; i++) {
//	printf("%d	%f	%f	%f	\n",i+1,Oa[i].alt()/piover,Oa[i].az()/piover,Oa[i].phi()/piover);
}
//   ********** %% 3. Find out size of sinecorrs; give shape of qualitystrip %%

unlink("CCL.img");
unlink("CCL.hed");
unlink("core");

float *CCLData; // 
EMData *CCL = new EMData,*tmp2 = new EMData;
float CCLDataMax = .00;

	if (rcom) {
		CCL->commonLines(IM[1],IM[3]);
		CCL->realFilter(0,0);
		tmp2->commonLinesR(IM[1],IM[3]);
		tmp2->realFilter(0,0);
		CCL->mult(tmp2);
	}
	else CCL -> commonLines(IM[1],IM[3],IMODE);   // 1 means use phase residuals
	delete tmp2;

	CCLxSize = (*CCL).xSize()  ; CCLySize= (*CCL).ySize();	
	printf("CCLxSize = %d ,m CCLySize = %d \n", CCLxSize, CCLySize);
	(*CCL).doneData();
	(*CCL).writeIMAGIC("CCL.hed",-1);
			printf("finished CCL of %d and 4 \n ", j+1);
	

if (IWRITEer) {	 unlink("qualSquare.img"); 
				unlink("qualSquare.hed"); }
//printf("just unlinked qualSquare \n");fflush(stdout);



//   ********** %% 4. Start jofinterest monster loop!!!! %%


float *sinecorr;

for (irepeat = alreadyRefinedABit ; irepeat < 1+ alreadyRefinedABit; irepeat++){

	jofinterestmin=3;
	if (irepeat >0) jofinterestmin = 1;
	jofinterestmin=atoi(argv[2])-1;
	Nmax=atoi(argv[3]);  // maximum of particles to anchor
//	if (irepeat==alreadyRefinedABit) Nmax=jofinterest+1;

	for (jofinterest= jofinterestmin; jofinterest < Nmax; jofinterest++) {  // particles to anchor
										
//Usage: eulerrefine.parallel <imagefile> 
//        <beginning particle to angle> <ending particle to angle> <ending anchor value>  
//<csymnum><NumProcs><Number of Rounds>[file=<eulers file,if new>]

		if (whichRound==0) { 
			Ntemp = jofinterest ; 
			if (MAXCOMPARY < Ntemp) Ntemp=MAXCOMPARY;}
		

		if (whichRound!=0) {
			Ntemp = MAXCOMPARE;
			if (MAXCOMPARY < Ntemp) Ntemp=MAXCOMPARY;}
	
//         " "        then check only the values <= jofinterest

// *********** %% 4.1 Create a table of sinecorrs %%

		EMData *IMjofint = (EMData*)imageList->objectAt(jofinterest);

		printf(" Ntemp= %d \n",Ntemp);fflush(stdout);

		for (j=0; j <Ntemp ; j++){  // the anchors we use
			IM[j] = (EMData*)imageList->objectAt(j);
			CCLof[j]=new EMData;

			if (rcom) {
				EMData *tmp3 = new EMData;
				CCLof[j]->commonLines(IMjofint,IM[j]);
				CCLof[j]->realFilter(0,0);
				tmp3->commonLinesR(IMjofint,IM[j]);
				tmp3->realFilter(0,0);
				CCLof[j]->mult(tmp3);
			}
			else CCLof[j]->commonLines(IMjofint,IM[j],IMODE);   // 1 means use phase residuals to calculate
//												//  Notice the order IMjofint, IM. Makes data
			//delete tmp3;

			if (j==0 && jofinterest==jofinterestmin) 
					printf("Round %d \n",whichRound);
//				if ( (j==0) || (j==Ntemp-1) || (j==MAXCOMPAR-1)	)
			printf(" finished sinecorr twixt %d and %d, total sinecorrs= %d \r",
				jofinterest+1,j+1,j+1);	fflush(stdout);	
		} // end j
		printf(" \n");
 

		unlink(".lock..tmp.sinecorr.hed");unlink(".lock..tmp.sinecorr.img");
		unlink(".tmp.sinecorr.img"); unlink(".tmp.sinecorr.hed"); 
		unlink("self.img");     unlink("self.hed"); 


///////////////////////////////////////////////////////////////////////////////
//            %%  Prepare to Paralyze

		for (j=0; j< Ntemp  ; j++){   // Change me!!!
			if (1)	{
					CCLof[j]->filter(0,20,1);
					if (j== jofinterest) CCLof[j]->writeIMAGIC(".tmp.sinecorr.img",-1);
					else  CCLof[j]->writeIMAGIC(".tmp.sinecorr.img",-1);}
		}
//		if (irepeat>0) sleep(100);

		if (jofinterest==jofinterestmin) printf("\n Finished sinecorr table, CCLxSize= %d \n", CCLxSize);


		unlink(".tmp.angleFile");  
		unlink(".tmp.qualJazz");  

		tmpAngles=fopen(".tmp.angleFile","wb");
		for (i=0; i < Ntemp; i++){
			fprintf(tmpAngles,"%d	%f	%f	%f	\n",i,Oa[i].alt(),Oa[i].az(),Oa[i].phi());fflush(tmpAngles);
//			printf("%d	%f	%f	%f	\n",i,Oa[i].alt(),Oa[i].az(),Oa[i].phi());fflush(stdout);
		}
//  %% P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2%%
// ********************************************************   //
//          %%    4.2   Now start guessing angles

		if (jofinterest==jofinterestmin) 
			printf("  Should make ithetaDmax*itiltmax*CCLxSize = %4.2f million guesses. \n", 
  				((float)ithetaDmax*icostiltDmax*CCLxSize)/1000000);

preAngTime = time(0);

		for (int ipass = 0; ipass < NumProcs; ipass++){
			toPass[ipass].threadNum = ipass ;
			toPass[ipass].Ntemp     = Ntemp;
			toPass[ipass].NumProcs = NumProcs;
			toPass[ipass].jofinterest = jofinterest;
			toPass[ipass].csymnum     =  csymnum;
			toPass[ipass].CCLxSize = CCLxSize;
			toPass[ipass].CCLySize = CCLySize;
			toPass[ipass].ithetaDmax = ithetaDmax;
			toPass[ipass].icostiltDmax  = icostiltDmax;
			}

//		stuffqualSquare(toPass);

		
		for (ithread=0; ithread < NumProcs;  ithread++){ 
			pthread_create(&thrd[ithread],NULL,stuffqualSquare,&toPass[ithread]);
		}
						
		for (ithread=0; ithread < NumProcs; ithread++) pthread_join(thrd[ithread],NULL);


		unlink(".tmp.angleFile");  
		unlink(".tmp.sinecorr.img"); unlink(".tmp.sinecorr.hed"); 

postAngTime = time(0);
printf("time spent angling = %d seconds \n",postAngTime-preAngTime);fflush(stdout);

//  %% P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2P2%%

			FILE *qualJazz=fopen(".tmp.qualJazz","rb");

			for (int icostiltDcount =0;icostiltDcount < icostiltDmax; icostiltDcount++) {
				fscanf(qualJazz,"%d",&icostiltD);
				fscanf(qualJazz,"%f	%d	%d	\n",&VecAltqualSquare[icostiltD],
					&VecAltiphiD[icostiltD],&VecAltithetaD[icostiltD]  );
			}

//			sleep(30);

			unlink(".tmp.qualJazz");
			
			quality = -500;
	
			for (icostiltD=0; icostiltD < icostiltDmax; icostiltD++){ // changeme!! Make sure I'm OK
				if ( VecAltqualSquare[icostiltD] > quality) { 
					quality= VecAltqualSquare[icostiltD];  
					iphiDbest   =  VecAltiphiD[icostiltD] ;  
					ithetaDbest = VecAltithetaD[icostiltD] ;
					icostiltDbest  = icostiltD;
				} // ends quality
			}	// ends icostiltD

//			if (jofinterest ==0 ) ithetaD += ithetaDmax; // printf("funny \n");

		tiltDbest = 180./PI*acos(1.-  ((float)icostiltDbest)/((float)icostiltDmax)      );
		thetaDbest= ((float)ithetaDbest*360)/((float)ithetaDmax);
		phiDbest  =	((float)iphiDbest*360)/((float)CCLxSize) ;


		printf(" particle  %d has eulers   \n", jofinterest+1);
		printf(" alt = %3.3f,\t az = %3.3f,\t phi = %3.3f,\t quality= %2.3f \n",
			tiltDbest,thetaDbest,phiDbest,quality/((float) (csymnum*Ntemp-1) ));

//  Now change the eulerangles of jinterest

		Oa[jofinterest].setAngle(tiltDbest*piover,thetaDbest*piover,phiDbest*piover);

// *************   %% 4.7 Start tidying up %%
		printf("start tidying up \n ");
		for (j=0; j< Ntemp  ; j++){   // Change me!!!
			CCLof[j] -> doneData();
			delete CCLof[j];
		}// ends j              

	}  // ends jofinterest loop
 
	for (i=0; i < N; i++){
		printf("%d	%f	%f	%f	\n",i+1,Oa[i].alt()/piover,Oa[i].az()/piover,Oa[i].phi()/piover);
	} // ends i print loop

	if (irepeat == 0) {
		if (IWRITEer) {
					printf("written qualSquare \n"); fflush(stdout); }
	} // ends if irepeat


} // ends repeat loop 

// ****************** %% 5. Write out imagic files correctly %%
printf("just finished main loop \n"); fflush(stdout);

List *imageListfinal = readImages(argv[1],0,-1);

EMData *IMtemp;

//imageListfinal->replaceObjectAt(0,tmpcopy);
	

for (i=0; i < N ; i++) {    // Don't touch the first one
//	 	if (i==0) IMtemp=tmpcopy ;
//		else 
			IMtemp=(EMData*)imageListfinal->objectAt(i);
		IMtemp->setRAlign(Oa[i].alt() , Oa[i].az() , Oa[i].phi() );
		IMtemp->setNImg(1);
		IMtemp->writeIMAGIC(argv[1],i);	
	
}
endTime = time(0);
printf("total elapsed time =  %d \n", endTime-startTime);

	
LOG(Ref,argv[1],LOG_OUTFILE,NULL);
LOG(Ref,NULL,LOG_END,NULL);


} // ends program

void* stuffqualSquare(void *p){
stuffqualSquareArgs v = *(stuffqualSquareArgs *) p;

	int ithread = v.threadNum;
	int Ntemp = v.Ntemp;
	int NumProcs = v.NumProcs;
	int jofinterest = v.jofinterest;
	int csymnum = v.csymnum;
	int CCLxSize = v.CCLxSize;
	int CCLySize = v.CCLySize;
	int ithetaDmax = v.ithetaDmax;
	int icostiltDmax  = v.icostiltDmax;
	char s[1024];

	sprintf(s,"stuffqualSquareGuts %d %d %d %d %d %d %d %d %d",
		ithread,Ntemp,NumProcs,jofinterest,csymnum,CCLxSize,
			CCLySize,ithetaDmax,icostiltDmax);

	system(s);

	return NULL;
}

