#!/usr/bin/env python
###	extractmonomers.py	Steven Ludtke	05/22/2007
### This program will extract monomers from individual particles
###

#N extractmonomers.py
#F This program will extract individual monomers from individual symmetric particles using masks after normal refinement
#T X
#1
#P <cls file>	Path to a .cls file to extract particles from
#U extractmonomers.py <cls file> <3d model> <monomer mask> <particle-monomer mask> <symmetry> <sf file> [dump] [mask]
#D After running a normal refinement with symmetry imposed, it may be interesting to investigate
#E dynamics of individual monomers. This program will take a refined .cls file and extract
#E all of the monomers from each particle by subtracting away the unwanted density (with
#E CTF taken into account).  The result will be a stack of monomer images which can be used for
#E further processing.
#D This program is HIGHLY EXPERIMENTAL !  Contact Steve Ludtke before using.


from EMAN import *
from sys import argv
from math import *
from sys import exit

if len(argv) <7:
	print "extractmonomers.py <cls file> <3d model> <monomer mask> <particle-monomer mask> <symmetry> <sf file> [<newbox>] [dump] [mask]"
	exit(1)

appinit(argv)

clspath=argv[1]
nimg=fileCount(clspath)[0]-1

model=EMData()
model.readImage(argv[2],-1)

# mask for a monomer
maskmono=EMData()
maskmono.readImage(argv[3],-1)

# mask for the whole structure
maskrest=EMData()
maskrest.readImage(argv[4],-1)

#print model.zSize(),maskrest.zSize()
modelrest=model.copy(0)
modelrest.mult(maskrest)
modelrest.writeImage("modelrest.mrc")

sym=argv[5]

sffile=argv[6]
sf=XYData()
sf.readFile(sffile)
sf.logy()

oldbox=model.xSize()
try: newbox=int(argv[7])
except: newbox=oldbox

# dumps a lot of intermediate images to 'dump.img'
if "dump" in argv[7:] : dodump=1
else : dodump=0

# applies a per-monomer mask to extracted monomers
if "mask" in argv[7:] : domask=1
else : domask=0

oproj=EMData()
oproj.readImage(clspath,0)
euler=oproj.getEuler()

euler.setSym(sym)
for i in range(1,nimg+1):
	print "\r%d    "%i
	# read the particle and preprocess
	ptcl=EMData()
	ptcl.readImage(clspath,i)
	ptcl.edgeNormalize()
	
	# transform into 'projection' orientation
	ptcl.rotateAndTranslate()
	if ptcl.isFlipped() : ptcl.hFlip()

	
	# power spectrum of particle
	ptclf=ptcl.doFFT()
	ptclpow=[]
	ptclf.calcRadDist(ptcl.xSize()/2,0,1.,ptclpow)
	
	# projection of the unmasked model
	proj=model.project3d(euler.alt(),euler.az(),euler.phi(),-6)
	projf=proj.doFFT()
	projpow=[]
	projf.calcRadDist(ptcl.xSize()/2,0,1.,projpow)
	
	# CTF curve (4 is absolute SNR,5 is Wiener filter)
	ctf=ptcl.ctfCurve(5,sf)
	
	# Now compute a filter to best match the projection to the particle
	filt=[]
	for i in range(len(ptclpow)): 
		filt.append(sqrt(ctf[i*5]*ptclpow[i]/projpow[i]))	# the *5 is from the CTFOS #define
	
	projf.applyRadFn(len(filt),0,1.0,filt)
	projfilt=projf.doIFT()
	norm=projfilt.normalizeTo(ptcl)

	e1=euler.FirstSym()
	while (e1.valid()):
		if (dodump) :
			ptcl.writeImage("dump.hed",-1)
			proj.writeImage("dump.hed",-1)
		projrest=modelrest.project3d(e1.alt(),e1.az(),e1.phi(),-6)
		projmask=maskmono.project3d(e1.alt(),e1.az(),e1.phi(),-6)
		projmask.realFilter(2,.01)
		
		# power spectrum of projection	
		projrestf=projrest.doFFT()
		
		projrestf.applyRadFn(len(filt),0,1.0,filt)
		projrestfilt=projrestf.doIFT()		# now we apply the same normalization to the masked out
		projrestfilt.multConst(norm[0])		# projection
		projrestfilt.addConst(norm[1])
		
		if (dodump) :
			projfilt.writeImage("dump.hed",-1)
			psub=ptcl-projfilt
			psub.writeImage("dump.hed",-1)
			projrestfilt.writeImage("dump.hed",-1)
#		projfilt.writeImage("outf.hed",-1)
		
		
		psub2=ptcl-projrestfilt
		psub2.edgeNormalize()
		if (dodump) : psub2.writeImage("dump.hed",-1)
		
		if (domask) : psub2.mult(projmask)
		projmask.cmCenter()
		psub3=psub2.clip(int((oldbox-newbox)/2-projmask.Dx()),int((oldbox-newbox)/2-projmask.Dy()),newbox,newbox)
		psub3.writeImage("monomers.hed",-1)
		
		e1=euler.NextSym()
		
	#	print psub.Sigma(),psub2.Sigma()
		
		#writelist("out.pt.txt",ptclpow,0,1.)
		#writelist("out.pr.txt",projpow,0,1.)
		#writelist("out.fl.txt",filt,0,1.)
		#writelist("out.ct.txt",ctf,0,0.2)
		#break
