// fftavg.C  1997 Steve Ludtke
// Copyright 1997 Baylor College of Medicine

//N fftavg
//F This program reads a set of images, transforms them, and generates and averaged power spectrum
//T LT
//1
//P <apix>	A/pixel
//P <image file>	Reads all images from a multi-image file
//2
//P <apix>	A/pixel
//P <Image 1>	First image
//P <Image 2>	Second image
//P	etc.
//U fftavg im*.mrc
//U fftavg 2248.img
//D This program reads a set of images (all the same size), does an FFT of each, then
//E averages them to generate an approximate power spectrum. This functionality is already
//E incorporated into <a href=ctfit.html>ctfit</a>, but this program may still be useful
//E for offline calculations. An x/y plot of the power spectrum is dumped to stdout, and
//E the averaged image is output to fftavg.tnf (a complex MRC file)

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

#define POINTS 200

main(int argc, char *argv[]) {
int i,j,k,n;
float y[POINTS+1];
EMData *d,*dd,*av;
float *d2,apix;

appinit(argc,argv);

if (argc<3) appusage(argv[0]);

apix=atof(argv[1]);

n=fileCount(argv[2]);
av=new EMData;
if (av->readImage(argv[2],0)) exit(1);
av->normalize();
av=av->doFFT();

if (n>1) {
	printf("Averaging %d images from one file\n",n);
	for (i=1; i<n; i++) {
		printf("%d\r",i);
		fflush(stdout);
		d=new EMData;
		if (d->readImage(argv[2],i)) { printf("Cannot read %s\n",argv[i]); exit(1); }
		d->normalize();
		dd=d->doFFT();
		av->addIncoherent(dd);
		delete d;
	}
} else {
	printf("Averaging %d images\n",argc-1);
	for (i=3; i<argc; i++) {
		printf("%d\r",i);
		fflush(stdout);
		d=new EMData;
		if (d->readImage(argv[i],0)) { printf("Cannot read %s\n",argv[i]); exit(1); }
		d->normalize();
		dd=d->doFFT();
		av->addIncoherent(dd);
		delete d;
	}
}

av->writeMRC("fftavg.tnf");
av->calcRadDist(POINTS,0,av->xSize()/((float)POINTS*2.0),y);

//for (i=0; i<POINTS; i++) printf("%f\t%4.6g\n",(1/(2.0*apix))*i/(float)POINTS,y[i]);
save_data(0,(1/(2.0*apix))*1.0/(float)POINTS,y,POINTS, "fftavg.txt");
}
