// footprint.C - Copyright 1998 Baylor College of Medicine
// 9/11/98 Steven Ludtke

//N footprint
//F This program generates a set of rotational and translational invariants for use in classification
//T LT
//1
//P <input file>	Input file with image set to be 'footprinted'
//P <output file>	Output file for binary footprints
//P [euler]	Also writes euler angles to binary footprint file
//U footprint start.img start.fp euler
//D This program determines a set of rotational and translational invariants for
//E a set of images and writes them to a binary file. There are several different
//E footprint programs. Each one uses a different technique for generating
//E invariants. All of these programs are classified as experimental, and are
//E not used in the normal refinement procedure.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

//#define NUMK	2
//#define NUMA	5
//#define NUMR	10
//#define NUMFP	(NUMA*NUMR)
#define NUMFP 20	// actually NUMFP-1 footprints

main(int argc, char *argv[]) {
int i,j,k,n,n2,nx,ny,euler=0,dump=0;
EMData *d,*d2,*r,*r1,*r2;
char s[120];
float *mx[NUMFP],*eul[3];
float a;
FILE *out;

appinit(argc,argv);

if (argc<3) appusage(argv[0]);

if (argc>3&&strcmp(argv[3],"euler")==0) euler=1;
for (i=3; i<argc; i++) {	
	if (strncmp(argv[i],"euler",5)==0) euler=1;
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

if (strlen(argv[1])>119) printf("WARNING: Path over 120 char will cause problems later.\n");

int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);

n=fileCount(argv[1]);
if (n<1) { printf("No data\n"); exit(0); }

d=new EMData;
d->readImage(argv[1],0);
nx=d->xSize();
ny=d->ySize();

if (nx!=ny) { printf("nx!=ny\n"); exit(1); }

for (i=0; i<NUMFP; i++) { mx[i]=(float*)malloc(sizeof(float)*n);  }
if (euler) {
	for (i=0; i<3; i++) eul[i]=(float *)malloc(sizeof(float)*n);
}

printf("Calculate footprints\n");

d=new EMData;
// calculate footprints
for (i=0; i<n; i++) {
	if (d->readImage(argv[1],i)) printf("Image read failed at %d\n",i);
	if (euler) {
		eul[0][i]=d->alt();
		eul[1][i]=d->az();
		eul[2][i]=d->phi();
	}
	d->applyMask(-1,6);
	d->edgeNormalize();
	d2=d->clip(-nx/2,-ny/2,nx*2,ny*2);

	r=d2->makeRFP();
	r->normalize();
	r->applyMask(-1,6);
	r1=r->copy();
	r2=r->copy();
	
	for (j=0; j<NUMFP; j++) {
//		a=PI/((float)(j+2)*2.0);
		a=(float)(j+1)*(PI/4.0)/(NUMFP+1);
		r1->setRAlign(a,0,0);
		r1->rotateAndTranslate();
		r2->setRAlign(-a,0,0);
		r2->rotateAndTranslate();
//		mx[j][i]=r1->dot(r2);
		mx[j][i]=r1->lcmp(r2);
//		r1->writeIMAGIC("out.hed",-1);
//		r2->writeIMAGIC("out.hed",-1);		
	}

	delete d2;
	delete r1;
	delete r2;

	if (i%50==0) { printf(" %d done\r",i); fflush(stdout); }
}
printf("%d done\n",n);

// output log of values for better scaling properties
//for (i=0; i<n; i++) {
//	for (j=0; j<NUMFP; j++) mx[j][i]=log(mx[j][i]);
//}

// center each component
/*
for (i=0; i<NUMFP; i++) {
	a=0;
	for (j=0; j<n; j++) a+=mx[i][j];
	a/=(float)n;
	for (j=0; j<n; j++) mx[i][j]-=a;
}
*/

// Normalize vectors
for (i=0; i<n; i++) {
	a=0;
	for (j=0; j<NUMFP; j++) a+=SQR(mx[j][i]);	// { a+=mx[j][i]*mx[j][i]; }
	a=sqrt(a);
	for (j=0; j<NUMFP; j++) mx[j][i]/=a;
}


// make axes equivalent, ie std dev of each -> 1 
/*
for (i=0; i<NUMFP; i++) {
	t=0;
	for (j=0; j<n; j++) t+=mx[i][j]*mx[i][j];

	t=sqrt(t/(float)n);	// std dev
	
	for (j=0; j<n; j++) mx[i][j]/=t;
}
*/

// write footprint file
// 120 byte string (filespec)
// 4 byte integer (n)
// 4 byte integer (n2 elements/image)
// n2 arrays of n, 4 byte floats
out=fopen(argv[2],"wb");
strncpy(s,argv[1],119);
s[119]=0;
fwrite(s,120,1,out);
fwrite(&n,4,1,out);
n2=NUMFP;
fwrite(&n2,4,1,out);
for (i=0; i<NUMFP; i++) fwrite(mx[i],sizeof(float)*n,1,out);
if (euler) {
	for (i=0; i<3; i++) fwrite(eul[i],sizeof(float)*n,1,out);
}
fclose(out);

LOG(Ref,argv[2],LOG_OUTFILE,NULL);
LOG(Ref,NULL,LOG_END,NULL);

// similarity matrix
/*
for (i=0; i<n; i++) {
	for (j=0; j<n; j++) {
		r=0;
		for (k=0; k<16; k++) r+=mx[k][j]*mx[k][i];
		printf("%3.0f",floor(r*49.999)+50.0);
	}
	printf("\n");
}
*/

}
