// footprintr.C  Copyright 1999 Baylor College of Medicine
// 2/3/99 Steven Ludtke
// This program generates rotational 'footprints' for a set of images.
// that is, a set of rotationally invariant parameters for each image
//N footprintr
//F This program generates a set of rotational and translational invariants for use in classification
//T LT
//1
//P <input file>	Input file with image set to be 'footprinted'
//P <output file>	Output file for binary footprints
//P [euler]	Also writes euler angles to binary footprint file
//U footprint start.img start.fp euler
//D This program determines a set of rotational and translational invariants for
//E a set of images and writes them to a binary file. There are several different
//E footprint programs. Each one uses a different technique for generating
//E invariants. All of these programs are classified as experimental, and are
//E not used in the normal refinement procedure.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

#define NUMA	10
#define NUMR	10
#define NUMFP	(NUMA*NUMR)

main(int argc, char *argv[]) {
int i,j,k,n,n2,x,y,nx,ny,euler=0,mask=-1,imask=2,dumprefs=0;
float t,r,w,sc,c[NUMFP],*eul[3];
EMData *d,*e,*ref[2][NUMFP];
float *data;
char s[120];
float f,*mx[NUMFP];
FILE *out;

appinit(argc,argv);

if (argc<3) appusage(argv[0]);

for (i=3; i<argc; i++) {	
	if (strncmp(argv[i],"mask=",5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"imask=",5)==0) imask=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"euler",5)==0) euler=1;
	else if (strncmp(argv[i],"dumprefs",8)==0) dumprefs=1;
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

if (strlen(argv[1])>119) printf("WARNING: Path over 120 char will cause problems later.\n");

n=fileCount(argv[1]);
if (n<1) { printf("No data\n"); exit(0); }

d=new EMData;
d->readImage(argv[1],0);
e=d->makeRFP(0,0);
nx=e->xSize();
ny=e->ySize();

if (mask==-1) mask=nx/2;

if (nx!=ny) { printf("nx!=ny\n"); exit(1); }

int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);

if (dumprefs) {
	unlink("refs.fp.hed");
	unlink("refs.fp.img");
}

printf("Preparing references\n");

w=(mask-imask)/(float)(NUMR+1);		// barely overlapping gaussians
for (i=0; i<NUMR; i++) {
	for (j=0; j<NUMA; j++) {
		ref[0][i+j*NUMR]=new EMData;
		ref[0][i+j*NUMR]->setSize(nx,ny,1);
		data=ref[0][i+j*NUMR]->getData();
		sc=(i+1)*w+imask;
		// sc is the radius at the center of each band
		for (y=0; y<ny; y++) {
			for (x=0; x<nx; x++) {
				t=atan2((float)(y-ny/2),(float)(x-nx/2));
				r=hypot((float)(y-ny/2),(float)(x-nx/2))-sc;
				data[x+y*nx]=sin(t*j)*exp(-r*r/(w*w/2.4))/sc;
			}
		}
		ref[0][i+j*NUMR]->doneData();
		ref[0][i+j*NUMR]->update();
//		ref[0][i+j*NUMR]->normalize();		
		if (dumprefs) ref[0][i+j*NUMR]->writeIMAGIC("refs.fp.hed",-1);

		ref[1][i+j*NUMR]=new EMData;
		ref[1][i+j*NUMR]->setSize(nx,ny,1);
		data=ref[1][i+j*NUMR]->getData();
		sc=(i+1)*w;
		// sc is the radius at the center of each band
		for (y=0; y<ny; y++) {
			for (x=0; x<nx; x++) {
				t=atan2((float)(y-ny/2),(float)(x-nx/2));
				r=hypot((float)(y-ny/2),(float)(x-nx/2))-sc;
				data[x+y*nx]=cos(t*j)*exp(-r*r/(w*w/2.4))/sc;
			}
		}
		ref[1][i+j*NUMR]->doneData();
		ref[1][i+j*NUMR]->update();
//		ref[1][i+j*NUMR]->normalize();		
		if (dumprefs) ref[1][i+j*NUMR]->writeIMAGIC("refs.fp.hed",-1);
	}
}

for (i=0; i<NUMFP; i++) { mx[i]=(float*)malloc(sizeof(float)*n); c[i]=0; }
if (euler) {
	for (i=0; i<3; i++) eul[i]=(float *)malloc(sizeof(float)*n);
}

printf("Calculate dot products\n");
// calculate image dot products
for (i=0; i<n; i++) {
	if (d->readImage(argv[1],i)) printf("Image read failed at %d\n",i);
	d->edgeNormalize();
	e=d->makeRFP(0,0);
	e->normalize();
	if (euler) {
		eul[0][i]=d->alt();
		eul[1][i]=d->az();
		eul[2][i]=d->phi();
	}
	for (j=0; j<NUMFP; j++) {
		mx[j][i]=hypot(e->dot(ref[0][j]),e->dot(ref[1][j]));
		c[j]+=mx[j][i];
	}
	if (i%50==0) { printf(" %d done\r",i); fflush(stdout); }
	delete e;
}
printf("%d done\n",n);


for (i=0; i<NUMFP; i++) c[i]/=(float)n;	// this is the center of the NUMFP dim object

// translate each to center and normalize
/*for (i=0; i<n; i++) {
	r=0;
	for (j=0; j<NUMFP; j++) { mx[j][i]-=c[j]; r+=mx[j][i]*mx[j][i]; }
	r=sqrt(r);
	for (j=0; j<NUMFP; j++) { 
		mx[j][i]/=r; 
	}
}
*/

// translate each to center

for (i=0; i<n; i++) {
	for (j=0; j<NUMFP; j++) mx[j][i]-=c[j];
}

// make axes equivalent, ie std dev of each -> 1 
/*
for (i=0; i<NUMFP; i++) {
	t=0;
	for (j=0; j<n; j++) t+=mx[i][j]*mx[i][j];

	t=sqrt(t/(float)n);	// std dev
	
	for (j=0; j<n; j++) mx[i][j]/=t;
}
*/

// write footprint file
// 120 byte string (filespec)
// 4 byte integer (n)
// 4 byte integer (n2 elements/image)
// n2 arrays of n, 4 byte floats
out=fopen(argv[2],"wb");
strncpy(s,argv[1],119);
s[119]=0;
fwrite(s,120,1,out);
fwrite(&n,4,1,out);
n2=NUMFP;
fwrite(&n2,4,1,out);
for (i=0; i<NUMFP; i++) fwrite(mx[i],sizeof(float)*n,1,out);
if (euler) {
	for (i=0; i<3; i++) fwrite(eul[i],sizeof(float)*n,1,out);
}
fclose(out);

LOG(Ref,argv[2],LOG_OUTFILE,NULL);
LOG(Ref,NULL,LOG_END,NULL);

// similarity matrix
/*
for (i=0; i<n; i++) {
	for (j=0; j<n; j++) {
		r=0;
		for (k=0; k<16; k++) r+=mx[k][j]*mx[k][i];
		printf("%3.0f",floor(r*49.999)+50.0);
	}
	printf("\n");
}
*/

}
