// Copyright 	Wen Jiang	July 2002

#include "EMData.h"
#include <srfftw_mpi.h>

int main(int argc, char** argv){

int local_nz, local_z_start, local_ny_after_transpose,
    local_y_start_after_transpose, total_local_size;
int x, y, z, nx, ny, nz, mx, my, mz, myrank, newsize=0, dothresh=0;
rfftwnd_mpi_plan plan;
fftw_real *data, *work;
fftw_complex *cdata;
float apix=0, threshold=0;;
char* dprfile=NULL;
EMData *map1, *map2;

MPI_Init(&argc,&argv);
MPI_Comm_rank(MPI_COMM_WORLD,&myrank);

if(argc<4) {
	if (myrank==0) fprintf(stderr,"Usage: %s <input map1> <input map2> <output fsc file> [apix=<A/pixel>] [resize=<n>] [thresh=<number>] [dpr=<file name>]\n", argv[0]);
	MPI_Finalize();
	exit(1);
}
else if(argc>4) {
	for (int i = 4; i < argc; i++) {
		if (strncmp(argv[i], "resize=", 7) == 0) {
			newsize=atoi(&argv[i][7]);
		}
		else if (strncmp(argv[i], "apix=", 5) == 0) {
			apix=atof(&argv[i][5]);
		}
		else if (strncmp(argv[i], "thresh=", 7) == 0) {
			threshold=atof(&argv[i][7]);
			dothresh=1;
		}
		else if (strncmp(argv[i], "dpr=", 4) == 0) {
			dprfile=&argv[i][4];
		}
		else {
			if (myrank==0) fprintf(stderr, "%s: unknown modifier '%s'\n", argv[0], argv[i]);
			MPI_Finalize();
			exit(1);
		}
	}
}


map1 = new EMData;	map2 = new EMData;
map1->readImage(argv[1],-1,1);	// only read the header at this moment
map2->readImage(argv[2],-1,1);

mx=map1->xSize(), my=map1->ySize(), mz=map1->zSize();
if( mx!=map2->xSize() || my!=map2->ySize() || mz!=map2->zSize() ) {
	if (myrank==0) fprintf(stderr,"ERROR: the two input maps %s (%dx%dx%d) and %s (%dx%dx%d) are not equal sized\n",argv[1], mx,my,mz, argv[2], map2->xSize(), map2->ySize(), map2->zSize());
	MPI_Finalize();
	exit(1);
}

if( mx==my && 2*mz-1==mx) { // the special virus 5fold half map
	nx=ny=nz=mx+1;
}
else {	// the normal cubic map
	nx=mx;	ny=my;	nz=mz;
}
if(newsize) {
	nx=ny=nz=newsize;
}

/* create the forward and backward plans: */
plan = rfftw3d_mpi_create_plan(MPI_COMM_WORLD,
							   nz, ny, nx,		// fftw uses C-array convention, but EMAN data uses Fortran-array convention
							   FFTW_REAL_TO_COMPLEX,
							   FFTW_ESTIMATE);
rfftwnd_mpi_local_sizes(plan, &local_nz, &local_z_start,
					   &local_ny_after_transpose,
					   &local_y_start_after_transpose,
					   &total_local_size);

data = (fftw_real*) calloc(1,sizeof(fftw_real) * total_local_size*2);

/* workspace is the same size as the data: */
//work = (fftw_real*) calloc(1,sizeof(fftw_real) * total_local_size*2);
work=NULL;

/* initialize data to f(x,y,z): */
if ( mx==my && 2*mz-1==mx ) { // the special virus 5fold half map
	for (z = 0; z < local_nz; ++z) {
		int z2=z+local_z_start+(2*mz-nz)/2;
		if(z2<=0 || z2>=2*mz) continue;
		if( z2>=mz) {	// upper half is exactly the same as the input map
			map1->readImage(argv[1],z2-mz);	
			map2->readImage(argv[2],z2-mz);
			//printf("read section %d\tz2=%d\tmz=%d\tlocal_nz=%d\tz=%d\n",z2-mz,z2,mz,local_nz,z);
			float *data1=map1->getDataRO();
			float *data2=map2->getDataRO();
			if(dothresh){
				for(int i=0; i<mx*my; i++){
					if(data1[i]>threshold) data1[i]-=threshold;
					else data1[i]=0;
					if(data2[i]>threshold) data2[i]-=threshold;
					else data2[i]=0;
				}
			}
			for (y = 0; y < my; ++y) {
				int y2=y+(my+1-ny)/2-1;
				if(y2<0 || y2>=my) continue;
				for (x = 0; x < mx; ++x) {
					int x2=x+(mx+1-nx)/2-1;
					if(x2<0 || x2>=mx) continue;
					data[2*((z*ny + y +1) * (2*(nx/2+1)) + x + 1)    ] = data1[y2*mx+x2];
					data[2*((z*ny + y +1) * (2*(nx/2+1)) + x + 1) + 1] = data2[y2*mx+x2];
					//printf("(%d,%d,%d)=(%f\t%f)\n",x,y,z,data[2*((z*ny + y) * (2*(nx/2+1)) + x)    ],data[2*((z*ny + y) * (2*(nx/2+1)) + x) + 1]);
				}
			}
			map1->doneData();	map2->doneData();
		}
		else { // lower half is is two fold related to upper half arond y axis => x -> -x, z -> -z
			map1->readImage(argv[1],mz-z2);	
			map2->readImage(argv[2],mz-z2);
			//printf("read section %d\tz2=%d\tmz=%d\tlocal_nz=%d\tz=%d\n",mz-z2,z2,mz,local_nz,z);
			float *data1=map1->getDataRO();
			float *data2=map2->getDataRO();
			if(dothresh){
				for(int i=0; i<mx*my; i++){
					if(data1[i]>threshold) data1[i]-=threshold;
					else data1[i]=0;
					if(data2[i]>threshold) data2[i]-=threshold;
					else data2[i]=0;
				}
			}
			for (y = 0; y < my; ++y) {
				int y2=y+(my+1-ny)/2-1;
				if(y2<0 || y2>=my) continue;
				for (x = 0; x < mx; ++x) {
					int x2=x+(mx+1-nx)/2-1;
					if(x2<0 || x2>=mx) continue;
					data[2*((z*ny + y +1) * (2*(nx/2+1)) + x +1)    ] = data1[y2*mx+mx-x2-1];
					data[2*((z*ny + y +1) * (2*(nx/2+1)) + x +1) + 1] = data2[y2*mx+mx-x2-1];
					//printf("(%d,%d,%d)=(%f\t%f)\n",x,y,z,data[2*((z*ny + y) * (2*(nx/2+1)) + x)    ],data[2*((z*ny + y) * (2*(nx/2+1)) + x) + 1]);
				}
			}
			map1->doneData();	map2->doneData();
		}
	}
}
else {							
	for (z = 0; z < local_nz; ++z) {
		int z2=z+local_z_start+(mz-nz)/2;
		if(z2<0 || z2>=mz) continue;
		map1->readImage(argv[1],z2);	// only read the header at this moment
		map2->readImage(argv[2],z2);
		float *data1=map1->getDataRO();
		float *data2=map2->getDataRO();
		if(dothresh){
			for(int i=0; i<mx*my; i++){
				if(data1[i]>threshold) data1[i]-=threshold;
				else data1[i]=0;
				if(data2[i]>threshold) data2[i]-=threshold;
				else data2[i]=0;
			}
		}
		for (y = 0; y < ny; ++y) {
			int y2=y+(my-ny)/2;
			if(y2<0 || y2>=my) continue;
			for (x = 0; x < nx; ++x) {
				int x2=x+(mx-nx)/2;
				if(x2<0 || x2>=mx) continue;
				data[2*((z*ny + y) * (2*(nx/2+1)) + x)    ] = data1[y2*mx+x2];
				data[2*((z*ny + y) * (2*(nx/2+1)) + x) + 1] = data2[y2*mx+x2];
				//printf("(%d,%d,%d)=(%f\t%f)\n",x,y,z,data[2*((z*ny + y) * (2*(nx/2+1)) + x)    ],data[2*((z*ny + y) * (2*(nx/2+1)) + x) + 1]);
			}
		}
		map1->doneData();	map2->doneData();
	}
}
delete map1;	delete map2;

/* Now, compute the forward transform: */
rfftwnd_mpi(plan, 2, data, work, FFTW_NORMAL_ORDER);	// 2 means 2 data elements per data point 

/* the data is now complex, so typecast a pointer: */
cdata = (fftw_complex*) data;

float* fsc  = (float*) calloc(1,sizeof(float)*nx/2);	// store the fsc for current slab
float* sum1 = (float*) calloc(1,sizeof(float)*nx/2);
float* sum2 = (float*) calloc(1,sizeof(float)*nx/2);

float *dpr, *dsum;
if(dprfile){
	dpr  = (float*) calloc(1,sizeof(float)*nx/2);	// store the fsc for current slab
	dsum = (float*) calloc(1,sizeof(float)*nx/2);
}

for (z = 0; z < local_nz; ++z) {
		int z2 = z+local_z_start;
		if ( z+local_z_start>=nz/2 ) z2=nz-(z+local_z_start);
		for (y = 0; y < ny; ++y) {
				int y2 = y;
				if ( y>=ny/2) y2=ny-y;
				for (x = 0; x < nx/2; ++x) {
						int r=round(sqrt(hypot3(x,y2,z2)));
						if(r>=nx/2) continue;
						else {
							float re1=cdata[2*((z*ny + y) * ((nx/2+1)) + x)  ].re, im1=cdata[2*((z*ny + y) * ((nx/2+1)) + x)  ].im ;
							float re2=cdata[2*((z*ny + y) * ((nx/2+1)) + x)+1].re, im2=cdata[2*((z*ny + y) * ((nx/2+1)) + x)+1].im ;
							fsc[r]  +=  re1*re2+im1*im2;
							sum1[r] +=  re1*re1+im1*im1;
							sum2[r] +=  re2*re2+im2*im2;
							if(dprfile){
								float amp1=sqrt(re1*re1+im1*im1),	amp2=sqrt(re2*re2+im2*im2);
								float phasediff=fmod(fabs((atan2(im1,re1)-atan2(im2,re2)))*180./PI,360.0);
								if(phasediff>=180.0) phasediff=360.0-phasediff;
								phasediff=phasediff*phasediff;
								dpr[r] +=(amp1+amp2)*phasediff;
								dsum[r]+=amp1+amp2;

							}
							//printf("rank=%d\t%d\t%f\t%f\t%f\t%f\t%f\t%f\t%f\n",myrank,r,fsc[r],sum1[r],sum2[r], re1, im1, re2, im2);
						}
				}
		}
}

free(data);
free(work);
rfftwnd_mpi_destroy_plan(plan);

// final collection
float* ffsc  = (float*) calloc(1,sizeof(float)*nx/2);	// store the final reduced fsc from all processes
float* fsum1 = (float*) calloc(1,sizeof(float)*nx/2);
float* fsum2 = (float*) calloc(1,sizeof(float)*nx/2);

MPI_Reduce(fsc, ffsc, ny/2, MPI_FLOAT, MPI_SUM, 0, MPI_COMM_WORLD);
MPI_Reduce(sum1, fsum1, ny/2, MPI_FLOAT, MPI_SUM, 0, MPI_COMM_WORLD);
MPI_Reduce(sum2, fsum2, ny/2, MPI_FLOAT, MPI_SUM, 0, MPI_COMM_WORLD);

float *fdpr, *fdsum;
if(dprfile){
	fdpr  = (float*) calloc(1,sizeof(float)*nx/2);	// store the fsc for current slab
	fdsum = (float*) calloc(1,sizeof(float)*nx/2);

	MPI_Reduce(dpr, fdpr, ny/2, MPI_FLOAT, MPI_SUM, 0, MPI_COMM_WORLD);
	MPI_Reduce(dsum, fdsum, ny/2, MPI_FLOAT, MPI_SUM, 0, MPI_COMM_WORLD);
}

if(myrank==0){
	FILE* fscfile=fopen(argv[3],"w");
	for(x=0;x<nx/2;x++){
		float invx=0, f=0;
		if(apix) invx=x/(nx*apix);
		else	 invx=x;
		if(fsum1[x]*fsum2[x]>0) f=ffsc[x]/sqrt(fsum1[x]*fsum2[x]);
		fprintf(fscfile,"%f\t%f\n",invx,f);
	}
	fclose(fscfile);
	if(dprfile){
		FILE* dprfilefp=fopen(dprfile,"w");
		for(x=0;x<nx/2;x++){
			float invx=0, f=0;
			if(apix) invx=x/(nx*apix);
			else	 invx=x;
			if(fdpr[x]>0) f=sqrt(fdpr[x]/fdsum[x]);
			fprintf(fscfile,"%f\t%f\n",invx,f);
		}
		fclose(dprfilefp);
	}
}

MPI_Finalize();
}
