// iminfo.C  1997  Steve Ludtke
//N iminfo
//F This program provides basic information about image files
//T LM
//1
//P <input file>	File to examine
//P [all]	Give detailed information about each image in a stack file
//P [stat]	Statistical information about the image(s)
//P [first=<n>]	Start with particle number n.
//P [last=<n>]	End with particle number n.
//U iminfo start.img
//U iminfo proj.img all
//D This is a simple program which provides information on any image
//E file supported by EMAN. The 'all' option gives detailed info on each
//E image in a stacked image file.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

//#define RTOD	57.2957795

main(int argc, char *argv[]) {
int i,j,n,nn,t=0,type,euler=0,stat=0,last=-1,first=0;
EMData *d;

appinit(argc,argv);

if (argc<2) appusage(argv[0]);

for (i=2; i<argc; i++) {	
	if (strcmp(argv[i],"all")==0) euler=1;
	else if (strncmp(argv[i],"stat",4)==0) stat=1;
	else if (strncmp(argv[i],"first=",6)==0) first=atoi(argv[i]+6);
	else if (strncmp(argv[i],"last=",5)==0) last=atoi(argv[i]+5);
}

for (i=1; i<argc; i++) {
	if (strncmp(argv[i],"all",3)==0 || strncmp(argv[i],"stat",4)==0 || strncmp(argv[i],"first=",6)==0 || strncmp(argv[i],"last=",5)==0) continue;
	n=fileCount(argv[i],&type);
	t+=n;
	printf("%s contains %d images in ",argv[i],n);
	switch (type) {
	case 0:
		printf("MRC or unknown");
		break;
	case 1:
		printf("Spider");
		break;
	case 2:
		printf("Imagic");
		break;
	case 4:
		printf("PGM/GIF/TIFF");
		break;
	case 5:
	        printf("LST");
		break;
	case 6:
	        printf("PIF");
		break;
	case 7:
	        printf("DM3");
		break;
	case 8:
	        printf("TIFF");
                break;
	case 9:
	       printf("VTK");
	       break;
	case 10:
	       printf("HDF5");
	       break;
	case 11:
	      printf("PNG");
	      break;
	case 12:
	      printf("EM");
	      break;
	case 13:
	      printf("AMIRA");
	      break;
	default:
	        printf("Unknown");	
	}
	printf(" format\n");
}

if (t!=n) printf("%d total images\n",t);

d=new EMData;
if (euler || n==1) {
	if (last==-1 || last>n-1) last=n;
	nn=0;
	for (i=first; i<last; i++) {
		if (stat) d->readImage(argv[1],(i==0&&type==0)?-1:i,0);
		else d->readImage(argv[1],(i==0&&type==0)?-1:i,1);
		if (d->NImg()>0) nn+=d->NImg();
		if (d->zSize()==1) {
			printf("%d. %dx%d\t%1.2f %1.2f %1.2f\t%.2f,%.2f\t%d",i,d->xSize(),d->ySize(),d->alt()*RTOD,d->az()*RTOD,
			d->phi()*RTOD,d->get_center_x(),d->get_center_y(),d->NImg());
			if (stat) printf("\tM=%1.3g S=%1.3g Sk=%1.3g K=%1.3g apix=%g",d->Mean(),d->Sigma(),d->Skewness(),d->Kurtosis(),d->Pixel());
			printf("  %s\n",d->Name());
		}
		else {
			printf("%d. %dx%dx%d\t%1.2f %1.2f %1.2f\t%d",i,d->xSize(),d->ySize(),d->zSize(),d->alt()*RTOD,
			d->az()*RTOD,d->phi()*RTOD,d->NImg());
			if (stat) printf("\tM=%1.3g S=%1.3g Sk=%1.3g K=%1.3g",d->Mean(),d->Sigma(),d->Skewness(),d->Kurtosis());
			printf("  %s\n",d->Name());
		}
	}
	if (nn) printf("\nTotal of %d raw particles represented\n",nn);
}
else {
	d->readImage(argv[1],-1,1);
	printf("Image(s) are %dx%dx%d\n",d->xSize(),d->ySize(),d->zSize());
}

float m=1.0*d->xSize()*d->ySize()*d->zSize()*sizeof(float);
float mm=1.0*d->xSize()*d->ySize()*d->zSize()*sizeof(float)*t;
char *ss = "bytes", *s2 = "bytes";
if (m>1024.0) { ss="kilobytes"; m/=1024.0; }
if (m>1024.0) { ss="megabytes"; m/=1024.0; }
if (mm>1024.0) { s2="kilobytes"; mm/=1024.0; }
if (mm>1024.0) { s2="megabytes"; mm/=1024.0; }

if (n==1) printf("This image will occupy %1.1f %s of memory\n",m,ss);
else printf("Each image will occupy %1.1f %s of memory.\nThe entire set would take %1.1f %s.\n",
	m,ss,mm,s2);

}
