// make3dr.C 3/98 Steve Ludtke
// This program takes a set of projections (classes) with euler angles and constructs
// a 3d model in fourier space using a gaussian interpolation algorithm.
// This 'r' version also varies the euler angles of the individual slices 

// This version has been abandoned. Refinement in fourier space works poorly
// due to interpolation artifacts


#include <stdio.h>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include <math.h>

//#define NITER 8

main(int argc, char *argv[]) {
int mode=5,mask=-1,goodbad=0,pad=0,NITER;
float sigm=3.0,hard=0;
List *data,*dataf,*dataf2;
EMData *d,*d2,*f,*out,*out2;
int i,j,k,g,imc=0,n,ny,ny2;
float norm,pr,thr,r,rec;
float alt,az,phi,alt3,az3,phi3,rstep,range=90.0;
float prm,prs,prn,prb,be[3];
char *file="threed.mrc",*resmap=NULL,*sym="c1";
Euler e,*a;

//appinit(argc,argv);

if (argc<2) { printf("make3dr\nWARNING: this version unsupported\nUsage: make3d <input file> [out=<outfile>] [hard=<max pr>] [range=<max ang diff>]\
 [sym=<c4,icos,etc>] [pad=<padsize>] [mode=<1-6>] [mask=<rad>] [keep=<sigmamult>] [goodbad] [resmap=<name>]\n"); exit(0); }

n=fileCount(argv[1]);
if (n<3) { printf("Input file too small ?!?!\n"); exit(1); }

for (i=2; i<argc; i++) {	
	if (strncmp(argv[i],"out=",4)==0) file=&argv[i][4];
	else if (strncmp(argv[i],"mode=",5)==0) mode=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"hard=",5)==0) hard=atoi(&argv[i][5])*PI/180.0;
	else if (strncmp(argv[i],"mask=",5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"keep=",5)==0) sigm=atof(&argv[i][5]);
	else if (strncmp(argv[i],"goodbad",7)==0) goodbad=1;
	else if (strncmp(argv[i],"resmap=",7)==0) resmap=&argv[i][7];
	else if (strncmp(argv[i],"range=",6)==0) range=atof(&argv[i][6]);
	else if (strncmp(argv[i],"sym=",4)==0) {
		if (tolower(argv[i][4])=='c' || tolower(argv[i][4])=='i') sym=&argv[i][4];
		else { printf("Unsupported symmetry\n"); exit(1); }
	}
	else if (strncmp(argv[i],"pad=",4)==0) pad=atoi(&argv[i][4]);
	else { printf("Unknown argument '%s'\n",argv[i]); exit(1); }
}

if (goodbad) {
	unlink("3drgood.hed");
	unlink("3drbad.hed");
	unlink("3drgood.img");
	unlink("3drbad.img");	
}

dataf=new List;
dataf2=new List;

// Read slices
printf("Read data\n");
data=readImages(argv[1],0,-1);
if (data->count()<3) {
	printf("Need at least 3 images!\n");
	exit(1);
}
d=(EMData *)data->objectAt(0);
ny=d->ySize();
if (pad) ny2=pad; else ny2=ny;

int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);

// prepare slices
printf("Transform slices\n");
for (i=0; i<data->count(); i++) {
	d=(EMData *)data->objectAt(i);
	if (d->NImg()<0) { delete d; continue; }
	if (d->NImg()==0) d->setNImg(1);
	d->normalize();
	d->applyMask(mask,0);
	d->getEuler()->setSym(sym);

	d2=d->clip((ny-ny2)/2,(ny-ny2)/2,ny2,ny2);
	f=d2->doFFT();
	f->toCorner();
	d2->gimmeFFT();
	dataf->addObject(f);
	f->setRAlign(d->alt(),d->az(),d->phi());
	f->setNImg(d->NImg());

//	d2->toCorner();
//	d2->applyMask(0,20);		// not sure if this is good or bad
								// may improve phase residual calc.
//	d2->toCorner();
	f=d2->doFFT();
	d2->gimmeFFT();
	dataf2->addObject(f);
	f->setRAlign(d->alt(),d->az(),d->phi());
	f->setNImg(d->NImg());
	
	delete d2;
	delete d;	
}

out=new EMData;
out->setup4IS(ny2);
out2=NULL;

thr=PI/2.0;
rstep=range*2.0*PI/180.0;
NITER=(int)floor(log(range/.1)/log(2.0));
for (j=0; j<NITER; j++) {
	if (hard) thr=hard*(1.0+(NITER-j-1)/(float)(NITER-1));
	rstep/=2.0;	// 45 deg by the time we get to 1st use
	
	imc=0;
	prm=prs=prn=rec=0;
	for (i=0; i<dataf->count(); i++) {
		g=1;
		d=(EMData *)dataf->objectAt(i);
/*		if (d->NImg()<=0 && j==NITER-1 && goodbad) {
			f=d->doIFT();
			d->gimmeFFT();
			f->toCorner();
			f->writeIMAGIC("3drbad.hed",-1);
			delete f;
		} */
		if (d->NImg()<=0) continue;

//		if (j<4) rstep=10.0*PI/180.0;
//		if (j<6) rstep=3.0*PI/180.0;
//		else if (j<8) { rstep=1.0*PI/180.0; if (mode<5) mode=5; }
//		else { rstep=.25*PI/180.0; mode=6; }

//		if (j==9) mode=5;
		
		if (j!=0) {
			d2=(EMData *)dataf2->objectAt(i);
			alt=d->alt();
			az=d->az();
			phi=d->phi();
			prb=PI;
			for (alt3=alt-rstep*2.0; alt3<=alt+rstep*2.0; alt3+=rstep) {
				for (az3=az-rstep*2.0; az3<=az+rstep*2.0; az3+=rstep) {
					for (phi3=phi-rstep*2.0; phi3<=phi+rstep*2.0; phi3+=rstep) {
						r=out2->normSlice(d2,alt3,az3,phi3,&pr);
						if (pr<prb) { prb=pr; be[0]=alt3; be[1]=az3; be[2]=phi3; norm=r; }
					}
				}
			}
			d->setRAlign(be[0],be[1],be[2]);
			pr=prb;
			if (norm!=0) { d->multConst(1.0/norm); d2->multConst(1.0/norm); }
			prm+=pr;
			prs+=pr*pr;
			prn+=1.0;
			printf("%d/%d  %d\t%5.1f  %5.1f  %5.1f\t%f\t%f ",i,dataf->count(),
				d2->NImg(),(d2->alt()-alt)*180.0/PI,
				(d2->az()-az)*180.0/PI,(d2->phi()-phi)*180.0/PI,
				norm,pr*180.0/PI);
			if (d->alt()!=alt || d->az()!=az || d2->phi()!=phi) rec++;
			if (pr>thr||norm<.01||norm>100.0) { g=0; printf(" X\n"); }
			else printf("\n");
		}
		else printf("%d/%d  %d\t%5.1f  %5.1f  %5.1f\n",i,dataf->count(),d->NImg(),
				d->alt()*180.0/PI,d->az()*180.0/PI,d->phi()*180.0/PI); 

		if (d->NImg()>0 && g) {
			imc+=d->NImg();
/*			for (k=0; k<rsym; k++) {
				out->insertSlice(d,d->alt(),d->az()+k*2.0*PI/(float)rsym,
					d->phi(),mode,j<3?1:d->NImg());
			}*/
			a=d->getEuler();
			for (e=a->FirstSym(); e.valid(); e=a->NextSym()) {
				out->insertSlice(d,e.alt(),e.az(),e.phi(),mode,d->NImg()); 			
			}
			
		}
		if (j==NITER-1 && goodbad) {
			d->toCorner();
			f=d->doIFT();
			d->gimmeFFT();
			if (g) f->writeIMAGIC("3drgood.hed",-1);
			else f->writeIMAGIC("3drbad.hed",-1);
			delete f;
		}
	}
	
	if (prn) {
		prm/=prn;
		prs=sqrt(prs/prn-prm*prm);
		printf("%d. Phase residual Mean=%f   Sigma=%f   used %d images  %1.0f refined\n",j,prm*180.0/PI,prs*180.0/PI,imc,rec);
		thr=prm+prs*sigm;
	}

	if (out2) {
		out2->doneSlice();
		delete out2;
	}
	out2=out;
	out=new EMData;
	out->setup4IS(ny2);
}

if (goodbad) {
	LOG(Ref,"3dgood.hed",LOG_OUTFILE,NULL);
	LOG(Ref,"3dbad.hed",LOG_OUTFILE,NULL);
}

out2->doneSlice();
out2->toCorner();
out=out2->doIFT();

out->applyMask(-1,100+mode);
if (pad) {
	out=out->clip((ny2-ny)/2,(ny2-ny)/2,(ny2-ny)/2,ny,ny,ny);
}
out->applyMask(mask,0);
out->writeMRC(file);

LOG(Ref,file,LOG_OUTFILE,NULL);

if (resmap) {
	float *d1,*d2;
	
	f=out2->Parent();
	out->setSize(ny2,ny2,ny2);
	out->zero();
	d1=f->getDataRO();
	d2=out->getData();
	
	for (i=0; i<ny2; i++) {
		for (j=0; j<ny2; j++) {
			for (k=ny2/2; k<ny2; k++) {
				d2[k+j*ny2+i*ny2*ny2]=sqrt(d1[(k-ny2/2)*2+j*(ny2+2)+i*(ny2+2)*ny2]);
				if (k!=ny2/2 && i!=0 && j!=0) {
					d2[(ny2-k)+(ny2-j)*ny2+(ny2-i)*ny2*ny2]=sqrt(d1[(k-ny2/2)*2+j*(ny2+2)+i*(ny2+2)*ny2]);
				}
			}
		}
	}
	f->doneData();
	out->doneData();
//	out=out->clip((ny2-ny)/2,(ny2-ny)/2,(ny2-ny)/2,ny,ny,ny);
	out->writeMRC(resmap);

	LOG(Ref,resmap,LOG_OUTFILE,NULL);
}
LOG(Ref,NULL,LOG_END,NULL);

}

