// startAny.C  1999  Phil Baldwin

//N startAny
//F This program will generate a 3D model from a set of class averages, using cross common lines.
//T LR
//1
//P <imagefile>	File containing class averages to be used
//P [sym=<c?>]	Cn symmetry if any
//P [proc=<num proc>]	Number of processors to use
//P [rounds=<2-5>]	Rounds of Euler angle determination to use
//P [mask=<rad>]	Mask radius
//P [lp=<radius>]	Lowpass filter radius in Fourier pixels
//U startAny classes.hed sym=c3 mask=42
//D This program uses cross common lines to determine the relative orientations of a set of
//E class averages, then builds a 3d model from them. This will not work well on
//E individual particle images, as they are generally too noisy. Typically 
//E <a href=startnrclasses.html>startnrclasses</a> is used to generate reference free
//E class averages. The user then manually selects 10 or 20 of these class-averages,
//E then runs them through startAny to generate a 3D model.
//D Despite a great deal of time and effort put into this program, it still doesn't
//E work well on some data sets. Usually it will generate results which are
//E adequate for use as a preliminary model for refinement.

#include <stdio.h>
//#include <unistd.h>
#include <math.h>
#include <ctype.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

/****************************************************************/
main(int argc, char *argv[] ) {

appinit(argc,argv);


	if (argc < 2 ) appusage(argv[0]);
	


	int csymnum=1;
	int NumProcs =1;
	int NumRounds =3; 
	char s[1024];
	int mask=-1;
	int filtlp = -1;
	


for (int i=2; i<argc; i++) {
	if (strncmp("sym=c",argv[i],5)==0 || strncmp("sym=C",argv[i],5)==0) {csymnum=atoi(&argv[i][5]);}
	else if (strncmp("proc=",argv[i],5)==0) {NumProcs= atoi(&argv[i][5]);}
	else if (strncmp("rounds=",argv[i],7)==0) {NumRounds= atoi(&argv[i][7]);}
	else if (strncmp("mask=",argv[i],5)==0) {mask= atoi(&argv[i][5]);}
	else if (strncmp("lp=",argv[i],3)==0) {filtlp= atoi(&argv[i][3]);}
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}


int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);



	List *imageList=readImages(argv[1],0,-1);
	int N=imageList->count(); 


//	EMData *IM= (EMData*)imageList->objectAt(0);
//	int partSize = IM -> xSize();

	if (csymnum !=1) {
		sprintf(s,"symsort %s sym=c%d mask=%d",argv[1],csymnum,mask);
		system(s);
	}
	
	sprintf(s,"c1startup %s %d fixed=1",argv[1],10);
//	system(s);


//     Refine angles; Make Model ; Reproject in Direction of Eulers; 
//             Align Originals to Reprojections;   
//	              Make More Reprojections
//                Make A New File using Reprojections and (aligned) Originals
//              Angle the aligned originals
//


//Usage: eulerrefine.parallel <imagefile> 
//        <beginning particle to angle> <ending particle to angle> <ending anchor value>  
//<csymnum><NumProcs><Number of Rounds>[file=<eulers file,if new>]

	int iToAngle, fToAngle, fAnchor;
	int whichRound;
	int iToAngleTemp, fToAngleTemp;

	for (int iRounds=0; iRounds< NumRounds; iRounds++) {
		
		iToAngle=2; fToAngle=N; fAnchor=N; whichRound=iRounds;
		if (iRounds==0) { iToAngle=4; fToAngle=N; 
						 fAnchor=3; }	

		iToAngleTemp = iToAngle;

		while (fToAngle >= iToAngleTemp) {
			fToAngleTemp =fToAngle;
			if (fToAngle> iToAngleTemp+90) fToAngleTemp =iToAngle+90;
			printf("eulerrefine %s iToAngle=%d fToAngle=%d   \
			  fAnchor=%d csymnum=%d NumProcs=%d whichRound=%d \n",  
			  argv[1],iToAngleTemp,fToAngleTemp,fAnchor,csymnum, 
			  				NumProcs,whichRound);// 

			sprintf(s,"eulerrefine40 %s %d %d %d %d %d %d", 
				argv[1],iToAngleTemp,fToAngleTemp,fAnchor,
								csymnum, NumProcs,whichRound);// 
//		sprintf(s,"eulerrefine %s %d %d %d",argv[1],csymnum, NumProcs, 1);// That last														// integer is the number of rounds
			system(s);
			iToAngleTemp += 89;
			}
	}

	
	
	sprintf(s,"make3d %s out=threed.0a.mrc mode=2 hard=30",argv[1]);
	if (csymnum >1) sprintf(s+strlen(s)," sym=c%0d",csymnum);
	system(s);
	

	sprintf(s,"proc3d threed.0a.mrc threed.0a.mrc norm mask=%d",mask);
	if (filtlp != -1 ) sprintf(s+strlen(s)," lp=%d",filtlp);
	system(s);
	
	puts(s);

	
	LOG(Ref,"threed.0a.mrc",LOG_OUTFILE,NULL);
	LOG(Ref,NULL,LOG_END,NULL);

	}
	
