// startcsym.C  12/98  Steve Ludtke
// 12/4/98  based on starticos.c
// This should be able to generate low quality initial models for c? symmetries

//N startcsym
//F This program generates a preliminary 3d model for objects with Cn or Dn symmetry
//T LR
//1
//P <centered particles>	Raw centered particles to use to build a model
//P <# to keep>	Number of particles to use for each view
//P [sym=<c?>]	Symmetry of model
//P [imask=<rad>]	Inside mask used to exclude inside regions
//P [nosym]	Skips the initial symmetry search
//P [fixrot=<side angle>]	Used to correct side view orientation when startcsym makes a mistake
//U startcsym start.hed 50 sym=c7
//D This program uses a very simple technique to generate a VERY rough 3d
//E model from a set of particle images. It seaches for particles with the
//E best n-fold rotational symmetry ('top' view), and for other particles
//E with good mirror symmetry, but poor n-fold rotational symmetry ('side'
//E views). It generates class averages for each of these two views, then
//E generates a 3D model based on them.
//D Specify the number of particles to use for each view. This number should
//E be at least 10-20, and at most ~10% of the total particle data set.
//E 50-100 is good for typical data sets of 2000 or more particles.
//D This routine will not work well for all data sets. Sometimes the relative
//E orientation of the side view will come out wrong, and will need to
//E be manually adjusted with the 'fixrot' parameter. Sometimes it won't
//E work at all, and startAny will have to be used.

#include <stdio.h>
#include "EMData.h"
#include "List.h"
#include "util.h"
#include <math.h>

//#define NITER 8

main(int argc, char *argv[]) {
EMData *imtop,*imside,*ftop,*fside,*cts,*vol,*vol2;
int nx,ny,imask=-1,ccl=0,nosym=0,csym=0;
Euler *a,e;
float phi[4],pr,prt,bpr,bphi,lphi,bnorm,*d,val[181],dot,ang,fixrot=0;
int mode=2,nk,n,i,j,k;
char *sym="icos";
char s[1024];
char *file;

appinit(argc,argv);

if (argc<3) appusage(argv[0]);

nk=atoi(argv[2]);
if (nk<1) { printf("# to keep must be >0\n"); exit(1); }

n=fileCount(argv[1]);
if (n<3) { printf("Input file too small ?!?!\n"); exit(1); }
if (n/3<nk) { printf("<# to keep> should be less than 1/3 number of particles\n"); exit(1); }

for (i=3; i<argc; i++) {	
	if (strncmp(argv[i],"out=",4)==0) file=&argv[i][4];
	else if (strncmp(argv[i],"mode=",5)==0) mode=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"sym=",4)==0) {
		if (tolower(argv[i][4])!='c') exit(1);
		csym=atoi(&argv[i][5]);
	}
	else if (strncmp(argv[i],"imask=",6)==0) imask=atoi(&argv[i][6]);
	else if (strncmp(argv[i],"ccl",3)==0) ccl=1;
	else if (strncmp(argv[i],"nosym",5)==0) nosym=1;
	else if (strncmp(argv[i],"fixrot=",7)==0) fixrot=atof(&argv[i][7]);
	else { printf("Unknown argument '%s'\n",argv[i]); exit(1); }
}

int Ref=LOGbegin(argc,argv);

if (!csym) error(ERR_EXIT,"Must specify symmetry");

if (!nosym) {
	system("/bin/rm -f cls*lst sym.hed sym.img");

// for each symmetry axis (5,3,2 fold), find the best representative
// particles and store them for alignment
	LOG(Ref,argv[1],LOG_INFILE,NULL);
	LOG(Ref,argv[1],LOG_OUTFILE,NULL);
/*	sprintf(s,"symsort %s sym=c%0d imask=%d",argv[1],csym,imask);
	system(s);
	sprintf(s,"proc2d %s cls0000.hed last=%d",argv[1],nk-1);
	system(s);

	sprintf(s,"symsort %s sym=c%0d mirror imask=%d",argv[1],csym,imask);
	system(s);
	sprintf(s,"proc2d %s cls0001.hed last=%d",argv[1],nk-1);
	system(s);*/
	
	sprintf(s,"symbest %s cls0000.lst sym=c%0d mirror=cls0001.lst keep=%d",argv[1],csym,nk-1);
	system(s);

	LOG(Ref,"cls0000.lst",LOG_OUTFILE,NULL);

// Now generate classums for each of the symmetric axes
	sprintf(s,"classalignall median keep=1 12 imask=%d",imask);
	system(s);

// Now we actually enforce the symmetry. This could be considered
// cheating a little, but is fine for preliminary images
	sprintf(s,"proc2d classes.hed sym.hed sym=c%0d first=0 last=0",csym);
	system(s);
	sprintf(s,"proc2d classes.hed sym.hed first=1 last=1");
	system(s);
	LOG(Ref,"sym.hed",LOG_OUTFILE,NULL);

}

// now read in the final symmetric classums from above

LOG(Ref,"sym.hed",LOG_INFILE,NULL);
imtop=new EMData;
if (imtop->readImage("sym.hed",0)) { printf("classes.hed not created, must have been an error.\n"); exit(1); }
 ftop=imtop->doFFT();
 ftop->toCorner();
imtop->gimmeFFT();
 ftop->setRAlign(0,0,0);
 ftop->setNImg(1);

imside=new EMData;
if (imside->readImage("sym.hed",1)) { printf("classes.hed too short, must have been an error.\n"); exit(1); }
imside=imside->copy();

nx=imside->xSize();
ny=imside->ySize();

vol=new EMData;
vol->setup4IS(ny);
vol2=new EMData;
vol2->setup4IS(ny);

/*
printf("Calculate common lines\n");
cts=new EMData;

cts->commonLines(ftop,fside,1);

if (ccl) {
	unlink("ccl.hed");
	unlink("ccl.img");
	cts->writeIMAGIC("ccl.hed",-1);
	LOG(Ref,"ccl.hed",LOG_OUTFILE,NULL);
}

printf("Searching\n");
// 5 -> 3 fold
if (cts->ySize()!=360) exit(1);

d=cts->getDataRO();
phi[0]=0;
phi[1]=0;
pr=1000.0;
for (i=0; i<180; i++) {
	for (j=0; j<360; j++) {
		if (d[i+j*360]<pr) { pr=d[i+j*360]; phi[0]=i; phi[1]=j; }
	}
}
c53->doneData();
printf("5 fold at %f\n",phi[0]);
printf("3 fold at %f\n",phi[1]);

// 5 -> 2 fold
d=c52->getDataRO();
phi[2]=0;
phi[3]=0;
pr=1000.0;
for (i=0; i<180; i++) {
	for (j=0; j<180; j++) {
		if (d[i+j*360]<pr) { pr=d[i+j*360]; phi[2]=i; phi[3]=j; }
	}
}
c52->doneData();
printf("5 fold at %f\n",phi[2]);
printf("2 fold at %f\n",phi[3]);


// OK now build a model

vol->setup4IS(ny);


/////////////////////////////////////
/*
// 3 fold again
bphi=0;
bpr=1.0e5;

for (phi=30.0*PI/180.0; phi<150.0*PI/180.0; phi+=.25*PI/180.0) {
	vol2->normSlice(f3,f3->alt(),f3->az(),phi,&pr);
	if (pr<bpr) { bpr=pr; bphi=phi; }
}

a=f3->getEuler();
a->setAngle(a->alt(),a->az(),bphi);
for (e=a->FirstSym(); e.valid(); e=a->NextSym()) {
	vol->insertSlice(f3,e.alt(),e.az(),e.phi(),mode,1); 			
}
printf("Best 3f pr @ %f (%f)\n",bphi*180.0/PI,bpr);

// 2 fold
bphi=0;
bpr=1.0e5;

for (phi=45.0*PI/180.0; phi<235.0*PI/180.0; phi+=.25*PI/180.0) {
	vol2->normSlice(f2,f2->alt(),f2->az(),phi,&pr);
	if (pr<bpr) { bpr=pr; bphi=phi; }
}

a=f2->getEuler();
a->setAngle(a->alt(),a->az(),bphi);
for (e=a->FirstSym(); e.valid(); e=a->NextSym()) {
	vol->insertSlice(f2,e.alt(),e.az(),e.phi(),mode,1); 			
}
printf("Best 2f pr @ %f (%f)\n",bphi*180.0/PI,bpr);


// 5 fold again
a=f5->getEuler();
a->setAngle(a->alt(),a->az(),-phi[0]);
for (e=a->FirstSym(); e.valid(); e=a->NextSym()) {
	vol->insertSlice(f5,e.alt(),e.az(),e.phi(),mode,1); 			
}
*/

// locate mirror axis, must be z
cts=imside->copy();
cts->vFlip();
cts->setParent(NULL);
imside->rotAlign(cts);

ang=imside->alt();
imside->setRAlign(ang,0,0);
imside->rotateAndTranslate();
dot=imside->dot(cts);
imside->setRAlign(ang+PI,0,0);
imside->rotateAndTranslate();
if (imside->dot(cts)>dot) ang+=PI; 
printf("angle %f -> %f\n",ang,ang/2.0+PI/2.0);
if (fixrot) {
	ang+=fixrot*PI/90.0;
}

imside->setRAlign(ang/2.0+PI/2.0,0,0);
imside->rotateAndTranslate();
delete cts;
imside->writeIMAGIC("sym.hed",2);

 fside=imside->doFFT();
 fside->toCorner();
imside->gimmeFFT();
 fside->setRAlign(PI/2.0,0,0);
 fside->setNImg(1);

// add the symmetric side views
vol->setup4IS(ny);	
for (i=0; i<csym; i++) {
	vol->insertSlice(fside,PI/2.0,(i*2.0*PI/(float)csym),0,mode,1);
}

// now see where the top view fits best
bphi=0;
bpr=1.0e5;
for (lphi=0; lphi<PI/2.0; lphi+=PI/180.0) {
	vol->normSlice(ftop,0,lphi,0,&pr);
printf("%f\t%f\n",lphi*180.0/PI,pr);
	if (pr<bpr) { bpr=pr; bphi=lphi; }
}
printf("Best top pr @ %f (%f)\n",bphi*180.0/PI,bpr);

for (i=0; i<csym; i++) {
	vol->insertSlice(ftop,0,bphi+PI*2.0*i/(float)csym,0,mode,1);
}

vol->doneSlice();
vol->toCorner();
vol2=vol->doIFT();
vol2->applyMask(-1,100+mode);
vol2->applyMask(-1,0);
vol2->writeMRC("threed.0a.mrc");
LOG(Ref,"threed.0a.mrc",LOG_OUTFILE,NULL);
LOG(Ref,NULL,LOG_END,NULL);
}
