// startnrclasses.C 1998  Steve Ludtke

//N startnrclasses
//F This program takes a set of particle images and generates reference free class averages
//T LC
//1
//P <infile>	Input file of raw (centered) particles
//P <# pt per class>	Approximate number of particles to use for each class average
//P [imask=<rad>]	Inside mask radius for data exclusion
//P [mask=<rad>]	Mask radius
//P [parm=<# fp>]	Number of MSA basis vectors to use in classification, default is usually adequate
//P [proc=<n proc>]	Processors to run on
//P [nobad]	Exclude empty classes from the result, losing correlation to cls files
//U startnrclasses start.img 50
//D This program is used to generate reference free class averages. It is fully automated. All
//E you specify is the approximate number of particles to use for each class. Generally this
//E routine will produce at least a few good representative class averages. It may not
//E cover all possible orientations, and there may be many class averages containing almost
//E identical averages. This program is generally used in conjunction with 
//E <a href=startAny.html>startAny</a>. First startnrclasses is used to generate a
//E large number of class averages. The user then manually selects several representing
//E as diverse a range of orientations as possible, and runs them through startAny to
//E generate an initial model for refinement.

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

main(int argc, char *argv[]) {
int i,j,k,n,r2=0;
int mask=-1,imask=-1,ppc=0,fpp=8,proc=0,precen=0,nobad=0;
char s[1024],t[1024];
EMData *im;
List *data;

appinit(argc,argv);

if (argc<3) appusage(argv[0]);

ppc=atoi(argv[2]);
if (ppc<5) { printf("particles per class must be > 4\n"); exit(1); }

for (i=3; i<argc; i++) {	
	if (strncmp(argv[i],"mask=",5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"parm=",5)==0) fpp=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"proc=",5)==0) proc=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"imask=",6)==0) imask=atoi(&argv[i][6]);
	else if (strncmp(argv[i],"nobad",5)==0) nobad=1;
	else if (strncmp(argv[i],"precen",6)==0) precen=1;
	else if (strncmp(argv[i],"round2",6)==0) r2=1;
	else { printf("Unknown option %s\n",argv[i]); exit(1); }
}

int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);
n=fileCount(argv[1]);

// Calculate footprints
strcpy(t,argv[1]);
if (t[strlen(t)-4]=='.') t[strlen(t)-4]=0;
sprintf(s,"footprintr %s %s.fp",argv[1],t);
if (mask!=-1) sprintf(s+strlen(s)," mask=%d",mask);
if (imask!=-1) sprintf(s+strlen(s)," imask=%d",imask);
system(s);

// MSA to reduce dimensionality
sprintf(s,"msa %s.fp %s.msa.fp %d",t,t,fpp);
system(s);

// make classes
sprintf(s,"classes %s.msa.fp %d",t,n/ppc);
system(s);

// align classes
strcpy(s,"classalignall median keep=1 12 nobad");
if (proc) sprintf(s+strlen(s)," proc=%d",proc);
if (precen) strcat(s," precen");
if (imask>0) sprintf(s+strlen(s)," imask=%d",imask);
if (mask>0) sprintf(s+strlen(s)," mask=%d",mask);
system(s);

unlink("classes.nr.hed");
unlink("classes.nr.img");
rename("classes.hed","classes.nr.hed");
rename("classes.img","classes.nr.img");

LOG(Ref,"classes.nr.hed",LOG_OUTFILE,NULL);

if (!r2) {
	for (j=0; j<10000; j++) {
		sprintf(s,"cls%04d.hed",j);
		if (access(s,R_OK)) break;
		sprintf(s,"bcls%04d.hed",j);
		if (access(s,R_OK)) continue;
		unlink(s);
		sprintf(s,"bcls%04d.img",j);
		unlink(s);
	}
}
else {
unlink("nogood.hed");
unlink("nogood.img");

// now add the bad images to the end of start.cen.hed
for (j=0; j<10000; j++) {
	sprintf(s,"cls%04d.hed",j);
	if (access(s,R_OK)) break;
	sprintf(s,"bcls%04d.hed",j);
	if (access(s,R_OK)) continue;
	data=readImages(s,0,-1);
	for (k=0; k<data->count(); k++) {
		im=(EMData *)data->objectAt(k);
		im->writeIMAGIC("nogood.hed",-1);
		delete im;
	}
	delete data;
	unlink(s);
	sprintf(s,"bcls%04d.img",j);
	unlink(s);
}


n=fileCount("nogood.hed");

// now collect 'bad' particles and make some more classes
strcpy(s,"footprintr nogood.hed nogood.fp");
if (mask!=-1) sprintf(s+strlen(s)," mask=%d",mask);
if (imask!=-1) sprintf(s+strlen(s)," imask=%d",imask);
system(s);

// MSA to reduce dimensionality
sprintf(s,"msa nogood.fp nogood.msa.fp %d",fpp);
system(s);

// make classes
sprintf(s,"classes nogood.msa.fp %d",n/ppc);
system(s);

unlink("nogood.hed");
unlink("nogood.img");

// align classes
strcpy(s,"classalignall median keep=1.1 12 nobad");
if (proc) sprintf(s+strlen(s)," proc=%d",proc);
if (precen) strcat(s," precen");
if (imask>0) sprintf(s+strlen(s)," imask=%d",imask);
if (mask>0) sprintf(s+strlen(s)," mask=%d",mask);
system(s);

system("proc2d classes.hed classes.nr.hed");
}

LOG(Ref,NULL,LOG_END,NULL);
}
