// symbest.C Copyright 1998 Steve Ludtke
// 12/98
// This program looks for particles with the given symmetry

//N symbest
//F This program searches a large set of particles for those with the best symmetry.
//T LT
//1
//P <input file>	Input file constining raw particles
//P <output file>	Output file with the selected best particles
//P sym=<Cn>	Symmetry to search for
//P [keep=<N>]	Number of particles to keep
//P [mirror=<mirror out>]	Also search for particles with mirror symmetry
//P [mask=<rad>]	Mask radius
//P [imask=<rad>]	Inside mask radius
//U symbest start.img best.img sym=c4
//D This program searches a set of particles for those with the best rotational and/or
//E mirror symmetry. They should be precentered with <a href=cenalignint.html>cenalignint</a>
//E or a similar utility. This program is generally indirectly through the use of
//E <a href=startcsym.html>startcsym</a> or <a href=starticos.html>starticos</a>.

#include <stdio.h>
#include <math.h>
#include <ctype.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

struct SORT { float val,valm; int n; };

int cmp(const void *d1, const void *d2) {
float c;
c=((SORT *)d1)->val-((SORT *)d2)->val;
if (c<0) return 1;
if (c>0) return -1;
return 0;
}

int cmpm(const void *d1, const void *d2) {
float c;
c=((SORT *)d1)->valm-((SORT *)d2)->valm;
if (c<0) return 1;
if (c>0) return -1;
return 0;
}

main(int argc, char *argv[]) {
int csym=0,n,imask=-1,mask=-1,keep=25;
int i,rfp=0;
float lc,norm=1.0,t;
char *mirror=NULL;
List *data;
SORT *sort;
EMData *tmp,*msk,*d1,*d2;

appinit(argc,argv);

if (argc<2) appusage(argv[0]);

for (i=3; i<argc; i++) {	
	if (strncmp(argv[i],"sym=",4)==0) {
		if (tolower(argv[i][4])!='c' && tolower(argv[i][4])!='m') { printf("Only C/M symmetry currently supported!\n"); exit(1); }
		if (tolower(argv[i][4])=='m') csym=-1;
		else csym=atoi(&argv[i][5]);
	}
	else if (strncmp(argv[i],"mirror=",7)==0) mirror=argv[i]+7;
	else if (strncmp(argv[i],"rfp",6)==0) rfp=1;
	else if (strncmp(argv[i],"keep=",5)==0) keep=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"mask=",5)==0) mask=atoi(&argv[i][5]);
	else if (strncmp(argv[i],"imask=",6)==0) imask=atoi(&argv[i][6]);
	else { printf("Unknown argument '%s'\n",argv[i]); exit(1); }
}

if (csym<=1) { printf("Invalid symmetry\n"); exit(1); }

int Ref=LOGbegin(argc,argv);
LOG(Ref,argv[1],LOG_INFILE,NULL);

//data=readImages(argv[1],-1,-1);
//n=data->count();
n=fileCount(argv[1]);

if (n<2) { printf("Not enough images to sort!\n"); exit(0); }
sort=(SORT *)malloc(n*sizeof(SORT));

d1=new EMData;
d1->readImage(argv[1],0);
if (rfp) d2=d1->makeRFP();
else d2=d1;

msk=d2->copy();
msk->setParent(NULL);
msk->one();
msk->applyMask(mask,4);
msk->applyMask(imask,5);
//norm=msk->dot(msk);

// do the symmetry comparisons
for (i=0; i<n; i++) {
	d1->readImage(argv[1],i);
	sort[i].n=i;
	d1->normalize();
	if (rfp) d2=d1->makeRFP();
	else d2=d1;
//	d1->radialSubtract();
	tmp=d2->copy();
//	tmp=d1->copy();
	sort[i].val=0;
	for (float f=2.0*PI/(float)csym; f<(2.0-1.0/(float)csym)*PI; f+=2.0*PI/(float)csym) {
		tmp->setRAlign(f,0,0);
		tmp->rotateAndTranslate();
		tmp->mult(msk);
//		sort[i].val+=tmp->dot(d1);
//		t=tmp->dot(d2);
		t=2.0-tmp->lcmp(d2);
		sort[i].val+=t;
//		if (t<sort[i].val) sort[i].val=t;
	}
	if (mirror) {
		tmp->setParent(NULL);
		tmp->vFlip();
		tmp->rotAlign(d2);
		tmp->rotateAndTranslate();
		sort[i].valm=-sort[i].val+(2.0-tmp->lcmp(d2));
		printf("%d. %f\t%f\n",i,sort[i].val/(float)(csym-1),sort[i].valm/(float)(csym-1));
	}
	else printf("%d. %f\n",i,sort[i].val/((float)(csym-1)));
	delete tmp;
}

if (mirror) {
	qsort(sort,n,sizeof(SORT),cmpm);
	LOG(Ref,mirror,LOG_OUTFILE,NULL);
	for (i=0; i<keep; i++) {
		char s[40];
		sprintf(s,"%1.1f",sort[i].valm);
//		d1->readImage(argv[1],sort[i].n);
//		d1->writeIMAGIC(mirror,-1);
		d1->writeLST(mirror,-1,argv[1],sort[i].n,s);
	}
}

qsort(sort,n,sizeof(SORT),cmp);

LOG(Ref,argv[2],LOG_OUTFILE,NULL);

for (i=0; i<keep; i++) {
	char s[40];
	sprintf(s,"%1.1f",sort[i].val);
//	d1->readImage(argv[1],sort[i].n);
//	d1->writeIMAGIC(argv[2],i);
	d1->writeLST(argv[2],-1,argv[1],sort[i].n,s);
}
LOG(Ref,NULL,LOG_END,NULL);

}
