// this is a version of footprint2 which also outputs an ascii file

#include <stdio.h>
#include <math.h>
#include "EMData.h"
#include "List.h"
#include "util.h"

main(int argc, char *argv[]) {
int i,j,k,n,n2,x,y,nx,ny;
float t,r,w,sc,c[32];
EMData *d,*e,*ref[2][32];
float *data;
char s[120];
float f,*mx[32];
float *alt,*az;
FILE *out;

if (argc!=3) { printf("footprint.c\nUsage:  <input file> <output file>\n"); exit(0); }

dbread();
if (strlen(argv[1])>119) printf("WARNING: Path over 120 char will cause problems later.\n");

n=fileCount(argv[1]);
if (n<1) { printf("No data\n"); exit(0); }

d=new EMData;
d->readImage(argv[1],0);
nx=d->xSize()*2+2;
ny=d->ySize()*2;

if (nx-2!=ny) { printf("nx!=ny\n"); exit(1); }

//unlink("foot.hed");
//unlink("foot.img");

printf("Preparing references\n");

w=ny/128.0;		// barely overlapping gaussians
w*=w;
for (i=0; i<8; i++) {
	for (j=0; j<4; j++) {
		ref[0][i+j*8]=new EMData;
		ref[0][i+j*8]->setSize(nx,ny,1);
		ref[0][i+j*8]->setComplex(1);
		ref[0][i+j*8]->setRI(1);
		data=ref[0][i+j*8]->getData();
		sc=(i+1)*ny/64.0;
		// sc is the radius of the center of each band
		// ny/10.0 normalization is arbitrary
		// factor of 3 for i=1 is because very low freq. may be noisy
		for (x=0; x<nx; x+=2) {
			for (y=0; y<ny; y++) {
				t=atan2((float)(y-ny/2),(float)(x/2));
				r=hypot((float)(y-ny/2),(float)(x/2))-sc;
				data[x+y*nx]=ny/10.0*sin(t*2*j)*exp(-r*r/w)/sc;
				if (i==0) data[x+y*nx]/=3.0;
				data[x+y*nx+1]=0;
			}
		}
		ref[0][i+j*8]->doneData();
		ref[0][i+j*8]->update();
//		ref[0][i+j*8]->normalize();		

		ref[1][i+j*8]=new EMData;
		ref[1][i+j*8]->setSize(nx,ny,1);
		ref[1][i+j*8]->setComplex(1);
		ref[1][i+j*8]->setRI(1);
		data=ref[1][i+j*8]->getData();
		sc=(i+1)*ny/64.0;
		for (x=0; x<nx; x+=2) {
			for (y=0; y<ny; y++) {
				t=atan2((float)(y-ny/2),(float)(x/2));
				r=hypot((float)(y-ny/2),(float)(x/2))-sc;
				data[x+y*nx]=ny/10.0*cos(t*2*j)*exp(-r*r/w)/sc;
				if (i==0) data[x+y*nx]/=3.0;
				data[x+y*nx+1]=0;
			}
		}
		ref[1][i+j*8]->doneData();
		ref[1][i+j*8]->update();
//		ref[1][i+j*8]->normalize();
//sprintf(s,"foot.%02d.mrc",i+j*8);
//ref[1][i+j*8]->writeMRC(s);

	}
}

for (i=0; i<32; i++) { mx[i]=(float*)malloc(sizeof(float)*n); c[i]=0; }

alt=(float *)malloc(sizeof(float)*n);
az=(float *)malloc(sizeof(float)*n);

printf("Calculate dot products\n");
// calculate image dot products
for (i=0; i<n; i++) {
	if (d->readImage(argv[1],i)) printf("Image read failed at %d\n",i);
	d->normalize();
	e=d->clip(-ny/4,-ny/4,ny,ny);
	alt[i]=d->Dax();
	az[i]=d->Daz2();
	delete d;
	d=e;
	d->toCorner();
	e=d->doFFT();
	e->ri2ap();
	for (j=0; j<32; j++) {
		mx[j][i]=hypot(e->dot(ref[0][j]),e->dot(ref[1][j]));
		c[j]+=mx[j][i];
	}
	if (i%50==0) { printf(" %d done\r",i); fflush(stdout); }
}
printf("%d done\n",n);


for (i=0; i<32; i++) c[i]/=(float)n;	// this is the center of the 32 dim object

// translate each to center and normalize
for (i=0; i<n; i++) {
//	printf("\n%4d  ",i);
	r=0;
	for (j=0; j<32; j++) { mx[j][i]-=c[j]; r+=mx[j][i]*mx[j][i]; }
	r=sqrt(r);
	for (j=0; j<32; j++) { 
		mx[j][i]/=r; 
//		printf("%3.0f",floor(mx[j][i]*99.999)); 
//		printf("%6.2f ",mx[j][i]*10.0);
	}
//	printf("\n");
}

out=fopen("tfp","w");
for (j=0; j<n; j++) {
	fprintf(out,"%f\t%f\t",alt[j],az[j]);
	for (i=0; i<32; i++) {
		fprintf(out,"%f\t",mx[i][j]);
	}
	fprintf(out,"\n");
}
fclose(out);

// write footprint file
// 120 byte string (filespec)
// 4 byte integer (n)
// 4 byte integer (n2 elements/image)
// n2 arrays of n, 4 byte floats

out=fopen(argv[2],"w");
strncpy(s,argv[1],119);
s[119]=0;
fwrite(s,120,1,out);
fwrite(&n,4,1,out);
n2=32;
fwrite(&n2,4,1,out);
for (i=0; i<32; i++) fwrite(mx[i],sizeof(float)*n,1,out);
fclose(out);

// similarity matrix
/*
for (i=0; i<n; i++) {
	for (j=0; j<n; j++) {
		r=0;
		for (k=0; k<16; k++) r+=mx[k][j]*mx[k][i];
		printf("%3.0f",floor(r*49.999)+50.0);
	}
	printf("\n");
}
*/

}
