//
//	UnixToWin.h	
//	Christopher Booth, June 2002
//	
//	Assorted definitions, preprocessor directives and include statements to make EMAN build
//	using the Windows API	
//
//	Note:  If you are building a new workspace or project be sure to include the compiler flag /TP to force the 
//	compiler to assume all files are written in C++ otherwise ".C" files are assumed to be written in C and it 
//	won't even come close to compiling.

//
//	To link to libEM.lib you will also need to find/build/link to the following
//	FFTW								fftw.lib rfftw.lib 
//	TIFF								libtiff.lib 
//	Windows Stuff (already provided)	Ws2_32.lib 
//
//	Another thing that seems to be important is that if you are linking the libEM.lib you need to ensure
//	that you are building using the Single Threaded library (/ML and /MLd) because that is what libEM
//	uses when it is built.


#ifndef __UNIX_TO_WINDOWS__
#define __UNIX_TO_WINDOWS__




#include <windows.h>
#include <winbase.h> 
#include <io.h>
#include <float.h>
#include <direct.h>
#include <process.h>
#include <fcntl.h>					// defines _O_BINARY_


#define sleep(time) Sleep(time/1000)	// sleep takes time in seconds (Unix) 
#define usleep(time) Sleep(time*1000)	// usleep takes time in micro-seconds (Unix)
										// Sleep takes time in milli-seconds (Windows)


// Macros to convert the UNIX command to the corresponding Windows command
#define random rand			
#define srandom srand
#define access _access
#define popen _popen
#define pclose _pclose
#define finite _finite
#define getcwd _getcwd
#define getpid _getpid
#define getppid _getpid		// there seems to be no corresponding get parent process id function for windows
#define mkdir(a,b) _mkdir(a)


#define R_OK 04
#define W_OK 02
#define F_OK 00


#pragma warning(disable:4101)   // unreferenced local variable
#pragma warning(disable:4244)	// 'return' : conversion from 'double' to 'float', possible loss of data
#pragma warning(disable:4305)	// initializing' : truncation from 'const double' to 'float'
#pragma warning(disable:4786)   // identifier was truncated to '255' characters in the debug information



//
// This includes some of the functions 
// that weren't provided in the windows api
//


float copysign(float a,float b);	// makes a have the same sign as b
double erfc(double x);				// returns complementary error function, Numerical Recipes 2nd ed. on p. 214



// ifndef'd out the code in the following functions: 
//
//From util.c
// int file_lock_wait(FILE *file);			// now returns -1 to indicate nothing happened
// int file_lock(FILE *file);				// now returns -1 to indicate nothing happened
// void file_unlock(FILE *file);
// void benchmark_start(int tag);
// void benchmark_stop(int tag);
// void benchmark_report();
// int run(char *s);						// now returns -1`to indicate nothing happened
//


#endif // __UNIX_TO_WINDOWS__
